Package: recount
Title: Explore and download data from the recount project
Version: 1.37.0
Date: 2024-05-21
Authors@R: c(person("Leonardo", "Collado-Torres", role = c("aut", "cre"),
    email = "lcolladotor@gmail.com", comment = c(ORCID = "0000-0003-2140-308X")),
    person("Abhinav", "Nellore", role = "ctb",
    email = "anellore@gmail.com"), person(c("Andrew", "E."), "Jaffe",
    role = "ctb", email = "andrew.jaffe@libd.org",
    comment = c(ORCID = "0000-0001-6886-1454")), person(c("Margaret", "A."),
    "Taub", role = "ctb", email = "mtaub@jhsph.edu"), person("Kai", "Kammers",
    role = "ctb", email = "kai.kammers@gmail.com"), person(c("Shannon", "E."),
    "Ellis", role = "ctb", email = "sellis18@jhmi.edu",
    comment = c(ORCID = "0000-0002-9231-0481")), person(c("Kasper",
    "Daniel"), "Hansen", role = c("ctb"),
    email = "kasperdanielhansen@gmail.com",
    comment = c(ORCID = "0000-0003-0086-0687")), person("Ben", "Langmead",
    role = "ctb", email = 'langmea@cs.jhu.edu',
    comment = c(ORCID = "0000-0003-2437-1976")), person(c("Jeffrey", "T."),
    "Leek", role = c("aut", "ths"), email = "jtleek@gmail.com",
    comment = c(ORCID = "0000-0002-2873-2671")))
Depends: R (>= 3.5.0), SummarizedExperiment
Imports: BiocParallel, derfinder, downloader, GEOquery, GenomeInfoDb,
        GenomicRanges, IRanges, methods, RCurl, rentrez, rtracklayer
        (>= 1.35.3), S4Vectors, stats, utils
Suggests: AnnotationDbi, BiocManager, BiocStyle (>= 2.5.19), DESeq2,
        sessioninfo, EnsDb.Hsapiens.v79, GenomicFeatures, txdbmaker,
        knitr (>= 1.6), org.Hs.eg.db, RefManageR, regionReport (>=
        1.9.4), rmarkdown (>= 0.9.5), testthat (>= 2.1.0), covr,
        pheatmap, DT, edgeR, ggplot2, RColorBrewer
VignetteBuilder: knitr
Description: Explore and download data from the recount project available at
    https://jhubiostatistics.shinyapps.io/recount/. Using the recount package you can
    download RangedSummarizedExperiment objects at the gene, exon or exon-exon junctions level,
    the raw counts, the phenotype metadata used, the urls to the sample coverage
    bigWig files or the mean coverage bigWig file for a particular study. The
    RangedSummarizedExperiment objects can be used by different packages for
    performing differential expression analysis. Using
    http://bioconductor.org/packages/derfinder you can perform
    annotation-agnostic differential expression analyses with the data from the
    recount project as described at http://www.nature.com/nbt/journal/v35/n4/full/nbt.3838.html.
License: Artistic-2.0
Encoding: UTF-8
LazyData: true
LazyDataCompression: xz
URL: https://github.com/leekgroup/recount
BugReports: https://support.bioconductor.org/t/recount/
biocViews: Coverage, DifferentialExpression, GeneExpression, RNASeq,
        Sequencing, Software, DataImport, ImmunoOncology
RoxygenNote: 7.3.1
Roxygen: list(markdown = TRUE)
git_url: https://git.bioconductor.org/packages/recount
git_branch: devel
git_last_commit: 0f75cbd
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-11-04
NeedsCompilation: no
Packaged: 2025-11-05 00:35:32 UTC; biocbuild
Author: Leonardo Collado-Torres [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2140-308X>),
  Abhinav Nellore [ctb],
  Andrew E. Jaffe [ctb] (ORCID: <https://orcid.org/0000-0001-6886-1454>),
  Margaret A. Taub [ctb],
  Kai Kammers [ctb],
  Shannon E. Ellis [ctb] (ORCID: <https://orcid.org/0000-0002-9231-0481>),
  Kasper Daniel Hansen [ctb] (ORCID:
    <https://orcid.org/0000-0003-0086-0687>),
  Ben Langmead [ctb] (ORCID: <https://orcid.org/0000-0003-2437-1976>),
  Jeffrey T. Leek [aut, ths] (ORCID:
    <https://orcid.org/0000-0002-2873-2671>)
Maintainer: Leonardo Collado-Torres <lcolladotor@gmail.com>

# Generated by roxygen2: do not edit by hand

export(abstract_search)
export(add_metadata)
export(add_predictions)
export(all_metadata)
export(browse_study)
export(coverage_matrix)
export(download_retry)
export(download_study)
export(expressed_regions)
export(find_geo)
export(geo_characteristics)
export(geo_info)
export(getRPKM)
export(getTPM)
export(read_counts)
export(reproduce_ranges)
export(scale_counts)
export(snaptron_query)
import(BiocParallel)
import(GEOquery)
import(GenomeInfoDb)
import(GenomicRanges)
import(IRanges)
import(RCurl)
import(S4Vectors)
import(SummarizedExperiment)
import(derfinder)
import(downloader)
import(rentrez)
importFrom(RCurl,url.exists)
importFrom(methods,is)
importFrom(stats,runif)
importFrom(utils,browseURL)
importFrom(utils,read.table)
importMethodsFrom(rtracklayer,import)
importMethodsFrom(rtracklayer,import.bw)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# recount <img src="man/figures/logo.png" align="right" width="400px" />

<!-- badges: start -->

[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![Bioc release
status](http://www.bioconductor.org/shields/build/release/bioc/recount.svg)](https://bioconductor.org/checkResults/release/bioc-LATEST/recount)
[![Bioc devel
status](http://www.bioconductor.org/shields/build/devel/bioc/recount.svg)](https://bioconductor.org/checkResults/devel/bioc-LATEST/recount)
[![Bioc downloads
rank](https://bioconductor.org/shields/downloads/release/recount.svg)](http://bioconductor.org/packages/stats/bioc/recount/)
[![Bioc
support](https://bioconductor.org/shields/posts/recount.svg)](https://support.bioconductor.org/tag/recount)
[![Bioc
history](https://bioconductor.org/shields/years-in-bioc/recount.svg)](https://bioconductor.org/packages/release/bioc/html/recount.html#since)
[![Bioc last
commit](https://bioconductor.org/shields/lastcommit/devel/bioc/recount.svg)](http://bioconductor.org/checkResults/devel/bioc-LATEST/recount/)
[![Bioc
dependencies](https://bioconductor.org/shields/dependencies/release/recount.svg)](https://bioconductor.org/packages/release/bioc/html/recount.html#since)
[![Codecov test
coverage](https://codecov.io/gh/leekgroup/recount/branch/devel/graph/badge.svg)](https://codecov.io/gh/leekgroup/recount?branch=devel)
[![R build
status](https://github.com/leekgroup/recount/actions/workflows/check-bioc.yml/badge.svg)](https://github.com/leekgroup/recount/actions/workflows/check-bioc.yml)
[![GitHub
issues](https://img.shields.io/github/issues/leekgroup/recount)](https://github.com/leekgroup/recount/issues)
[![GitHub
pulls](https://img.shields.io/github/issues-pr/leekgroup/recount)](https://github.com/leekgroup/recount/pulls)
<!-- badges: end -->

Explore and download data from the recount project available at the
[recount2 website](https://jhubiostatistics.shinyapps.io/recount/).
Using the `recount` package you can download
*RangedSummarizedExperiment* objects at the gene, exon or exon-exon
junctions level, the raw counts, the phenotype metadata used, the urls
to the sample coverage bigWig files or the mean coverage bigWig file for
a particular study. The *RangedSummarizedExperiment* objects can be used
by different packages for performing differential expression analysis.
Using [derfinder](http://bioconductor.org/packages/derfinder) you can
perform annotation-agnostic differential expression analyses with the
data from the recount project.

## Documentation

For more information about `recount` check the vignettes [through
Bioconductor](http://bioconductor.org/packages/recount) or at the
[documentation website](http://leekgroup.github.io/recount).

## Installation instructions

Get the latest stable `R` release from
[CRAN](http://cran.r-project.org/). Then install `recount` from
[Bioconductor](http://bioconductor.org/) using the following code:

``` r
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("recount")
```

## Citation

Below is the citation output from using `citation('recount')` in R.
Please run this yourself to check for any updates on how to cite
**recount**.

``` r
print(citation("recount"), bibtex = TRUE)
#> To cite package 'recount' in publications use:
#> 
#>   Collado-Torres L, Nellore A, Kammers K, Ellis SE, Taub MA, Hansen KD,
#>   Jaffe AE, Langmead B, Leek JT (2017). "Reproducible RNA-seq analysis
#>   using recount2." _Nature Biotechnology_. doi:10.1038/nbt.3838
#>   <https://doi.org/10.1038/nbt.3838>,
#>   <http://www.nature.com/nbt/journal/v35/n4/full/nbt.3838.html>.
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Article{,
#>     title = {Reproducible RNA-seq analysis using recount2},
#>     author = {Leonardo Collado-Torres and Abhinav Nellore and Kai Kammers and Shannon E. Ellis and Margaret A. Taub and Kasper D. Hansen and Andrew E. Jaffe and Ben Langmead and Jeffrey T. Leek},
#>     year = {2017},
#>     journal = {Nature Biotechnology},
#>     doi = {10.1038/nbt.3838},
#>     url = {http://www.nature.com/nbt/journal/v35/n4/full/nbt.3838.html},
#>   }
#> 
#>   Collado-Torres L, Nellore A, Jaffe AE (2017). "recount workflow:
#>   Accessing over 70,000 human RNA-seq samples with Bioconductor
#>   [version 1; referees: 1 approved, 2 approved with reservations]."
#>   _F1000Research_. doi:10.12688/f1000research.12223.1
#>   <https://doi.org/10.12688/f1000research.12223.1>,
#>   <https://f1000research.com/articles/6-1558/v1>.
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Article{,
#>     title = {recount workflow: Accessing over 70,000 human RNA-seq samples with Bioconductor [version 1; referees: 1 approved, 2 approved with reservations]},
#>     author = {Leonardo Collado-Torres and Abhinav Nellore and Andrew E. Jaffe},
#>     year = {2017},
#>     journal = {F1000Research},
#>     doi = {10.12688/f1000research.12223.1},
#>     url = {https://f1000research.com/articles/6-1558/v1},
#>   }
#> 
#>   Ellis SE, Collado-Torres L, Jaffe AE, Leek JT (2018). "Improving the
#>   value of public RNA-seq expression data by phenotype prediction."
#>   _Nucl. Acids Res._. doi:10.1093/nar/gky102
#>   <https://doi.org/10.1093/nar/gky102>,
#>   <https://doi.org/10.1093/nar/gky102>.
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Article{,
#>     title = {Improving the value of public RNA-seq expression data by phenotype prediction},
#>     author = {Shannon E. Ellis and Leonardo Collado-Torres and Andrew E. Jaffe and Jeffrey T. Leek},
#>     year = {2018},
#>     journal = {Nucl. Acids Res.},
#>     doi = {10.1093/nar/gky102},
#>     url = {https://doi.org/10.1093/nar/gky102},
#>   }
#> 
#>   Collado-Torres L, Nellore A, Kammers K, Ellis SE, Taub MA, Hansen KD,
#>   Jaffe AE, Langmead B, Leek JT (2023). _Explore and download data from
#>   the recount project_. doi:10.18129/B9.bioc.recount
#>   <https://doi.org/10.18129/B9.bioc.recount>,
#>   https://github.com/leekgroup/recount - R package version 1.27.0,
#>   <http://www.bioconductor.org/packages/recount>.
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Manual{,
#>     title = {Explore and download data from the recount project},
#>     author = {Leonardo Collado-Torres and Abhinav Nellore and Kai Kammers and Shannon E. Ellis and Margaret A. Taub and Kasper D. Hansen and Andrew E. Jaffe and Ben Langmead and Jeffrey T. Leek},
#>     year = {2023},
#>     url = {http://www.bioconductor.org/packages/recount},
#>     note = {https://github.com/leekgroup/recount - R package version 1.27.0},
#>     doi = {10.18129/B9.bioc.recount},
#>   }
#> 
#>   Frazee AC, Langmead B, Leek JT (2011). "ReCount: A multi-experiment
#>   resource of analysis-ready RNA-seq gene count datasets." _BMC
#>   Bioinformatics_. doi:10.1186/1471-2105-12-449
#>   <https://doi.org/10.1186/1471-2105-12-449>,
#>   <https://doi.org/10.1186/1471-2105-12-449>.
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Article{,
#>     title = {ReCount: A multi-experiment resource of analysis-ready RNA-seq gene count datasets},
#>     author = {Alyssa C. Frazee and Ben Langmead and Jeffrey T. Leek},
#>     year = {2011},
#>     journal = {BMC Bioinformatics},
#>     doi = {10.1186/1471-2105-12-449},
#>     url = {https://doi.org/10.1186/1471-2105-12-449},
#>   }
#> 
#>   Razmara A, Ellis SE, Sokolowski DJ, Davis S, Wilson MD, Leek JT,
#>   Jaffe AE, Collado-Torres L (2019). "recount-brain: a curated
#>   repository of human brain RNA-seq datasets metadata." _bioRxiv_.
#>   doi:10.1101/618025 <https://doi.org/10.1101/618025>,
#>   <https://doi.org/10.1101/618025>.
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Article{,
#>     title = {recount-brain: a curated repository of human brain RNA-seq datasets metadata},
#>     author = {Ashkaun Razmara and Shannon E. Ellis and Dustin J. Sokolowski and Sean Davis and Michael D. Wilson and Jeffrey T. Leek and Andrew E. Jaffe and Leonardo Collado-Torres},
#>     year = {2019},
#>     journal = {bioRxiv},
#>     doi = {10.1101/618025},
#>     url = {https://doi.org/10.1101/618025},
#>   }
#> 
#>   Imada E, Sanchez DF, Collado-Torres L, Wilks C, Matam T, Dinalankara
#>   W, Stupnikov A, Lobo-Pereira F, Yip C, Yasuzawa K, Kondo N, Itoh M,
#>   Suzuki H, Kasukawa T, Hon CC, de Hoon MJ, Shin JW, Carninci P, Jaffe
#>   AE, Leek JT, Favorov A, Franco GR, Langmead B, Marchionni L (2020).
#>   "Recounting the FANTOM CAGE–Associated Transcriptome." _Genome
#>   Research_. doi:10.1101/gr.254656.119
#>   <https://doi.org/10.1101/gr.254656.119>,
#>   <https://doi.org/10.1101/gr.254656.119>.
#> 
#> A BibTeX entry for LaTeX users is
#> 
#>   @Article{,
#>     title = {Recounting the FANTOM CAGE–Associated Transcriptome},
#>     author = {Eddie-Luidy Imada and Diego Fernando Sanchez and Leonardo Collado-Torres and Christopher Wilks and Tejasvi Matam and Wikum Dinalankara and Aleksey Stupnikov and Francisco Lobo-Pereira and Chi-Wai Yip and Kayoko Yasuzawa and Naoto Kondo and Masayoshi Itoh and Harukazu Suzuki and Takeya Kasukawa and Chung Chau Hon and Michiel JL {de Hoon} and Jay W Shin and Piero Carninci and Andrew E. Jaffe and Jeffrey T. Leek and Alexander Favorov and Glória R Franco and Ben Langmead and Luigi Marchionni},
#>     year = {2020},
#>     journal = {Genome Research},
#>     doi = {10.1101/gr.254656.119},
#>     url = {https://doi.org/10.1101/gr.254656.119},
#>   }
```

Please note that the `recount` was only made possible thanks to many
other R and bioinformatics software authors, which are cited either in
the vignettes and/or the paper(s) describing this package.

## Code of Conduct

Please note that the recount project is released with a [Contributor
Code of
Conduct](https://contributor-covenant.org/version/2/0/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

## Development tools

- Continuous code testing is possible thanks to [GitHub
  actions](https://www.tidyverse.org/blog/2020/04/usethis-1-6-0/)
  through *[usethis](https://CRAN.R-project.org/package=usethis)*,
  *[remotes](https://CRAN.R-project.org/package=remotes)*,
  *[sysreqs](https://github.com/r-hub/sysreqs)* and
  *[rcmdcheck](https://CRAN.R-project.org/package=rcmdcheck)* customized
  to use [Bioconductor’s docker
  containers](https://www.bioconductor.org/help/docker/) and
  *[BiocCheck](https://bioconductor.org/packages/3.17/BiocCheck)*.
- Code coverage assessment is possible thanks to
  [codecov](https://codecov.io/gh) and
  *[covr](https://CRAN.R-project.org/package=covr)*.
- The [documentation website](http://leekgroup.github.io/recount) is
  automatically updated thanks to
  *[pkgdown](https://CRAN.R-project.org/package=pkgdown)*.
- The code is styled automatically thanks to
  *[styler](https://CRAN.R-project.org/package=styler)*.
- The documentation is formatted thanks to
  *[devtools](https://CRAN.R-project.org/package=devtools)* and
  *[roxygen2](https://CRAN.R-project.org/package=roxygen2)*.

For more details, check the `dev` directory.

This package was developed using
*[biocthis](https://bioconductor.org/packages/3.17/biocthis)*.

## Teams involved

- [Jeff Leek’s lab at JHBSPH Biostatistics
  Department](http://jtleek.com/),
- [Ben Langmead’s lab at JHU Computer
  Science](http://www.langmead-lab.org/),
- [Kasper Daniel Hansen’s lab at JHBSPH Biostatistics
  Department](https://www.hansenlab.org/),
- [Leonardo Collado-Torres](http://lcolladotor.github.io/) and
  [Andrew E. Jaffe](http://aejaffe.com/) from
  [LIBD](https://www.libd.org/),
- [Abhinav Nellore’s lab at OHSU](http://nellore.bio/),
- Data hosted by [SciServer at JHU](https://www.sciserver.org/).

|                                                                                                                                                                               |                                                                                                              |                                                                                                                                                                         |                                                                                                                                                   |
|-------------------------------------------------------------------------------------------------------------------------------------------------------------------------------|--------------------------------------------------------------------------------------------------------------|-------------------------------------------------------------------------------------------------------------------------------------------------------------------------|---------------------------------------------------------------------------------------------------------------------------------------------------|
| <a href="http://www.langmead-lab.org/"><img src="http://www.langmead-lab.org/wp-content/uploads/2014/01/Screen-Shot-2014-02-02-at-5.20.13-PM-1024x199.png" width="250px"></a> | <a href="https://www.libd.org/"><img src="http://lcolladotor.github.io/img/LIBD_logo.jpg" width="250px"></a> | <a href="http://nellore.bio/"><img src="https://seekvectorlogo.net/wp-content/uploads/2018/08/oregon-health-science-university-ohsu-vector-logo.png" width="250px"></a> | <a href="https://www.sciserver.org/"><img src="https://skyserver.sdss.org/dr14/en/images/sciserver_logo_inverted_vertical.png" width="250px"></a> |

<!-- Global site tag (gtag.js) - Google Analytics -->
<script async src="https://www.googletagmanager.com/gtag/js?id=UA-78422749-1"></script>
<script>
  window.dataLayer = window.dataLayer || [];
  function gtag(){dataLayer.push(arguments);}
  gtag('js', new Date());

  gtag('config', 'UA-78422749-1');
</script>

#' Given a set of regions for a chromosome, compute the coverage matrix for a
#' given SRA study.
#'
#' Given a set of genomic regions as created by [expressed_regions], this
#' function computes the coverage matrix for a library size of 40 million 100 bp
#' reads for a given SRA study.
#'
#' @inheritParams expressed_regions
#' @param regions A [GRanges-class][GenomicRanges::GRanges-class] object with regions
#' for `chr` for which to calculate the coverage matrix.
#' @param chunksize A single integer vector defining the chunksize to use for
#' computing the coverage matrix. Regions will be split into different chunks
#' which can be useful when using a parallel instance as defined by
#' `bpparam`.
#' @param bpparam A [BiocParallelParam-class][BiocParallel::BiocParallelParam-class] instance which
#' will be used to calculate the coverage matrix in parallel. By default,
#' [SerialParam-class][BiocParallel::SerialParam-class] will be used.
#' @param verboseLoad If `TRUE` basic status updates for loading the data
#' will be printed.
#' @param scale If `TRUE`, the coverage counts will be scaled to read
#' counts based on a library size of 40 million reads. Set `scale` to
#' `FALSE` if you want the raw coverage counts. The scaling method is by
#' AUC, as in the default option of [scale_counts].
#' @param round If `TRUE`, the counts are rounded to integers. Set to
#' `TRUE` if you want to match the defaults of [scale_counts].
#'
#'
#' @return A [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class]
#' object with the counts stored in the assays slot.
#'
#' @details When using `outdir = NULL` the information will be accessed
#' from the web on the fly. If you encounter internet access problems, it might
#' be best to first download the BigWig files using [download_study]. This
#' might be the best option if you are accessing all chromosomes for a given
#' project and/or are thinking of using different sets of `regions` (for
#' example, from different cutoffs applied to [expressed_regions]).
#' Alternatively check the `SciServer` section on the vignette to see
#' how to access all the recount data via a R Jupyter Notebook.
#'
#' If you have `bwtool` installed, you can use
#' <https://github.com/LieberInstitute/recount.bwtool> for faster results.
#' Note that you will need to run [scale_counts] after running
#' `coverage_matrix_bwtool()`.
#'
#' @author Leonardo Collado-Torres
#' @export
#'
#' @importFrom utils read.table
#' @import derfinder GenomicRanges RCurl BiocParallel
#' SummarizedExperiment S4Vectors
#'
#' @seealso [download_study], [findRegions][derfinder::findRegions],
#' [railMatrix][derfinder::railMatrix]
#'
#' @examples
#'
#' ## Workaround for https://github.com/lawremi/rtracklayer/issues/83
#' download_study("SRP002001", type = "mean")
#' download_study("SRP002001", type = "samples")
#'
#' ## Define expressed regions for study DRP002835, chrY
#' regions <- expressed_regions("SRP002001", "chrY",
#'     cutoff = 5L,
#'     maxClusterGap = 3000L,
#'     outdir = "SRP002001"
#' )
#'
#' ## Now calculate the coverage matrix for this study
#' rse <- coverage_matrix("SRP002001", "chrY", regions, outdir = "SRP002001")
#'
#' ## One row per region
#' identical(length(regions), nrow(rse))
coverage_matrix <- function(
        project, chr, regions, chunksize = 1000,
        bpparam = NULL, outdir = NULL, chrlen = NULL, verbose = TRUE,
        verboseLoad = verbose, scale = TRUE, round = FALSE, ...) {
    ## Check inputs
    stopifnot(is.character(project) & length(project) == 1)
    stopifnot(is.character(chr) & length(chr) == 1)
    stopifnot((is.numeric(chunksize) | is.integer(chunksize)) & length(chunksize) == 1)

    ## Use table from the package
    url_table <- recount::recount_url

    ## Subset url data
    url_table <- url_table[url_table$project == project, ]
    if (nrow(url_table) == 0) {
        stop("Invalid 'project' argument. There's no such 'project' in the recount_url data.frame.")
    }

    ## Find chromosome length if absent
    if (is.null(chrlen)) {
        chrinfo <- read.table("https://raw.githubusercontent.com/nellore/runs/master/gtex/hg38.sizes",
            col.names = c("chr", "size"), colClasses = c(
                "character",
                "integer"
            )
        )
        chrlen <- chrinfo$size[chrinfo$chr == chr]
        stopifnot(length(chrlen) == 1)
    }

    samples_i <- which(grepl("[.]bw$", url_table$file_name) & !grepl(
        "mean",
        url_table$file_name
    ))
    ## Check if data is present, otherwise download it
    if (!is.null(outdir)) {
        ## Check sample files
        sampleFiles <- sapply(samples_i, function(i) {
            file.path(outdir, "bw", url_table$file_name[i])
        })
        if (any(!file.exists(sampleFiles))) {
            download_study(
                project = project, type = "samples", outdir = outdir,
                download = TRUE, ...
            )
        }

        ## Check phenotype data
        phenoFile <- file.path(outdir, paste0(project, ".tsv"))
        if (!file.exists(phenoFile)) {
            download_study(
                project = project, type = "phenotype",
                outdir = outdir, download = TRUE, ...
            )
        }
    } else {
        sampleFiles <- download_study(
            project = project, type = "samples",
            download = FALSE
        )
        phenoFile <- download_study(
            project = project, type = "phenotype",
            download = FALSE
        )
    }

    ## Read pheno data
    pheno <- .read_pheno(phenoFile, project)

    ## Get sample names
    m <- match(url_table$file_name[samples_i], pheno$bigwig_file)
    ## Match the pheno with the samples
    pheno <- pheno[m, ]
    if (project != "TCGA") {
        names(sampleFiles) <- pheno$run
    } else {
        names(sampleFiles) <- pheno$gdc_file_id
    }

    ## Define library size normalization factor
    if (scale) {
        targetSize <- 40e6
        totalMapped <- pheno$auc[m]
        mappedPerXM <- totalMapped / targetSize
    } else {
        mappedPerXM <- rep(1, nrow(pheno))
    }


    ## Keep only regions from the chr in question
    regions <- regions[seqnames(regions) == chr]

    ## Split regions into chunks
    nChunks <- length(regions) %/% chunksize
    if (length(regions) %% chunksize > 0) nChunks <- nChunks + 1
    names(regions) <- seq_len(length(regions))

    ## Split regions into chunks
    if (nChunks == 1) {
        regs_split <- list(regions)
        names(regs_split) <- "1"
    } else {
        regs_split <- split(regions, cut(seq_len(length(regions)),
            breaks = nChunks, labels = FALSE
        ))
    }

    ## Define bpparam
    if (is.null(bpparam)) bpparam <- BiocParallel::SerialParam()

    ## Load coverage data
    resChunks <- lapply(regs_split, derfinder:::.railMatChrRegion,
        sampleFiles = sampleFiles, chr = chr, mappedPerXM = mappedPerXM,
        L = 1, verbose = verbose, BPPARAM.railChr = bpparam,
        verboseLoad = verboseLoad, chrlen = chrlen
    )

    ## Group results from chunks
    coverageMatrix <- do.call(rbind, lapply(resChunks, "[[", "coverageMatrix"))

    if (round) coverageMatrix <- round(coverageMatrix, 0)

    ## Build a RSE object
    rse <- SummarizedExperiment::SummarizedExperiment(
        assays = list("counts" = coverageMatrix),
        colData = DataFrame(pheno), rowRanges = regions
    )

    ## Finish
    return(rse)
}


## Helper function for reading the phenotype files
.read_pheno <- function(phenoFile, project) {
    if (project %in% c("SRP012682", "TCGA")) {
        subsets <- c("SRP012682" = "gtex", "TCGA" = "tcga")
        res <- all_metadata(subsets[project], verbose = FALSE)
    } else {
        info <- readLines(phenoFile)
        res <- read.table(
            text = info[grepl(
                paste0("^project|^", project),
                info
            )], header = TRUE, stringsAsFactors = FALSE, sep = "\t",
            comment.char = "", quote = ""
        )
    }
    return(res)
}

context("Download data, scale, ER-level")

library("SummarizedExperiment")

## Download the example data included in the package
url <- download_study("SRP009615", outdir = file.path(tempdir(), "SRP009615"))

## Load the data
load(file.path(tempdir(), "SRP009615", "rse_gene.Rdata"))

## Check size once:
# > file.path(tempdir(), 'SRP009615', 'rse_gene.Rdata')
# [1] "/var/folders/cx/n9s558kx6fb7jf5z_pgszgb80000gn/T//Rtmptm6u0K/SRP009615/rse_gene.Rdata"
# >
# > system('ls -lh /var/folders/cx/n9s558kx6fb7jf5z_pgszgb80000gn/T//Rtmptm6u0K/SRP009615/rse_gene.Rdata')
# -rw-r--r--  1 lcollado  staff   3.0M Apr 27 16:35 /var/folders/cx/n9s558kx6fb7jf5z_pgszgb80000gn/T//Rtmptm6u0K/SRP009615/rse_gene.Rdata
unlink(file.path(tempdir(), "SRP009615", "rse_gene.Rdata"))

## Compare the data
test_that("Example RSE", {
    expect_equivalent(rse_gene, rse_gene_SRP009615)
})

## Temporary download URLs
test_that("Download URLs", {
    expect_equal(
        download_study("DRP000366", type = "mean", download = FALSE),
        "http://duffel.rail.bio/recount/DRP000366/bw/mean_DRP000366.bw"
    )
    expect_equal(
        download_study("DRP000366",
            type = "samples",
            download = FALSE
        ),
        "http://duffel.rail.bio/recount/DRP000366/bw/DRR000897.bw"
    )
    expect_equal(
        download_study("DRP000366", type = "rse-fc", download = FALSE),
        "http://duffel.rail.bio/recount/fc_rc/rse_fc_DRP000366.Rdata"
    )
})

## Test downloading a small project entirely
tmpdir <- file.path(tempdir(), "SRP002001")
urls <- download_study("SRP002001", type = "all", outdir = tmpdir)
expected_urls <- paste0(
    "http://duffel.rail.bio/recount/",
    rep(rep(c("v2/", ""), 3), c(2, 1, 3, 2, 1, 2)), "SRP002001/",
    c(
        "rse_gene.Rdata", "rse_exon.Rdata", "rse_jx.Rdata",
        "rse_tx.RData", "counts_gene.tsv.gz",
        "counts_exon.tsv.gz", "counts_jx.tsv.gz",
        "SRP002001.tsv", "files_info.tsv",
        "bw/SRR036661.bw", "bw/mean_SRP002001.bw"
    )
)
names(expected_urls) <- c(
    "rse-gene", "rse-exon", "rse-jx", "rse-tx",
    "counts-gene", "counts-exon", "counts-jx",
    "phenotype", "files-info", "samples", "mean"
)

## Compute md5sum locally
localfiles <- c(
    list.files(tmpdir, "[.]", full.names = TRUE),
    dir(file.path(tmpdir, "bw"), full.names = TRUE)
)
names(localfiles) <- c(list.files(tmpdir, "[.]"), dir(file.path(tmpdir, "bw")))

library("tools")
md5 <- sapply(localfiles, md5sum)
names(md5) <- names(localfiles)
## md5sum doesn't match for jx files?
md5 <- md5[-which(names(md5) %in% c(
    "files_info.tsv", "counts_jx.tsv.gz",
    "rse_jx.Rdata"
))]

## Get original md5sum
fileinfo <- read.table(file.path(tmpdir, "files_info.tsv"),
    header = TRUE,
    stringsAsFactors = FALSE
)

test_that("Project SRP002001", {
    expect_equal(urls, expected_urls)
    expect_equivalent(fileinfo$md5sum[match(names(md5), fileinfo$file)], md5)
})

scaleFac <- scale_counts(rse_gene_SRP009615, factor_only = TRUE)
scaleFac_mapped <- scale_counts(rse_gene_SRP009615,
    by = "mapped_reads",
    factor_only = TRUE
)
rse <- scale_counts(rse_gene_SRP009615, round = FALSE)

test_that("Scaling", {
    expect_equal(round(head(scaleFac), 2), c(
        "SRR387777" = 0.04,
        "SRR387778" = 0.03, "SRR387779" = 0.03, "SRR387780" = 0.04,
        "SRR389077" = 0.04, "SRR389078" = 0.04
    ))
    expect_gt(scaleFac_mapped[1], scaleFac[1])
    expect_gt(scaleFac_mapped[2], scaleFac[2])
    expect_gt(scaleFac_mapped[3], scaleFac[3])
    expect_equal(assay(rse, 1) / matrix(rep(scaleFac, each = 58037),
        ncol = 12
    ), assay(rse_gene_SRP009615, 1))
})

regions <- expressed_regions("SRP002001", "chrY", cutoff = 5, outdir = tmpdir)
## Artificially remove the mean coverage file so that the file will have to
## get downloaded on the first test, then it'll be present for the second
## test
unlink(localfiles["mean_SRP002001.bw"])

test_that("Expressed regions", {
    expect_equal(
        regions,
        expressed_regions("SRP002001", "chrY", cutoff = 5, outdir = tmpdir)
    )
})


rse_ER <- coverage_matrix("SRP002001", "chrY", regions, outdir = tmpdir)
## Same for the phenotype data and the sample bigwig file
unlink(localfiles["SRP002001.tsv"])
unlink(localfiles["SRR036661.bw"])

test_that("Coverage matrix", {
    expect_equal(
        rse_ER,
        coverage_matrix("SRP002001", "chrY", regions, outdir = tmpdir)
    )
    expect_equal(
        rse_ER,
        coverage_matrix("SRP002001", "chrY", regions,
                        outdir = tmpdir,
                        chunksize = 500
        )
    )
})

## Check size once:
# > tmpdir
# [1] "/var/folders/9f/82m1lr2n1fv1mk91plf2l_dr0000gn/T//Rtmp8kCCFV/SRP002001"
# > system("du -sh /var/folders/9f/82m1lr2n1fv1mk91plf2l_dr0000gn/T//Rtmp8kCCFV/SRP002001")
#  90M	/var/folders/9f/82m1lr2n1fv1mk91plf2l_dr0000gn/T//Rtmp8kCCFV/SRP002001
unlink(tmpdir, recursive = TRUE)


metadata <- all_metadata()
test_that("All metadata", {
    expect_equal(nrow(metadata), 50099)
})

phenoFile <- download_study(
    project = "SRP012289", type = "phenotype",
    download = FALSE
)
pheno <- read.table(phenoFile,
    header = TRUE, stringsAsFactors = FALSE,
    sep = "\t"
)
test_that("Correct phenotype information", {
    expect_equal(pheno$auc, 159080954)
})


## Test Snaptron
library("GenomicRanges")
junctions <- GRanges(seqnames = "chr2", IRanges(
    start = c(28971710:28971712, 29555081:29555083, 29754982:29754984),
    end = c(29462417:29462419, 29923338:29923340, 29917714:29917716)
))

junctions_v2 <- GRanges(seqnames = "chr2", IRanges(
    start = 29532116:29532118, end = 29694848:29694850
))

snap <- snaptron_query(junctions)
snap_v2 <- snaptron_query(junctions_v2, version = "srav2")
snap_gtex <- snaptron_query(junctions_v2, version = "gtex")
snap_tcga <- snaptron_query(junctions_v2, version = "tcga")

if (!is.null(snap)) {
    test_that("Snaptron", {
        expect_equal(length(snap), 3)
        expect_equal(ncol(mcols(snap)), 14)
        expect_equal(snap$left_annotated[[1]], as.character(NA))
        expect_equal(snaptron_query(junctions[1], verbose = FALSE), NULL)
        expect_equal(is(snap_v2$annotated, "CompressedCharacterList"), TRUE)
        expect_equal(snaptron_query(junctions_v2, verbose = FALSE), NULL)
        expect_equal(snap_gtex$type == "GTEx:I", TRUE)
        expect_equal(snap_tcga$type == "TCGA:I", TRUE)
    })
} else {
    warning("Snaptron_query() is not working! See https://github.com/ChristopherWilks/snaptron/issues/17 for more details.")
}


## Weird pheno files
## First 2 are separate from the rest
projects <- c(
    "SRP036843", "SRP029334", "SRP050563", "SRP055438", "SRP055749",
    "SRP058120", "SRP005342", "SRP007508", "SRP015668"
)
sapply(projects, download_study, type = "phenotype")
phenoFiles <- sapply(projects, function(x) {
    file.path(x, paste0(x, ".tsv"))
})
pheno <- mapply(recount:::.read_pheno, phenoFiles, projects, SIMPLIFY = FALSE)

pheno_tcga <- recount:::.read_pheno("doesntexist", "TCGA")
test_that("Weird pheno files", {
    expect_equivalent(sapply(pheno, nrow), c(3, 5, 4, 33, 16, 30, 12, 5, 33))
    expect_equal(dim(pheno_tcga), c(11284, 864))
})


test_that("RPKM", {
    expect_equal(getRPKM(rse_gene_SRP009615)[1, ], assays(rse_gene_SRP009615)$count[1, ] / (rowData(rse_gene_SRP009615)$bp_length[1] / 1000) / (colSums(assays(rse_gene_SRP009615)$count) / 1e6))
    expect_equal(getRPKM(rse_gene_SRP009615, length_var = NULL)[1, ], assays(rse_gene_SRP009615)$count[1, ] / (width(rowRanges(rse_gene_SRP009615))[1] / 1000) / (colSums(assays(rse_gene_SRP009615)$count) / 1e6))
    expect_equal(getRPKM(rse_gene_SRP009615, mapped_var = "mapped_read_count")[1, ], assays(rse_gene_SRP009615)$count[1, ] / (rowData(rse_gene_SRP009615)$bp_length[1] / 1000) / (colData(rse_gene_SRP009615)$mapped_read_count / 1e6))
    expect_equal(getRPKM(rse_gene_SRP009615, length_var = NULL, mapped_var = "mapped_read_count")[1, ], assays(rse_gene_SRP009615)$count[1, ] / (width(rowRanges(rse_gene_SRP009615))[1] / 1000) / (colData(rse_gene_SRP009615)$mapped_read_count / 1e6))
})

rse_pred <- add_predictions(rse_gene_SRP009615)
test_that("Predictions", {
    expect_equal(all(c("reported_sex", "predicted_sex", "accuracy_sex", "reported_samplesource", "predicted_samplesource", "accuracy_samplesource", "reported_tissue", "predicted_tissue", "accuracy_tissue", "reported_sequencingstrategy", "predicted_sequencingstrategy", "accuracy_sequencingstrategy") %in% colnames(colData(rse_pred))), TRUE)
})

## read_counts
download_study("DRP000499")
load("DRP000499/rse_gene.Rdata")
test_that("read_counts", {
    expect_equal(all(colSums(assays(read_counts(rse_gene, use_paired_end = FALSE))$counts) / colSums(assays(read_counts(rse_gene))$counts) == 2), TRUE)
    expect_equal(all(sign(colSums(assays(read_counts(rse_gene))$counts) / 1e6 - colData(rse_gene)$reads_downloaded / 1e6 / 2) == -1), TRUE)
})

## Clean up
for (p in c(projects, "DRP000499")) unlink(p, recursive = TRUE)

#' Files and URLs hosted by the recount project
#'
#' Files and URLs as provided by the recount project. This information is used
#' internally in [download_study].
#'
#' @name recount_url
#' @docType data
#' @format  A data.frame with 6 columns.
#' \describe{
#'     \item{path }{ the original path to the file before being uploaded,}
#'     \item{file_name }{ the file name,}
#'     \item{project }{ the SRA project id,}
#'     \item{version1 }{ A logical vector indicating whether the file was
#'     part of version 1 (reduced exons)}.
#'     \item{version 2}{ A logical vector indicating whether the file was
#'     updated in version 2 (disjoint exons)}. Further details in the recount
#'     website documentation tab.
#'     \item{url }{ the public URL for the given file.}
#' }
#'
#' @keywords datasets
#' @seealso [download_study]
#' @references <https://jhubiostatistics.shinyapps.io/recount/>
NULL

#' Compute a TPM matrix based on a RangedSummarizedExperiment object
#'
#' For some analyses you might be interested in transforming the counts into
#' TPMs which you can do with this function. This function uses the gene-level
#' RPKMs to derive TPM values (see Details).
#'
#'
#' @inheritParams getRPKM
#'
#' @return A matrix with the TPM values.
#'
#' @details For gene RSE objects, you will want to specify the `length_var`
#' because otherwise you will be adjusting for the total gene length instead
#' of the total exonic sequence length of the gene.
#'
#' As noted in https://support.bioconductor.org/p/124265/,
#' Sonali Arora et al computed TPMs in
#' https://www.biorxiv.org/content/10.1101/445601v2 using the formula:
#' TPM = FPKM / (sum of FPKM over all genes/transcripts) * 10^6
#'
#' Arora et al mention in their code that the formula comes from
#' https://doi.org/10.1093/bioinformatics/btp692; specifically
#' `1.1.1 Comparison to RPKM estimation` where they mention an important
#' assumption: Under the assumption of uniformly distributed reads, we note
#' that RPKM measures are estimates of ...
#'
#' There's also a blog post by Harold Pimentel explaining the relationship
#' between FPKM and TPM:
#' https://haroldpimentel.wordpress.com/2014/05/08/what-the-fpkm-a-review-rna-seq-expression-units/.
#'
#' @seealso [getRPKM]
#'
#' @author Sonali Arora, Leonardo Collado-Torres
#' @export
#' @references https://www.biorxiv.org/content/10.1101/445601v2
#' https://arxiv.org/abs/1104.3889
#'
#' @examples
#'
#' ## Compute the TPM matrix from the raw gene-level base-pair counts.
#' tpm <- getTPM(rse_gene_SRP009615)
getTPM <- function(rse, length_var = "bp_length", mapped_var = NULL) {
    # From https://support.bioconductor.org/p/124265/
    rpkm <- getRPKM(rse, length_var = length_var, mapped_var = mapped_var)
    apply(rpkm, 2, function(x) {
        (x / sum(x)) * 10^6
    })
}

#' Extract information from GEO for a given sample
#'
#' This function uses GEOquery to extract information for a given sample. The
#' GEO accession ids for the sample can be found in the study phenotype table.
#'
#' @return Returns a [DataFrame-class][S4Vectors::DataFrame-class] with the information
#' from GEO available for the given sample.
#'
#' @param geoid A character vector of length 1 with the GEO accession id for
#' a given sample.
#' @param verbose If `TRUE` the `geoid` will be shown.
#' @param sleep The number of seconds (or fraction) to wait before downloading
#' data using [getGEO][GEOquery::getGEO]. This is important if you are looking over
#' `geo_info()` given the constraints published at
#' <https://www.ncbi.nlm.nih.gov/books/NBK25497/>.
#' @param getGPL This argument is passed to [getGEO][GEOquery::getGEO] and is set
#' to `FALSE` by default to speed up the process.
#' @param destdir This argument is passed to [getGEO][GEOquery::getGEO].
#' @param ... Additional arguments passed to [getGEO][GEOquery::getGEO].
#'
#' @author Leonardo Collado-Torres, Andrew Jaffe
#' @export
#'
#' @import GEOquery IRanges S4Vectors
#'
#' @examples
#' geo_info("GSM836270")
geo_info <- function(
        geoid, verbose = FALSE, sleep = 1 / 2, getGPL = FALSE,
        destdir = tempdir(), ...) {
    if (is.na(geoid)) {
        return(NULL)
    }

    ## Check inputs
    stopifnot(is.character(geoid) & length(geoid) == 1)

    if (verbose) {
        message(paste(
            Sys.time(),
            "finding GEO information for GEO accession id", geoid
        ))
    }

    if (!file.exists(file.path(destdir, paste0(geoid, ".soft")))) {
        Sys.sleep(sleep)
    }

    ## Get data from GEO, with 3 retries, waiting between 0 and 2 seconds in
    ## between retries
    N.TRIES <- 3L
    while (N.TRIES > 0L) {
        geo <- tryCatch(
            GEOquery::getGEO(geoid, getGPL = getGPL, destdir = destdir, ...),
            error = function(e) {
                soft <- paste0(geoid, ".soft")
                soft_file <- file.path(destdir, soft)
                if (any(grepl("private", readLines(soft_file)))) {
                    message(paste(geoid, "is currently private"))
                    return(NA)
                } else if (any(grepl("blocked", readLines(soft_file)))) {
                    warning(paste("It seems like your IP access is blocked. Please check the file", soft_file, "for more details."))
                    return(NA)
                } else {
                    return(e)
                }
            }
        )
        if (!inherits(geo, "error")) {
            break
        }
        Sys.sleep(runif(n = 1, min = 2, max = 5))
        N.TRIES <- N.TRIES - 1L
    }

    ## Return and empty DataFrame if there was an issue with getGEO()
    if (!is(geo, "GSM")) {
        return(S4Vectors::DataFrame())
    }

    ## Extract the header information
    result <- geo@header

    ## Remove trailing \r on Windows
    if (.Platform$OS.type == "windows") {
        result <- lapply(result, function(x) gsub("\r", "", x))
    }

    ## Function for cleaning
    clean_geo <- function(pattern, varname, res) {
        charIndex <- grep(pattern, names(res))
        if (length(charIndex) > 0) {
            res <- c(
                res,
                IRanges::CharacterList(unlist(unname(result[charIndex])))
            )
            names(res)[length(res)] <- varname
            res <- res[-charIndex]
        }
        return(res)
    }

    ## Clean up the header information
    df <- data.frame(
        pattern = c(
            "characteristics_ch1", "data_processing", "contact_",
            "extract_", "library_", "relation", "series_",
            "supplementary_file_"
        ),
        varname = c(
            "characteristics", "data_processing", "contact", "extract",
            "library", "relation", "series", "supplementary_file"
        ),
        stringsAsFactors = FALSE
    )
    for (i in seq_len(nrow(df))) {
        result <- clean_geo(
            df$pattern[i],
            df$varname[i], result
        )
    }

    ## Finish
    return(S4Vectors::DataFrame(result))
}

context("Misc")

test_that("Loading packages", {
    expect_equal(recount:::.load_check("recount"), NULL)
    expect_error(recount:::.load_check("somecrazyname"))
})

info <- geo_info("GSM836270")
chars <- data.frame(
    "cells" = "K562", "shrna.expression" = "no",
    "treatment" = "Puromycin", stringsAsFactors = FALSE
)
test_that("Geo info", {
    expect_equal(info, geo_info("GSM836270", TRUE))
    expect_equal(info$geo_accession, "GSM836270")
    expect_equal(find_geo("SRX110461"), "GSM836270")
    expect_equal(find_geo(""), NA)
    expect_equal(geo_characteristics(info), chars)
    expect_equal(geo_info(NA), NULL)
    expect_equal(colnames(geo_characteristics(geo_info("GSM359183"))), "characteristics")
})

if (interactive()) {
    ## Open shiny app when running test in interactive mode.
    ## This will force the app to be available, which will then run the test
    ## successfully.
    ## If there is a better way to test this, please let me know!
    ## See thread at https://groups.google.com/forum/#!topic/shinyapps-users/ElMO_v1eurQ
    browseURL("https://jhubiostatistics.shinyapps.io/recount/")
    test_that("Shiny app is up", {
        expect_equal(
            RCurl::url.exists("https://jhubiostatistics.shinyapps.io/recount/"),
            TRUE
        )
    })
}

#' Search the abstracts from the SRA studies available via the recount project
#'
#' Given a text query, find the SRA project ids (study accession numbers) that
#' contain the text in their abstract as provided by the SRAdb Bioconductor
#' package.
#'
#' @param query A character vector with the text to search for via
#' [grep][base::grep] in the abstract info available at [recount_abstract].
#' @param id_only Whether to only return the project id or to return summary
#' information for the project(s) that match the query.
#' @param ... Additional arguments passed to [grep][base::grep].
#'
#' @return If `id_only = TRUE` it returns a character vector with the
#' project SRA ids (accession numbers). If `id_only = FALSE` it returns a
#' subset of [recount_abstract] for the abstracts that contained the query.
#'
#' @details Both the query and the abstracts are searched in lower case.
#'
#' For a more powerful search use the recount project website at
#' <https://jhubiostatistics.shinyapps.io/recount/>.
#'
#' @seealso [browse_study], [recount_abstract]
#'
#' @author Leonardo Collado-Torres
#' @export
#'
#' @examples
#' ## Find the Geuvadis consortium project
#' project_info <- abstract_search("Geuvadis consortium")
#'
#' ## See some summary information for this project
#' project_info
abstract_search <- function(query, id_only = FALSE, ...) {
    ## Check input
    stopifnot(is.character(query))
    stopifnot(is.logical(id_only))
    stopifnot(length(id_only) == 1)

    ## Use table from the package
    abstract_table <- recount::recount_abstract

    ## Get abstracts
    abstracts <- tolower(abstract_table$abstract)
    query <- tolower(query)
    i <- grep(query, abstracts, ...)

    if (id_only) {
        return(abstract_table$project[i])
    } else {
        return(abstract_table[i, ])
    }
}

context("Website metadata")

test_that("Browse studies at SRA", {
    expect_equal(length(browse_study(c("ERP001942", "ERP009768"), FALSE)), 2)
    expect_equal(browse_study("ERP001942", FALSE), "https://trace.ncbi.nlm.nih.gov/Traces/sra/?study=ERP001942")
})

test_that("Download studies", {
    expect_equal(download_study("ERP001942", download = FALSE), "http://duffel.rail.bio/recount/v2/ERP001942/rse_gene.Rdata")
    expect_equal(download_study("ERP001942", download = FALSE, version = 1), "http://duffel.rail.bio/recount/ERP001942/rse_gene.Rdata")
    expect_equal(download_study("ERP001942", type = "mean", download = FALSE), "http://duffel.rail.bio/recount/ERP001942/bw/mean_ERP001942.bw")
})


test_that("Finding abstracts", {
    expect_equal(
        abstract_search("Geuvadis consortium", id_only = TRUE),
        "ERP001942"
    )
    expect_equal(
        abstract_search("Geuvadis consortium"),
        recount_abstract[recount_abstract$project == "ERP001942", ]
    )
})

#' Download data for a given SRA study id from the recount project
#'
#' Download the gene or exon level
#' [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class] objects
#' provided by the recount project. Alternatively download the counts, metadata
#' or file information for a given SRA study id. You can also download the
#' sample bigWig files or the mean coverage bigWig file.
#'
#' @param project A character vector with one SRA study id.
#' @param type Specifies which files to download. The options are:
#' \describe{
#'     \item{rse-gene}{ the gene-level
#'     [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class] object in
#'     a file named rse_gene.Rdata.}
#'     \item{rse-exon}{ the exon-level
#'     [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class] object in
#'     a file named rse_exon.Rdata.}
#'     \item{rse-jx}{ the exon-exon junction level
#'     [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class] object in
#'     a file named rse_jx.Rdata.}
#'     \item{rse-tx}{ the transcript level
#'     [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class] object in
#'     a file named rse_tx.RData.}
#'     \item{counts-gene}{ the gene-level counts in a tsv file named
#'     counts_gene.tsv.gz.}
#'     \item{counts-exon}{ the exon-level counts in a tsv file named
#'     counts_exon.tsv.gz.}
#'     \item{counts-jx}{ the exon-exon junction level counts in a tsv file named
#'     counts_jx.tsv.gz.}
#'     \item{phenotype}{ the phenotype data for the study in a tsv file named
#'     `project`.tsv.}
#'     \item{files-info}{ the files information for the given study (including
#'     md5sum hashes) in a tsv file named files_info.tsv.}
#'     \item{samples}{ one bigWig file per sample in the study.}
#'     \item{mean}{ one mean bigWig file for the samples in the study,
#'     with each sample normalized to a 40 million 100 bp library using the
#'     total coverage sum (area under the coverage curve, AUC) for the given
#'     sample.}
#'     \item{all}{ Downloads all the above types. Note that it might take some
#'     time if the project has many samples. When using `type = 'all'` a
#'     small delay will be added before each download request to avoid
#'     request issues.}
#'     \item{rse-fc}{ Downloads the FANTOM-CAT/recount2 rse file described in
#'     Imada, Sanchez, et al., bioRxiv, 2019.}
#' }
#' @param outdir The destination directory for the downloaded file(s).
#' Alternatively check the `SciServer` section on the vignette to see
#' how to access all the recount data via a R Jupyter Notebook.
#' @param download Whether to download the files or just get the download urls.
#' @param version A single integer specifying which version of the files to
#' download. Valid options are 1 and 2, as described in
#' <https://jhubiostatistics.shinyapps.io/recount/> under the
#' documentation tab. Briefly, version 1 are counts based on reduced exons while
#' version 2 are based on disjoint exons. This argument mostly just matters for
#' the exon counts. Defaults to version 2 (disjoint exons).
#' Use `version = 1` for backward compatability with exon counts
#' prior to version 1.5.3 of the package.
#' @param ... Additional arguments passed to [download][downloader::download].
#'
#' @return Returns invisibly the URL(s) for the files that were downloaded.
#'
#' @details Check <http://stackoverflow.com/a/34383991> if you need to find
#' the effective URLs. For example,
#' <http://duffel.rail.bio/recount/DRP000366/bw/mean_DRP000366.bw> points to
#' a link from SciServer.
#'
#' Transcript quantifications are described in Fu et al, bioRxiv, 2018.
#' <https://www.biorxiv.org/content/10.1101/247346v2>
#'
#' FANTOM-CAT/recount2 quantifications are described in Imada,
#' Sanchez, et al., bioRxiv, 2019.
#' <https://www.biorxiv.org/content/10.1101/659490v1>
#'
#'
#' @author Leonardo Collado-Torres
#' @export
#'
#' @importFrom stats runif
#' @import downloader
#'
#' @examples
#' ## Find the URL to download the RangedSummarizedExperiment for the
#' ## Geuvadis consortium study.
#' url <- download_study("ERP001942", download = FALSE)
#'
#' ## See the actual URL
#' url
#' \dontrun{
#' ## Download the example data included in the package for study SRP009615
#'
#' url2 <- download_study("SRP009615")
#' url2
#'
#' ## Load the data
#' load(file.path("SRP009615", "rse_gene.Rdata"))
#'
#' ## Compare the data
#' library("testthat")
#' expect_equivalent(rse_gene, rse_gene_SRP009615)
#' }
#'
download_study <- function(
        project, type = "rse-gene", outdir = project,
        download = TRUE, version = 2, ...) {
    ## Check inputs
    stopifnot(is.character(project) & length(project) == 1)
    stopifnot(version %in% c(1, 2))
    stopifnot(type %in% c(
        "rse-gene", "rse-exon", "rse-jx", "rse-tx",
        "counts-gene", "counts-exon", "counts-jx",
        "phenotype", "files-info", "samples", "mean", "rse-fc", "all"
    ))
    stopifnot(length(type) == 1)
    stopifnot(is.logical(download) & length(download) == 1)

    ## Use table from the package
    url_table <- recount::recount_url

    ## URLs default to version 2 (disjoint exons).
    if (version == 1) url_table$url <- gsub("v2/", "", url_table$url)

    ## Subset url data
    url_table <- url_table[url_table$project == project, ]
    if (nrow(url_table) == 0) {
        stop("Invalid 'project' argument. There's no such 'project' in the recount_url data.frame.")
    }

    ## Use duffel for most links
    url_table$url <- gsub("http://idies.jhu.edu/recount/data", "http://duffel.rail.bio/recount", url_table$url)

    ## If all, download each type individually
    if (type == "all") {
        urls <- sapply(c(
            "rse-gene", "rse-exon", "rse-jx", "rse-tx",
            "counts-gene", "counts-exon", "counts-jx",
            "phenotype", "files-info", "samples", "mean"
        ), function(file_type) {
            Sys.sleep(runif(n = 1, min = 2, max = 5))
            download_study(
                project = project, type = file_type,
                outdir = outdir, download = download, version = version, ...
            )
        })
        return(invisible(urls))
    }

    ## Create output directory if needed
    if (download) {
        dir.create(outdir, showWarnings = FALSE)
        if (type %in% c("samples", "mean")) {
            ## Save bigwigs on their own folder
            outdir <- file.path(outdir, "bw")
            dir.create(outdir, showWarnings = FALSE)
        }
    }

    ## Select files to download and download them
    if (type != "samples") {
        filename <- switch(type,
            "rse-gene" = "rse_gene.Rdata",
            "rse-exon" = "rse_exon.Rdata",
            "rse-jx" = "rse_jx.Rdata",
            "rse-tx" = "rse_tx.RData",
            "counts-gene" = "counts_gene.tsv.gz",
            "counts-exon" = "counts_exon.tsv.gz",
            "counts-jx" = "counts_jx.tsv.gz",
            phenotype = paste0(project, ".tsv"),
            "files-info" = "files_info.tsv",
            mean = paste0("mean_", project, ".bw"),
            "rse-fc" = paste0("rse_fc_", project, ".Rdata")
        )
        url <- url_table$url[url_table$file_name == filename]
        if (length(url) == 0) {
            return(NULL)
        }
        if (download) {
            message(paste(
                Sys.time(), "downloading file", filename, "to",
                outdir
            ))
            xx <- download_retry(
                url = url,
                destfile = file.path(outdir, filename),
                mode = "wb",
                ...
            )
        }
    } else if (type == "samples") {
        url_table <- url_table[url_table$file_name != paste0(
            "mean_", project,
            ".bw"
        ), ]
        sample_urls <- url_table[grep("[.]bw$", url_table$file_name), ]
        url <- sample_urls$url
        if (download) {
            xx <- sapply(seq_len(nrow(sample_urls)), function(i, ...) {
                message(paste(
                    Sys.time(), "downloading file",
                    sample_urls$file_name[i], "to", outdir
                ))
                download_retry(
                    url = sample_urls$url[i],
                    destfile = file.path(outdir, sample_urls$file_name[i]),
                    mode = "wb",
                    ...
                )
            }, ...)
        }
    }

    ## Return the actual url(s)
    return(invisible(url))
}

## ----vignetteSetup, echo=FALSE, message=FALSE, warning = FALSE----------------
## Track time spent on making the vignette
startTime <- Sys.time()

## Bib setup
library("RefManageR")

## Write bibliography information
bib <- c(
    R = citation(),
    AnnotationDbi = RefManageR::BibEntry(
        bibtype = "manual",
        key = "AnnotationDbi",
        author = "Hervé Pagès and Marc Carlson and Seth Falcon and Nianhua Li",
        title = "AnnotationDbi: Annotation Database Interface",
        year = 2017, doi = "10.18129/B9.bioc.AnnotationDbi"
    ),
    BiocParallel = citation("BiocParallel")[1],
    BiocStyle = citation("BiocStyle")[1],
    derfinder = citation("derfinder")[1],
    DESeq2 = citation("DESeq2")[1],
    sessioninfo = citation("sessioninfo")[1],
    downloader = citation("downloader")[1],
    EnsDb.Hsapiens.v79 = citation("EnsDb.Hsapiens.v79")[1],
    GEOquery = citation("GEOquery")[1],
    GenomeInfoDb = RefManageR::BibEntry(
        bibtype = "manual",
        key = "GenomeInfoDb",
        author = "Sonali Arora and Martin Morgan and Marc Carlson and H. Pagès",
        title = "GenomeInfoDb: Utilities for manipulating chromosome and other 'seqname' identifiers",
        year = 2017, doi = "10.18129/B9.bioc.GenomeInfoDb"
    ),
    GenomicFeatures = citation("GenomicFeatures")[1],
    GenomicRanges = citation("GenomicRanges")[1],
    IRanges = citation("IRanges")[1],
    knitr = citation("knitr")[3],
    org.Hs.eg.db = citation("org.Hs.eg.db")[1],
    RCurl = citation("RCurl")[1],
    recount = citation("recount")[1],
    recountworkflow = citation("recount")[2],
    phenopredict = citation("recount")[3],
    regionReport = citation("regionReport")[1],
    rentrez = citation("rentrez")[1],
    RefManageR = citation("RefManageR")[1],
    rmarkdown = citation("rmarkdown")[1],
    rtracklayer = citation("rtracklayer")[1],
    S4Vectors = RefManageR::BibEntry(
        bibtype = "manual", key = "S4Vectors",
        author = "Hervé Pagès and Michael Lawrence and Patrick Aboyoun",
        title = "S4Vectors: S4 implementation of vector-like and list-like objects",
        year = 2017, doi = "10.18129/B9.bioc.S4Vectors"
    ),
    SummarizedExperiment = RefManageR::BibEntry(
        bibtype = "manual",
        key = "SummarizedExperiment",
        author = "Martin Morgan and Valerie Obenchain and Jim Hester and Hervé Pagès",
        title = "SummarizedExperiment: SummarizedExperiment container",
        year = 2017, doi = "10.18129/B9.bioc.SummarizedExperiment"
    ),
    testthat = citation("testthat")[1]
)

## ----'installDer', eval = FALSE-----------------------------------------------
# if (!requireNamespace("BiocManager", quietly = TRUE)) {
#     install.packages("BiocManager")
# }
# 
# BiocManager::install("recount")
# 
# ## Check that you have a valid Bioconductor installation
# BiocManager::valid()

## ----'citation'---------------------------------------------------------------
## Citation info
citation("recount")

## ----'ultraQuick', eval = FALSE-----------------------------------------------
# ## Load library
# library("recount")
# 
# ## Find a project of interest
# project_info <- abstract_search("GSE32465")
# 
# ## Download the gene-level RangedSummarizedExperiment data
# download_study(project_info$project)
# 
# ## Load the data
# load(file.path(project_info$project, "rse_gene.Rdata"))
# 
# ## Browse the project at SRA
# browse_study(project_info$project)
# 
# ## View GEO ids
# colData(rse_gene)$geo_accession
# 
# ## Extract the sample characteristics
# geochar <- lapply(split(colData(rse_gene), seq_len(nrow(colData(rse_gene)))), geo_characteristics)
# 
# ## Note that the information for this study is a little inconsistent, so we
# ## have to fix it.
# geochar <- do.call(rbind, lapply(geochar, function(x) {
#     if ("cells" %in% colnames(x)) {
#         colnames(x)[colnames(x) == "cells"] <- "cell.line"
#         return(x)
#     } else {
#         return(x)
#     }
# }))
# 
# ## We can now define some sample information to use
# sample_info <- data.frame(
#     run = colData(rse_gene)$run,
#     group = ifelse(grepl("uninduced", colData(rse_gene)$title), "uninduced", "induced"),
#     gene_target = sapply(colData(rse_gene)$title, function(x) {
#         strsplit(strsplit(
#             x,
#             "targeting "
#         )[[1]][2], " gene")[[1]][1]
#     }),
#     cell.line = geochar$cell.line
# )
# 
# ## Scale counts by taking into account the total coverage per sample
# rse <- scale_counts(rse_gene)
# 
# ## Add sample information for DE analysis
# colData(rse)$group <- sample_info$group
# colData(rse)$gene_target <- sample_info$gene_target
# 
# ## Perform differential expression analysis with DESeq2
# library("DESeq2")
# 
# ## Specify design and switch to DESeq2 format
# dds <- DESeqDataSet(rse, ~ gene_target + group)
# 
# ## Perform DE analysis
# dds <- DESeq(dds, test = "LRT", reduced = ~gene_target, fitType = "local")
# res <- results(dds)
# 
# ## Explore results
# plotMA(res, main = "DESeq2 results for SRP009615")
# 
# ## Make a report with the results
# library("regionReport")
# DESeq2Report(dds,
#     res = res, project = "SRP009615",
#     intgroup = c("group", "gene_target"), outdir = ".",
#     output = "SRP009615-results"
# )

## ----'er_analysis', eval = FALSE----------------------------------------------
# ## Define expressed regions for study SRP009615, only for chromosome Y
# regions <- expressed_regions("SRP009615", "chrY",
#     cutoff = 5L,
#     maxClusterGap = 3000L
# )
# 
# ## Compute coverage matrix for study SRP009615, only for chromosome Y
# system.time(rse_ER <- coverage_matrix("SRP009615", "chrY", regions))
# 
# ## Round the coverage matrix to integers
# covMat <- round(assays(rse_ER)$counts, 0)
# 
# ## Get phenotype data for study SRP009615
# pheno <- colData(rse_ER)
# 
# ## Complete the phenotype table with the data we got from GEO
# m <- match(pheno$run, sample_info$run)
# pheno <- cbind(pheno, sample_info[m, 2:3])
# 
# ## Build a DESeqDataSet
# dds_ers <- DESeqDataSetFromMatrix(
#     countData = covMat, colData = pheno,
#     design = ~ gene_target + group
# )
# 
# ## Perform differential expression analysis with DESeq2 at the ER-level
# dds_ers <- DESeq(dds_ers,
#     test = "LRT", reduced = ~gene_target,
#     fitType = "local"
# )
# res_ers <- results(dds_ers)
# 
# ## Explore results
# plotMA(res_ers, main = "DESeq2 results for SRP009615 (ER-level, chrY)")
# 
# ## Create a more extensive exploratory report
# DESeq2Report(dds_ers,
#     res = res_ers,
#     project = "SRP009615 (ER-level, chrY)",
#     intgroup = c("group", "gene_target"), outdir = ".",
#     output = "SRP009615-results-ER-level-chrY"
# )

## ----'install', eval = FALSE--------------------------------------------------
# install.packages("BiocManager")
# BiocManager::install("recount")

## ----'start', message=FALSE---------------------------------------------------
## Load recount R package
library("recount")

## ----'search_abstract'--------------------------------------------------------
## Find a project of interest
project_info <- abstract_search("GSE32465")

## Explore info
project_info

## ----'download'---------------------------------------------------------------
## Download the gene-level RangedSummarizedExperiment data
download_study(project_info$project)

## Load the data
load(file.path(project_info$project, "rse_gene.Rdata"))

## Delete it if you don't need it anymore
unlink(project_info$project, recursive = TRUE)

## ----'explore_rse'------------------------------------------------------------
rse_gene

## This is the sample phenotype data provided by the recount project
colData(rse_gene)

## At the gene level, the row data includes the gene Gencode ids, the gene
## symbols and the sum of the disjoint exons widths, which can be used for
## taking into account the gene length.
rowData(rse_gene)

## At the exon level, you can get the gene Gencode ids from the names of:
# rowRanges(rse_exon)

## ----'browse'-----------------------------------------------------------------
## Browse the project at SRA
browse_study(project_info$project)

## ----'sample_info', warning = FALSE-------------------------------------------
## View GEO ids
colData(rse_gene)$geo_accession

## Extract the sample characteristics
geochar <- lapply(split(colData(rse_gene), seq_len(nrow(colData(rse_gene)))), geo_characteristics)

## Note that the information for this study is a little inconsistent, so we
## have to fix it.
geochar <- do.call(rbind, lapply(geochar, function(x) {
    if ("cells" %in% colnames(x)) {
        colnames(x)[colnames(x) == "cells"] <- "cell.line"
        return(x)
    } else {
        return(x)
    }
}))

## We can now define some sample information to use
sample_info <- data.frame(
    run = colData(rse_gene)$run,
    group = ifelse(grepl("uninduced", colData(rse_gene)$title), "uninduced", "induced"),
    gene_target = sapply(colData(rse_gene)$title, function(x) {
        strsplit(strsplit(
            x,
            "targeting "
        )[[1]][2], " gene")[[1]][1]
    }),
    cell.line = geochar$cell.line
)

## ----'scale_counts'-----------------------------------------------------------
## Scale counts by taking into account the total coverage per sample
rse <- scale_counts(rse_gene)

##### Details about counts #####

## Scale counts to 40 million mapped reads. Not all mapped reads are in exonic
## sequence, so the total is not necessarily 40 million.
colSums(assays(rse)$counts) / 1e6

## Compute read counts
rse_read_counts <- read_counts(rse_gene)
## Difference between read counts and number of reads downloaded by Rail-RNA
colSums(assays(rse_read_counts)$counts) / 1e6 -
    colData(rse_read_counts)$reads_downloaded / 1e6
## Check the help page for read_counts() for more details

## ----'add_sample_info'--------------------------------------------------------
## Add sample information for DE analysis
colData(rse)$group <- sample_info$group
colData(rse)$gene_target <- sample_info$gene_target

## ----'de_analysis'------------------------------------------------------------
## Perform differential expression analysis with DESeq2
library("DESeq2")

## Specify design and switch to DESeq2 format
dds <- DESeqDataSet(rse, ~ gene_target + group)

## Perform DE analysis
dds <- DESeq(dds, test = "LRT", reduced = ~gene_target, fitType = "local")
res <- results(dds)

## ----'maplot', fig.cap = "MA plot for group differences adjusted by gene target for SRP009615 using gene-level data."----
## Explore results
plotMA(res, main = "DESeq2 results for SRP009615")

## ----'make_report', eval = FALSE----------------------------------------------
# ## Make a report with the results
# library("regionReport")
# report <- DESeq2Report(dds,
#     res = res, project = "SRP009615",
#     intgroup = c("group", "gene_target"), outdir = ".",
#     output = "SRP009615-results", nBest = 10, nBestFeatures = 2
# )

## ----'make_report_real', echo = FALSE, results = 'hide'---------------------------------------------------------------
library("regionReport")

## Load some packages used by regionReport
library("DT")
library("edgeR")
library("ggplot2")
library("RColorBrewer")

## Make it so that the report will be available as a vignette
original <- readLines(system.file("DESeq2Exploration", "DESeq2Exploration.Rmd",
    package = "regionReport"
))
vignetteInfo <- c(
    "vignette: >",
    "  %\\VignetteEngine{knitr::rmarkdown}",
    "  %\\VignetteIndexEntry{Basic DESeq2 results exploration}",
    "  %\\VignetteEncoding{UTF-8}"
)
new <- c(original[1:16], vignetteInfo, original[17:length(original)])
writeLines(new, "SRP009615-results-template.Rmd")

## Now create the report
report <- DESeq2Report(dds,
    res = res, project = "SRP009615",
    intgroup = c("group", "gene_target"), outdir = ".",
    output = "SRP009615-results", device = "png", template = "SRP009615-results-template.Rmd", nBest = 10, nBestFeatures = 2
)

## Clean up
file.remove("SRP009615-results-template.Rmd")

## ----'geneSymbols'----------------------------------------------------------------------------------------------------
## Load required library
library("org.Hs.eg.db")

## Extract Gencode gene ids
gencode <- gsub("\\..*", "", names(recount_genes))

## Find the gene information we are interested in
gene_info <- select(org.Hs.eg.db, gencode, c(
    "ENTREZID", "GENENAME", "SYMBOL",
    "ENSEMBL"
), "ENSEMBL")

## Explore part of the results
dim(gene_info)
head(gene_info)

## ----'download_bigwigs'-----------------------------------------------------------------------------------------------
## Normally, one can use rtracklayer::import() to access remote parts of BigWig
## files without having to download the complete files. However, as of
## 2024-05-20 this doesn't seem to be working well. So this is a workaround to
## issue https://github.com/lawremi/rtracklayer/issues/83
download_study("SRP009615", type = "mean")
download_study("SRP009615", type = "samples")

## ----'define_ers'-----------------------------------------------------------------------------------------------------
## Define expressed regions for study SRP009615, only for chromosome Y
regions <- expressed_regions("SRP009615", "chrY",
    cutoff = 5L,
    maxClusterGap = 3000L, outdir = "SRP009615"
)

## Briefly explore the resulting regions
regions

## ----'compute_covMat'-------------------------------------------------------------------------------------------------
## Compute coverage matrix for study SRP009615, only for chromosome Y
system.time(rse_ER <- coverage_matrix("SRP009615", "chrY", regions, outdir = "SRP009615"))

## Explore the RSE a bit
dim(rse_ER)
rse_ER

## ----'to_integer'-----------------------------------------------------------------------------------------------------
## Round the coverage matrix to integers
covMat <- round(assays(rse_ER)$counts, 0)

## ----'phenoData'------------------------------------------------------------------------------------------------------
## Get phenotype data for study SRP009615
pheno <- colData(rse_ER)

## Complete the phenotype table with the data we got from GEO
m <- match(pheno$run, sample_info$run)
pheno <- cbind(pheno, sample_info[m, 2:3])

## Explore the phenotype data a little bit
head(pheno)

## ----'ers_dds'--------------------------------------------------------------------------------------------------------
## Build a DESeqDataSet
dds_ers <- DESeqDataSetFromMatrix(
    countData = covMat, colData = pheno,
    design = ~ gene_target + group
)

## ----'de_analysis_ers'------------------------------------------------------------------------------------------------
## Perform differential expression analysis with DESeq2 at the ER-level
dds_ers <- DESeq(dds_ers,
    test = "LRT", reduced = ~gene_target,
    fitType = "local"
)
res_ers <- results(dds_ers)

## ----'maploters', fig.cap = "MA plot for group differences adjusted by gene target for SRP009615 using ER-level data (just chrY)."----
## Explore results
plotMA(res_ers, main = "DESeq2 results for SRP009615 (ER-level, chrY)")

## ----'report2', eval = FALSE------------------------------------------------------------------------------------------
# ## Create the report
# report2 <- DESeq2Report(dds_ers,
#     res = res_ers,
#     project = "SRP009615 (ER-level, chrY)",
#     intgroup = c("group", "gene_target"), outdir = ".",
#     output = "SRP009615-results-ER-level-chrY"
# )

## ----'gene_annotation'------------------------------------------------------------------------------------------------
## Gene annotation information
rowRanges(rse_gene_SRP009615)

## Also accessible via
rowData(rse_gene_SRP009615)

## ----'exon_annotation'------------------------------------------------------------------------------------------------
## Get the rse_exon object for study SRP009615
download_study("SRP009615", type = "rse-exon")
load(file.path("SRP009615", "rse_exon.Rdata"))

## Annotation information
rowRanges(rse_exon)

## Add a gene_id column
rowRanges(rse_exon)$gene_id <- rownames(rse_exon)

## Example subset
rse_exon_subset <- subset(rse_exon, subset = gene_id == "ENSG00000000003.14")
rowRanges(rse_exon_subset)

## ----"newer annotation"-----------------------------------------------------------------------------------------------
## Get the disjoint exons based on EnsDb.Hsapiens.v79 which matches hg38
exons <- reproduce_ranges("exon", db = "EnsDb.Hsapiens.v79")

## Change the chromosome names to match those used in the BigWig files
library("GenomeInfoDb")
seqlevelsStyle(exons) <- "UCSC"

## Get the count matrix at the exon level for chrY
exons_chrY <- keepSeqlevels(exons, "chrY", pruning.mode = "coarse")
exonRSE <- coverage_matrix("SRP009615", "chrY", unlist(exons_chrY),
    chunksize = 3000, outdir = "SRP009615"
)
exonMatrix <- assays(exonRSE)$counts
dim(exonMatrix)
head(exonMatrix)

## Summary the information at the gene level for chrY
exon_gene <- rep(names(exons_chrY), elementNROWS(exons_chrY))
geneMatrix <- do.call(rbind, lapply(split(
    as.data.frame(exonMatrix),
    exon_gene
), colSums))
dim(geneMatrix)
head(geneMatrix)

## ----'gene_fusions'---------------------------------------------------------------------------------------------------
library("recount")

## Download and load RSE object for SRP009615 study
download_study("SRP009615", type = "rse-jx")
load(file.path("SRP009615", "rse_jx.Rdata"))

## Delete it if you don't need it anymore
unlink("SRP009615", recursive = TRUE)

## Exon-exon junctions by class
table(rowRanges(rse_jx)$class)

## Potential gene fusions by chromosome
fusions <- rowRanges(rse_jx)[rowRanges(rse_jx)$class == "fusion"]
fusions_by_chr <- sort(table(seqnames(fusions)), decreasing = TRUE)
fusions_by_chr[fusions_by_chr > 0]

## Genes with the most fusions
head(sort(table(unlist(fusions$symbol_proposed)), decreasing = TRUE))

## ----snaptron---------------------------------------------------------------------------------------------------------
library("GenomicRanges")
junctions <- GRanges(seqnames = "chr2", IRanges(
    start = c(28971711, 29555082, 29754983),
    end = c(29462418, 29923339, 29917715)
))

snaptron_query(junctions)

## ----'rse-fc example'-------------------------------------------------------------------------------------------------
## Example use of download_study()
## for downloading the FANTOM-CAT `rse_fc`
## file for a given study
download_study("DRP000366", type = "rse-fc", download = FALSE)

## ----'recount-brain'--------------------------------------------------------------------------------------------------
## recount-brain citation details
citation("recount")[5]

## ----'downloadAll'----------------------------------------------------------------------------------------------------
## Get data.frame with all the URLs
library("recount")
dim(recount_url)

## Explore URLs
head(recount_url$url)

## ----'actuallyDownload', eval = FALSE---------------------------------------------------------------------------------
# ## Download all files (will take a while! It's over 8 TB of data)
# sapply(unique(recount_url$project), download_study, type = "all")

## ----Figure1, out.width="100%", fig.align="center", fig.cap = "SIgn up to SciServer", echo = FALSE--------------------
knitr::include_graphics("https://raw.githubusercontent.com/leekgroup/recount-website/master/sciserver_demo/register.png")

## ----Figure2, out.width="100%", fig.align="center", fig.cap = "Click on SciServer Compute,", echo = FALSE-------------
knitr::include_graphics("https://raw.githubusercontent.com/leekgroup/recount-website/master/sciserver_demo/sciserver-compute.png")

## ----Figure3, out.width="100%", fig.align="center", fig.cap = "Select the appropriate image and load the recount public volume.", echo = FALSE----
knitr::include_graphics("https://raw.githubusercontent.com/leekgroup/recount-website/master/sciserver_demo/new-container.png")

## ----Figure4, out.width="100%", fig.align="center", fig.cap = "Create a R notebook.", echo = FALSE--------------------
knitr::include_graphics("https://raw.githubusercontent.com/leekgroup/recount-website/master/sciserver_demo/new-R.png")

## ----Figure5, out.width="100%", fig.align="center", fig.cap = "Install recount and DESeq2 on SciServer.", echo = FALSE----
knitr::include_graphics("https://raw.githubusercontent.com/leekgroup/recount-website/master/sciserver_demo/demo1.png")

## ----Figure6, out.width="100%", fig.align="center", fig.cap = "recount files are available locally so remmeber to use the local data when working on SciServer.", echo = FALSE----
knitr::include_graphics("https://raw.githubusercontent.com/leekgroup/recount-website/master/sciserver_demo/demo2.png")

## ----'ER-sciserver', eval = FALSE-------------------------------------------------------------------------------------
# ## Expressed-regions SciServer example
# regions <- expressed_regions("SRP009615", "chrY",
#     cutoff = 5L,
#     maxClusterGap = 3000L, outdir = file.path(scipath, "SRP009615")
# )
# regions

## ----createVignette, eval=FALSE---------------------------------------------------------------------------------------
# ## Create the vignette
# library("rmarkdown")
# system.time(render("recount-quickstart.Rmd", "BiocStyle::html_document"))
# 
# ## Extract the R code
# library("knitr")
# knit("recount-quickstart.Rmd", tangle = TRUE)

## ----reproduce1, echo=FALSE-------------------------------------------------------------------------------------------
## Date the vignette was generated
Sys.time()

## ----reproduce2, echo=FALSE-------------------------------------------------------------------------------------------
## Processing time in seconds
totalTime <- diff(c(startTime, Sys.time()))
round(totalTime, digits = 3)

## ----reproduce3, echo=FALSE-------------------------------------------------------------------------------------------
## Session info
library("sessioninfo")
options(width = 120)
session_info()

## ----'datasetup', echo = FALSE, eval = FALSE--------------------------------------------------------------------------
# ## Code for re-creating the data distributed in this package
# 
# ## Genes/exons
# library("recount")
# recount_both <- reproduce_ranges("both", db = "Gencode.v25")
# recount_exons <- recount_both$exon
# save(recount_exons, file = "../data/recount_exons.RData", compress = "xz")
# recount_genes <- recount_both$gene
# save(recount_genes, file = "../data/recount_genes.RData", compress = "xz")
# 
# ## URL table
# load("../../recount-website/fileinfo/upload_table.Rdata")
# recount_url <- upload_table
# ## Create URLs
# is.bw <- grepl("[.]bw$", recount_url$file_name)
# recount_url$url <- NA
# recount_url$url[!is.bw] <- paste0(
#     "http://duffel.rail.bio/recount/",
#     recount_url$project[!is.bw], "/", recount_url$file_name[!is.bw]
# )
# recount_url$url[!is.bw & recount_url$version2] <-
#     paste0(
#         "http://duffel.rail.bio/recount/v2/",
#         recount_url$project[!is.bw & recount_url$version2], "/",
#         recount_url$file_name[!is.bw & recount_url$version2]
#     )
# recount_url$url[is.bw] <- paste0(
#     "http://duffel.rail.bio/recount/",
#     recount_url$project[is.bw], "/bw/", recount_url$file_name[is.bw]
# )
# save(recount_url, file = "../data/recount_url.RData", compress = "xz")
# 
# ## Add FC-RC2 data
# load("../data/recount_url.RData")
# load("../../recount-website/fileinfo/fc_rc_url_table.Rdata")
# recount_url <- rbind(recount_url, fc_rc_url_table)
# rownames(recount_url) <- NULL
# save(recount_url, file = "../data/recount_url.RData", compress = "xz")
# 
# 
# ## Abstract info
# load("../../recount-website/website/meta_web.Rdata")
# recount_abstract <- meta_web[, c("number_samples", "species", "abstract")]
# recount_abstract$project <- gsub('.*">|</a>', "", meta_web$accession)
# Encoding(recount_abstract$abstract) <- "latin1"
# recount_abstract$abstract <- iconv(recount_abstract$abstract, "latin1", "UTF-8")
# save(recount_abstract,
#     file = "../data/recount_abstract.RData",
#     compress = "bzip2"
# )
# 
# ## Example rse_gene file
# system("scp e:/dcl01/leek/data/recount-website/rse/rse_sra/SRP009615/rse_gene.Rdata .")
# load("rse_gene.Rdata")
# rse_gene_SRP009615 <- rse_gene
# save(rse_gene_SRP009615,
#     file = "../data/rse_gene_SRP009615.RData",
#     compress = "xz"
# )
# unlink("rse_gene.Rdata")

## ----'cleanup', echo = FALSE, results = 'hide'------------------------------------------------------------------------
## Clean up
unlink("SRP009615", recursive = TRUE)

## ----vignetteBiblio, results = 'asis', echo = FALSE, warning = FALSE, message = FALSE---------------------------------
## Print bibliography
PrintBibliography(bib, .opts = list(hyperlink = "to.doc", style = "html"))

#' Scale the raw counts provided by the recount project
#'
#' In preparation for a differential expression analysis, you will have to
#' choose how to scale the raw counts provided by the recount project. Note that
#' the raw counts are the sum of the base level coverage so you have to take
#' into account the read length or simply the total coverage for the given
#' sample (default option). You might want to do some further scaling to take
#' into account the gene or exon lengths. If you prefer to calculate read counts
#' without scaling check the function [read_counts].
#'
#' @param rse A [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class]
#' object as downloaded with [download_study].
#' @param by Either `auc` or `mapped_reads`. If set to `auc` it
#' will scale the counts by the total coverage of the sample. That is, the area
#' under the curve (AUC) of the coverage. If set to `mapped_reads` it will
#' scale the counts by the number of mapped reads, whether the library was
#' paired-end or not, and the desired read length (`L`).
#' @param targetSize The target library size in number of single end reads.
#' @param L The target read length. Only used when `by = 'mapped_reads'`
#' since it cancels out in the calculation when using `by = 'auc'`.
#' @param factor_only Whether to only return the numeric scaling factor or
#' to return a [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class]
#' object with the counts scaled. If set to `TRUE`, you have to multiply
#' the sample counts by this scaling factor.
#' @param round Whether to round the counts to integers or not.
#'
#' @return If `factor_only = TRUE` it returns a numeric vector with the
#' scaling factor for each sample. If `factor_only = FALSE` it returns a
#' [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class] object with
#' the counts already scaled.
#'
#' @details Rail-RNA <http://rail.bio> uses soft clipping when aligning
#' which is why we recommed using `by = 'auc'`.
#'
#' If the reads are from a paired-end library, then the `avg_read_length`
#' is the average fragment length. This is taken into account when using
#' `by = 'mapped_reads'`.
#'
#' @seealso [download_study], [read_counts]
#'
#' @importFrom methods is
#'
#' @author Leonardo Collado-Torres
#' @export
#' @import GenomicRanges SummarizedExperiment
#'
#' @examples
#' ## Load an example rse_gene object
#' rse_gene <- rse_gene_SRP009615
#'
#' ## Scale counts
#' rse <- scale_counts(rse_gene)
#'
#' ## Find the project used as an example
#' project_info <- abstract_search("GSE32465")
#'
#' ## See some summary information for this project
#' project_info
#'
#' ## Use the following code to re-download this file
#' \dontrun{
#' ## Download
#' download_study(project_info$project)
#'
#' ## Load file
#' load(file.path(project_info$project, "rse_gene.Rdata"))
#' identical(rse_gene, rse_gene_SRP009615)
#' }
#'
scale_counts <- function(
        rse, by = "auc", targetSize = 4e7, L = 100,
        factor_only = FALSE, round = TRUE) {
    ## Check inputs
    stopifnot(is(rse, "RangedSummarizedExperiment"))
    stopifnot(length(targetSize) == 1)
    stopifnot(is.numeric(targetSize) | is.integer(targetSize))
    stopifnot(length(L) == 1)
    stopifnot(is.numeric(L) | is.integer(L))
    stopifnot(is.character(by))
    stopifnot(by %in% c("auc", "mapped_reads"))
    stopifnot(is.logical(factor_only))
    stopifnot(length(factor_only) == 1)
    stopifnot(is.logical(round))
    stopifnot(length(round) == 1)

    ## Check RSE details
    counts <- SummarizedExperiment::assay(rse, 1)
    stopifnot(ncol(counts) == nrow(SummarizedExperiment::colData(rse)))
    if (by == "auc") {
        stopifnot("auc" %in% colnames(SummarizedExperiment::colData(rse)))
    } else if (by == "mapped_reads") {
        stopifnot(all(c(
            "avg_read_length", "mapped_read_count",
            "paired_end"
        ) %in% colnames(SummarizedExperiment::colData(rse))))
    }

    ## Scale counts
    if (by == "auc") {
        # L cancels out:
        # have to multiply by L to get the desired library size,
        # but then divide by L to take into account the read length since the
        # raw counts are the sum of base-level coverage.
        scaleFactor <- targetSize / SummarizedExperiment::colData(rse)$auc
    } else if (by == "mapped_reads") {
        scaleFactor <- targetSize * L *
            ifelse(SummarizedExperiment::colData(rse)$paired_end, 2, 1)^2 /
            (SummarizedExperiment::colData(rse)$mapped_read_count *
                SummarizedExperiment::colData(rse)$avg_read_length^2)
    }

    if (factor_only) {
        names(scaleFactor) <- rownames(SummarizedExperiment::colData(rse))
        return(scaleFactor)
    } else {
        scaleMat <- matrix(rep(scaleFactor, each = nrow(counts)),
            ncol = ncol(counts)
        )
        scaledCounts <- counts * scaleMat
        if (round) scaledCounts <- round(scaledCounts, 0)
        SummarizedExperiment::assay(rse, 1) <- scaledCounts
        return(rse)
    }
}

#' Reproduce the gene or exons used in the RangedSummarizedExperiment objects
#'
#' This function reproduces the gene or exon level information used for creating
#' the [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class]
#' objects provided by recount. The annotation is based on
#' Gencode v25 with the gene-level
#' information extracted with `genes()` (see
#' [transcripts][GenomicFeatures::transcripts] with default arguments.
#'
#' @param level Either `genes` or `exon`. It specifies whether to
#' return Gene or exon level information as a
#' [GRanges-class][GenomicRanges::GRanges-class] or
#' [GRangesList-class][GenomicRanges::GRangesList-class] object respectively. The gene level
#' information contains the width of the disjoint exons for that given gene
#' which can be used to normalize the counts provided by recount.
#' Can also be `both` in which case a 2 element list with the exon and the
#' gene output is returned.
#' @param db Either `Gencode.v25` (default) or
#' `EnsDb.Hsapiens.v79`. The default option reproduces the annotation
#' used when creating recount. EnsDb.Hsapiens.v79 can be used
#' for an alternative annotation as showcased in the recount vignette.
#'
#' @return Either a [GRanges-class][GenomicRanges::GRanges-class] object like
#' [recount_genes] or a [GRangesList-class][GenomicRanges::GRangesList-class] object
#' like [recount_exons].
#'
#' @details
#'
#' For Gencode.v25, we use the comprehensive gene annotation (regions:
#' `CHR`) from <https://www.gencodegenes.org/releases/25.html>
#' (GRCh38.p7).
#'
#' Note that gene symbols have changed over time. This answer in the
#' Bioconductor support forum details how to obtain the latest gene symbol
#' mappings: <https://support.bioconductor.org/p/126148/#126173>.
#'
#' @author Leonardo Collado-Torres
#' @export
#'
#' @import GenomicRanges
#'
#' @seealso [recount_genes], [recount_exons],
#'     <https://github.com/nellore>,
#'     <https://jhubiostatistics.shinyapps.io/recount/>
#'
#' @examples
#' \dontrun{
#' ## Reproduce gene level information
#' genes <- reproduce_ranges()
#'
#' ## Compare against recount_genes
#' length(genes)
#' length(recount_genes)
#' }
#'
reproduce_ranges <- function(level = "gene", db = "Gencode.v25") {
    ## Check input
    level <- tolower(level)
    stopifnot(level %in% c("gene", "exon", "both"))
    stopifnot(db %in% c("Gencode.v25", "EnsDb.Hsapiens.v79"))



    ## Load required packages
    .load_check(c("GenomicFeatures", "org.Hs.eg.db"))
    if (db == "Gencode.v25") {
        .load_check("txdbmaker")
        temp_gencode <- file.path(
            tempdir(),
            "gencode.v25.annotation.gff3.gz"
        )
        xx <- download_retry(
            url = paste0(
                "ftp://ftp.ebi.ac.uk/pub/databases/gencode/",
                "Gencode_human/release_25/gencode.v25.annotation.gff3.gz"
            ),
            destfile = temp_gencode
        )
        txdb <- txdbmaker::makeTxDbFromGFF(temp_gencode,
            format = "gff3", organism = "Homo sapiens"
        )
    } else if (db == "EnsDb.Hsapiens.v79") {
        .load_check("EnsDb.Hsapiens.v79")
        txdb <- EnsDb.Hsapiens.v79::EnsDb.Hsapiens.v79
    }

    ## Get genes with default option single.strand.genes.only = TRUE
    genes <- GenomicFeatures::genes(txdb)

    ## Get Exons
    exons <- GenomicFeatures::exonsBy(txdb, by = "gene")

    ## Keep only exons for gene ids that we selected previously
    if (!all(names(exons) %in% names(genes))) {
        warning("Dropping exons with gene ids not present in the gene list")
        exons <- exons[names(exons) %in% names(genes)]
    }

    ## Disjoin exons by gene so the exons won't be overlapping each other inside a gene
    exons <- GenomicRanges::disjoin(exons)

    if (level == "exon") {
        return(exons)
    }

    ## For 'gene' or 'both', continue by:
    ## * adding length of disjoint exons by gene
    genes$bp_length <- sum(GenomicRanges::width(exons))

    ## * adding gene symbol
    if (db == "Gencode.v25") {
        gene_info <- AnnotationDbi::mapIds(org.Hs.eg.db::org.Hs.eg.db,
            gsub("\\..*", "", names(genes)), "SYMBOL", "ENSEMBL",
            multiVals = "CharacterList"
        )
    } else if (db == "EnsDb.Hsapiens.v79") {
        gene_info <- AnnotationDbi::mapIds(org.Hs.eg.db::org.Hs.eg.db,
            names(genes), "SYMBOL", "ENSEMBL",
            multiVals = "CharacterList"
        )
    }
    genes$symbol <- gene_info

    if (level == "gene") {
        return(genes)
    } else if (level == "both") {
        return(list("exon" = exons, "gene" = genes))
    }
}

context("Annotation")

library("GenomicRanges")

# ranges_both <- reproduce_ranges("both", db = "Gencode.v25")
# exons <- ranges_both$exon
# genes <- ranges_both$gene
exons_ensembl <- reproduce_ranges("exon", db = "EnsDb.Hsapiens.v79")

test_that("Annotation length", {
    # expect_equal(length(exons), length(genes))
    # expect_gte(length(genes), length(recount_genes))
    # expect_gte(length(exons), length(recount_exons))
    expect_gte(length(exons_ensembl), 65774)
    expect_equal("ENSG00000000003" %in% names(exons_ensembl), TRUE)
    expect_equal(length(genome(recount_exons)), 25)
    expect_equal("GRCh38" %in% genome(exons_ensembl), TRUE)
    expect_equal(colnames(mcols(recount_genes)), c("gene_id", "bp_length", "symbol"))
    # expect_equal(colnames(mcols(genes)), c("gene_id", "bp_length", "symbol"))
})

#' Open a SRA study id in the SRA website
#'
#' Given a SRA study id get the url to browse the study using the SRA website.
#'
#' @param project A character vector with at least one SRA study id.
#' @param browse Whether to open the resulting URL in the browser.
#'
#' @return Returns invisibly the URL for exploring the study in the SRA website.
#'
#' @author Leonardo Collado-Torres
#' @export
#' @seealso [abstract_search]
#'
#' @importFrom utils browseURL
#'
#' @examples
#' ## Find the Geuvadis consortium project
#' id <- abstract_search("Geuvadis consortium", id_only = TRUE)
#' id
#'
#' ## Explore the Geuvadis consortium project
#' url <- browse_study(id)
#'
#' ## See the actual URL
#' url
browse_study <- function(project, browse = interactive()) {
    ## Check inputs
    stopifnot(is.logical(browse) & length(browse == 1))
    stopifnot(is.character(project))

    ## Construct url
    url <- paste0("https://trace.ncbi.nlm.nih.gov/Traces/sra/?study=", project)

    ## Finish
    if (browse) sapply(url, browseURL)
    return(invisible(url))
}

#' Attempt to load the namespace of one or more packages
#'
#' This function uses requireNamespace to try to load one or more packages.
#' If a package is missing, it will suggest how to install it via Bioconductor
#' before quitting.
#'
#' @param pkg A character vector with the names of the packages to check.
#'
#' @details Taken from the regionReport package
#' Updated after feedback from Marcel Ramos at
#' <https://github.com/leekgroup/recount/issues/14>
#'
#' @author Leonardo Collado-Torres
#' @noRd
#'

.load_check <- function(pkg) {
    ## Check the input
    stopifnot(is.character(pkg))
    ## Try to load the packages
    works <- sapply(pkg, requireNamespace, quietly = TRUE)

    ## If some failed, then give a useful error before quitting
    if (any(!works)) {
        x <- pkg[!works]
        stop(paste0(
            Sys.time(),
            " Package",
            ifelse(length(x) > 1, "s", ""),
            " ",
            paste(x, collapse = ", "),
            " ",
            ifelse(length(x) > 1, "are", "is"),
            " missing. Install ",
            ifelse(length(x) > 1, "them", "it"),
            " with BiocManager::install(",
            ifelse(length(x) > 1, "c(", ""),
            '"',
            paste(x, collapse = '", "'),
            '")',
            ifelse(length(x) > 1, ")", "")
        ))
    }
    return(invisible(NULL))
}

#' Add predicted phenotypes to a recount rse object
#'
#' Shannon Ellis et al (2017) predicted phenotypes based on expression data for
#' the samples in the recount2 project. Using this function you can add the
#' predictions to a
#' [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class] object
#' to the `colData()` slot.
#'
#' @param rse A [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class]
#' object as downloaded with [download_study]. If this argument is
#' not specified, the function will return the full predictions table.
#' @param is_tcga Set to `TRUE` only when `rse` is from TCGA.
#' Otherwise set to `FALSE` (default).
#' @param version The version number for the predicted phenotypes data. It has
#' to match one of the available numbers at
#' <https://github.com/leekgroup/recount-website/blob/master/predictions/>.
#' Feel free to check if there is a newer version than the default. The version
#' used is printed as part of the file name.
#' @param verbose If `TRUE` it will print a message of where the
#' predictions file is being downloaded to.
#'
#' @return A [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class]
#' object with the prediction columns appended to the `colData()` slot.
#' The predicted phenotypes are:
#' \describe{
#' \item{sex }{ male or female,}
#' \item{samplesource }{ cell_line or tissue,}
#' \item{tissue }{ tissue predicted based off of 30 tissues in GTEx,}
#' \item{sequencingstrategy }{ single or paired end sequencing.}
#' }
#' For each of the predicted phenotypes there are several columns as described
#' next:
#' \describe{
#' \item{reported_phenotype }{ `NA` when not available,}
#' \item{predicted_phenotype }{ `NA` when we did not predict, "Unassigned"
#' when prediction was ambiguous,}
#' \item{accuracy_phenotype }{ accuracy is assigned per dataset based on
#' comparison to samples for which we had reported phenotype information so
#' there are three distinct values per predictor (GTEx, TCGA, SRA) across all
#' studies.}
#' }
#'
#' @details If you use these predicted phenotypes please cite the Ellis et al
#' bioRxiv pre-print available at
#' https://www.biorxiv.org/content/early/2017/06/03/145656. See citation
#' details with citation('recount').
#'
#' @references
#' Ellis et al, bioRxiv, 2017.
#' https://www.biorxiv.org/content/early/2017/06/03/145656
#'
#' @author Leonardo Collado-Torres
#' @export
#'
#' @import downloader
#' @import SummarizedExperiment
#'
#' @examples
#'
#' ## Add the predictions to an example rse_gene object
#' rse_gene <- add_predictions(rse_gene_SRP009615)
#'
#' ## Explore the predictions
#' colData(rse_gene)
#'
#' ## Download all the latest predictions
#' PredictedPhenotypes <- add_predictions()
add_predictions <- function(
        rse, is_tcga = FALSE, version = "latest",
        verbose = TRUE) {
    ## For a NOTE in R CMD check
    PredictedPhenotypes <- NULL
    if (version == "latest") {
        version <- tryCatch(
            suppressWarnings(readLines(
                "https://raw.githubusercontent.com/leekgroup/recount-website/master/predictions/latestVersion.txt"
            ))[1],
            error = function(e) {
                print(e)
                v <- "0.0.05"
                message(paste(Sys.time(), "Failed to check the latest version, using version", v))
                return(v)
            }
        )
    }

    ## Download file
    predfile <- paste0("PredictedPhenotypes_v", version, ".rda")
    url <- paste0(
        "https://github.com/leekgroup/recount-website/blob/master/predictions/",
        predfile, "?raw=true"
    )
    destfile <- file.path(tempdir(), predfile)

    if (verbose) message(paste(Sys.time(), "downloading the predictions to", destfile))
    download_retry(
        url = url,
        destfile = destfile,
        mode = "wb"
    )
    load(destfile, verbose = verbose)
    if (missing(rse)) {
        return(PredictedPhenotypes)
    }

    if (is_tcga) {
        map <- match(colData(rse)$gdc_file_id, PredictedPhenotypes$sample_id)
    } else {
        map <- match(colData(rse)$run, PredictedPhenotypes$sample_id)
    }
    colData(rse) <- cbind(colData(rse), PredictedPhenotypes[
        map,
        !colnames(PredictedPhenotypes) %in% c("sample_id", "dataset")
    ])
    return(rse)
}

#' Gene annotation used in recount
#'
#' Gene annotation extracted from Gencode v25 (GRCh38.p7) used in recount.
#' Data extraced on October 12th, 2017. The symbols were updated compared
#' to the version from January 17th, 2017.
#' It includes the sum of the width of the disjoint exons which can be
#' used for normalizing the counts provided in the
#' [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class] objects.
#'
#' @name recount_genes
#' @docType data
#' @format A [GRanges-class][GenomicRanges::GRanges-class] with one range per gene. The
#' names match their Gencode v25 ids. The [GRanges-class][GenomicRanges::GRanges-class]
#' has three metadata columns.
#' \describe{
#'     \item{gene_id }{  the Gencode v25 ids, identical to the names.}
#'     \item{bp_length }{ the sum of the width of the disjoint exons for that
#'     given gene.}
#'     \item{symbol }{ a CharacterList with the corresponding gene symbols.}
#' }
#' @keywords datasets
#' @seealso [reproduce_ranges], [recount_exons]
#' @references <https://jhubiostatistics.shinyapps.io/recount/>
NULL

#' RangedSummarizedExperiment at the gene level for study SRP009615
#'
#' [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class] at the gene
#' level for study SRP009615. Used as an example in [scale_counts].
#' Matches the version2 file
#' (details at <https://jhubiostatistics.shinyapps.io/recount/>) under the
#' documentation tab.
#'
#' @name rse_gene_SRP009615
#' @docType data
#' @format A [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class] as
#' created by the recount project for study with SRA id (accession number)
#' SRP009615.
#'
#' @keywords datasets
#' @seealso [scale_counts], [download_study]
#' @references <https://jhubiostatistics.shinyapps.io/recount/>
NULL

#' Query Snaptron to get data from exon-exon junctions present in Intropolis
#'
#' This function uses the Snaptron API to query specific exon-exon junctions
#' that are available via Intropolis as described in the vignette.
#'
#' @param junctions A [GRanges-class][GenomicRanges::GRanges-class] object with the
#' exon-exon junctions of interest. The chromosome names should be in UCSC
#' format, such as 'chr1'. The strand information is ignored in the query.
#' @param version Either `srav1`, `srav2`, `gtex` or
#' `tcga`. SRA Version 1 of Intropolis has the
#' exon-exon junctions from about 20 thousand RNA-seq samples in hg19
#' coordinates. SRA Version 2 has the data from about 50 thousand RNA-seq
#' samples aligned to hg38. GTEx has about 30 million junctions from about 10
#' thousand samples from the GTEx consortium on hg38 coordinates. Finally,
#' TCGA has about 36 million junctions from about 11 thousand samples
#' from the TCGA consortium on hg38 coordinates.
#' @param verbose If `TRUE` status updates will be printed.
#' @param async Defaults to `TRUE` but in some situations it might be
#' preferrable to set it to `FALSE`. This argument gets passed to
#' [getURL][RCurl::getURL]. Check <https://github.com/ChristopherWilks/snaptron/issues/11>
#' for more details.
#'
#' @return A [GRanges-class][GenomicRanges::GRanges-class] object with the results from
#' the Snaptron query. For information on the different columns please see
#' <http://snaptron.cs.jhu.edu>.
#'
#' @references Please cite <http://snaptron.cs.jhu.edu>
#' if you use this function as Snaptron is a separate project from recount.
#' Thank you!
#'
#' @author Leonardo Collado-Torres
#' @export
#'
#' @import GenomicRanges
#' @import RCurl
#'
#' @examples
#'
#' library("GenomicRanges")
#' ## Define some exon-exon junctions (hg19 coordinates)
#' junctions <- GRanges(seqnames = "chr2", IRanges(
#'     start = c(28971710:28971712, 29555081:29555083, 29754982:29754984),
#'     end = c(29462417:29462419, 29923338:29923340, 29917714:29917716)
#' ))
#'
#' ## Check against Snaptron SRA version 1 (hg19 coordinates)
#' snaptron_query(junctions)
#' \dontrun{
#' ## Check another set of junctions against SRA version 2 (more data, hg38
#' ## coordinates)
#' junctions_v2 <- GRanges(seqnames = "chr2", IRanges(
#'     start = 29532116:29532118, end = 29694848:29694850
#' ))
#' snaptron_query(junctions_v2, version = "srav2")
#'
#' ## Check these junctions in GTEx and TCGA data
#' snaptron_query(junctions_v2, version = "gtex")
#' snaptron_query(junctions_v2, version = "tcga")
#' }
#'
snaptron_query <- function(junctions, version = "srav1", verbose = TRUE, async = TRUE) {
    ## Check input
    stopifnot(is(junctions, "GRanges"))
    stopifnot(all(grepl("chr", seqlevels(junctions))))
    version <- tolower(version)
    stopifnot(version %in% c("srav1", "srav2", "gtex", "tcga"))

    ## Build query URLs
    urls <- paste0(
        "http://snaptron.cs.jhu.edu/", version,
        "/snaptron?regions=", seqnames(junctions), ":", start(junctions), "-",
        end(junctions), "&exact=1&header=0"
    )

    ## Get results
    if (verbose) message(paste(Sys.time(), "querying Snaptron"))
    query_res <- getURL(urls, async = async)

    ## Split by line
    if (verbose) message(paste(Sys.time(), "processing results"))
    query_split <- strsplit(query_res, "\n")
    names(query_split) <- NULL

    ## Are there any valid ones?
    valid <- which(elementNROWS(query_split) > 0)
    if (length(valid) == 0) {
        message(paste(
            Sys.time(),
            "found no exon-exon junctions in Intropolis version", version,
            "matching your query: this version uses",
            ifelse(version == "srav1", "hg19", "hg38"), "coordinates."
        ))
        return(NULL)
    }

    ## Extract information
    if (verbose) message(paste(Sys.time(), "extracting information"))
    info <- lapply(query_split[valid], function(jxs) {
        matrix(strsplit(jxs, "\t")[[1]], ncol = 18)
    })

    ## Build result
    res <- do.call(rbind, info)
    colnames(res) <- c(
        "type", "snaptron_id", "chromosome", "start", "end",
        "length", "strand", "annotated", "left_motif", "right_motif",
        "left_annotated", "right_annotated", "samples",
        "samples_count", "coverage_sum", "coverage_avg", "coverage_median",
        "source_dataset_id"
    )

    ## Helper function for some special variables
    to_chr_list <- function(x) {
        r <- strsplit(x, ",")
        i <- which(sapply(r, function(y) {
            y[[1]] == "0"
        }))
        if (length(i) > 0) r[i] <- NA
        return(CharacterList(r))
    }

    ## Build into a GRanges object
    result <- GRanges(
        seqnames = res[, "chromosome"],
        IRanges(as.numeric(res[, "start"]), as.numeric(res[, "end"])),
        strand = res[, "strand"]
    )

    ## Add other variables
    result$type <- as.factor(res[, "type"])
    result$snaptron_id <- as.integer(res[, "snaptron_id"])
    result$annotated <- to_chr_list(res[, "annotated"])
    result$left_motif <- res[, "left_motif"]
    result$right_motif <- res[, "right_motif"]
    result$left_annotated <- to_chr_list(res[, "left_annotated"])
    result$right_annotated <- to_chr_list(res[, "right_annotated"])

    ## Remove leading comma
    res[, "samples"] <- gsub("^,", "", res[, "samples"])
    ## Split sample ids from the read coverage by sample
    sample_info <- IntegerList(strsplit(res[, "samples"], ":|,"))
    sample_idx <- LogicalList(lapply(elementNROWS(sample_info), function(y) {
        rep(c(TRUE, FALSE), y / 2)
    }))
    result$samples <- sample_info[sample_idx]
    result$read_coverage_by_sample <- sample_info[!sample_idx]

    ## Continue with other variables
    result$samples_count <- as.integer(res[, "samples_count"])
    result$coverage_sum <- as.integer(res[, "coverage_sum"])
    result$coverage_avg <- as.numeric(res[, "coverage_avg"])
    result$coverage_median <- as.numeric(res[, "coverage_median"])
    result$source_dataset_id <- as.integer(res[, "source_dataset_id"])

    ## Finish
    return(result)
}

#' Summary information at the project level for the recount project
#'
#' A data.frame with summary information at the project level for the studies
#' analyzed in the recount project.
#'
#' @name recount_abstract
#' @docType data
#' @format A data.frame with 4 columns.
#' \describe{
#'     \item{number_samples }{ the number of samples in the study,}
#'     \item{species }{ the species of the study,}
#'     \item{abstract }{ the abstract text as provided by the SRAdb Bioconductor
#'     package,}
#'     \item{project }{ the SRA project id.}
#' }
#'
#' @keywords datasets
#' @seealso [download_study]
#' @references <https://jhubiostatistics.shinyapps.io/recount/>
NULL

#' Retry multiple times to download a file
#'
#' This function is based on the Bioconductor guidelines for querying data from
#' the web at <http://bioconductor.org/developers/how-to/web-query/>. It
#' will run [download][downloader::download] a set of `N.TRIES` times before
#' giving up. We implemented this function to reduce the number of Bioconductor
#' build errors due to the occassional errors from our data hosting server.
#'
#' @param url The URL to download. Passed to [download][downloader::download].
#' @param destfile The destination file. Defaults to the base name of the URL.
#' Passed to [download][downloader::download].
#' @param mode Mode for writing the file. The default `wb` is used for
#' binary files. This value is passed to [download][downloader::download] which
#' passes it to [download.file][utils::download.file].
#' @param N.TRIES The number of download attempts before giving up; default: 3.
#' Should be an integer of length one with a value greater than 0.
#' @param ... Additional arguments passed to [download][downloader::download].
#'
#' @return An invisible integer code as specified in
#' [download.file][utils::download.file].
#' @export
#'
#' @examples
#'
#' ## Download the first files_info.tsv file (version 1)
#' download_retry(
#'     recount_url$url[which(recount_url$file_name == "files_info.tsv")[1]]
#' )
download_retry <- function(
        url, destfile = basename(url), mode = "wb",
        N.TRIES = 3L, ...) {
    ## Based on http://bioconductor.org/developers/how-to/web-query/
    ## and downloader::download()

    N.TRIES <- as.integer(N.TRIES)
    stopifnot(length(N.TRIES) == 1L, !is.na(N.TRIES))
    stopifnot(N.TRIES > 0L)

    while (N.TRIES > 0L) {
        result <- tryCatch(downloader::download(
            url = url, destfile = destfile, mode = mode, ...
        ), error = identity)
        if (!inherits(result, "error")) {
            break
        }
        ## Wait between 0 and 2 seconds between retries
        Sys.sleep(runif(n = 1, min = 2, max = 5))
        N.TRIES <- N.TRIES - 1L
    }

    if (N.TRIES == 0L) {
        stop(
            "'download_retry()' failed:",
            "\n  URL: ", url,
            "\n  error: ", conditionMessage(result)
        )
    }

    invisible(result)
}

test_that("Downloading with retries", {
    expect_error(download_retry("http://duffel.rail.bio/recount/v2/DRP000366/files_info.tsv_fake", N.TRIES = 1))
    expect_error(download_retry("http://duffel.rail.bio/recount/v2/DRP000366/files_info.tsv", N.TRIES = 0))
    expect_error(download_retry("http://duffel.rail.bio/recount/v2/DRP000366/files_info.tsv", N.TRIES = -1L))
})

#' Build a data.frame from GEO's charactersitics for a given sample
#'
#' This function builds a data.frame from the GEO characteristics extracted
#' for a given sample. The names of the of columns correspond to the field
#' names. For a given SRA project, this information can be combined for all
#' samples as shown in the examples section.
#'
#' @param pheno A [DataFrame-class][S4Vectors::DataFrame-class] as created by
#' [geo_info].
#'
#' @return A 1 row data.frame with the characteristic fields as column names
#' and the values as the entries on the first row. If the authors of the study
#' used the same names for all samples, you can then combine them using
#' [rbind][base::cbind].
#'
#' @author Leonardo Collado-Torres
#' @export
#'
#' @import S4Vectors
#'
#' @examples
#'
#' ## Load required library
#' library("SummarizedExperiment")
#'
#' ## Get the GEO accession ids
#' # geoids <- sapply(colData(rse_gene_SRP009615)$run[1:2], find_geo)
#' ## The previous example code works nearly all the time but it
#' ## can occassionally fail depending on how rentrez is doing.
#' ## This code makes sure that the example code runs.
#' geoids <- tryCatch(
#'     sapply(colData(rse_gene_SRP009615)$run[1:2], find_geo),
#'     error = function(e) {
#'         c(
#'             "SRR387777" = "GSM836270",
#'             "SRR387778" = "GSM836271"
#'         )
#'     }
#' )
#'
#' ## Get the data from GEO
#' geodata <- do.call(rbind, sapply(geoids, geo_info))
#'
#' ## Add characteristics in a way that we can access easily later on
#' geodata <- cbind(geodata, geo_characteristics(geodata))
#'
#' ## Explore the original characteristics and the result from
#' ## geo_characteristics()
#' geodata[, c("characteristics", "cells", "shrna.expression", "treatment")]
geo_characteristics <- function(pheno) {
    ## Check inputs
    stopifnot("characteristics" %in% colnames(pheno))

    if (is.character(pheno$characteristics)) {
        ## Solves https://support.bioconductor.org/p/116480/
        pheno$characteristics <- IRanges::CharacterList(
            lapply(lapply(pheno$characteristics, str2lang), eval)
        )
    }

    ## Separate information
    result <- lapply(pheno$characteristics, function(sampleinfo) {
        ## Check if there are colons
        if (any(!as.vector(sapply(sampleinfo, grepl, pattern = ":")))) {
            res <- data.frame(
                "characteristics" = paste(unlist(sampleinfo),
                    collapse = ", "
                ),
                stringsAsFactors = FALSE
            )
            return(res)
        }

        info <- strsplit(sampleinfo, ": ")

        ## Get variable names
        varNames <- sapply(info, "[[", 1)
        varNames <- make.names(tolower(varNames))

        ## Construct result
        res <- matrix(sapply(info, "[[", 2), nrow = 1)
        colnames(res) <- varNames
        res <- data.frame(res, stringsAsFactors = FALSE)

        ## Finish
        return(res)
    })

    ## Finish
    result <- do.call(rbind, result)
    return(result)
}

#' This function downloads the metadata for all projects.
#'
#' Download the metadata from all the projects. This can be useful for finding
#' samples of interests across all projects.
#'
#' @param subset Either `sra`,  `gtex` or `tcga`. Specifies
#' which metadata file to download.
#' @param verbose If `TRUE` it will print a message of where the file is
#' being downloaded to.
#'
#' @return A [DataFrame-class][S4Vectors::DataFrame-class] object with the phenotype
#' metadata.
#'
#' @author Leonardo Collado-Torres
#' @export
#'
#' @import downloader
#'
#' @examples
#'
#' metadata <- all_metadata()
#' @details Note that for `subset = 'gtex'`, there are more variables than
#' the ones we have for 'sra'. This information corresponds to file
#' GTEx_Data_V6_Annotations_SampleAttributesDS.txt available at
#' <http://www.gtexportal.org/home/datasets>. There you can find the
#' information describing these variables.
#'
#' For TCGA we acquired metadata information from 3 different sources:
#' - GDC: via a json query
#' - CGC: via json queries and a custom script to merge the tables
#' - TCGAbiolinks: we used to to parse GDC's XML files
#' For more information, check <https://github.com/leekgroup/recount-website/tree/master/metadata/tcga_prep>.
#'

all_metadata <- function(subset = "sra", verbose = TRUE) {
    ## For R CMD check
    metadata_clean <- NULL

    ## check inputs
    subset <- tolower(subset)
    stopifnot(subset %in% c("sra", "gtex", "tcga"))
    stopifnot(length(subset) == 1)

    ## Download file
    metafile <- paste0("metadata_clean_", subset, ".Rdata")
    url <- paste0(
        "https://github.com/leekgroup/recount-website/blob/master/metadata/",
        metafile, "?raw=true"
    )
    destfile <- file.path(tempdir(), metafile)

    if (verbose) message(paste(Sys.time(), "downloading the metadata to", destfile))
    download_retry(
        url = url,
        destfile = destfile,
        mode = "wb"
    )
    load(destfile)
    return(metadata_clean)
}

#' Exon annotation used in recount
#'
#' Exon annotation extracted from Gencode v25 (GRCh38.p7) used in recount.
#' Data extraced on October 12th, 2017.
#'
#' @name recount_exons
#' @docType data
#' @format A [GRangesList-class][GenomicRanges::GRangesList-class] with one element per
#' gene. The names match the gene Gencode v25 ids.
#'
#' @keywords datasets
#' @seealso [reproduce_ranges], [recount_genes]
#' @references <https://jhubiostatistics.shinyapps.io/recount/>
NULL

#' Identify expressed regions from the mean coverage for a given SRA project
#'
#' This function uses the pre-computed mean coverage for a given SRA project to
#' identify the expressed regions (ERs) for a given chromosome. It returns a
#' [GRanges-class][GenomicRanges::GRanges-class] object with the expressed regions as
#' defined by [findRegions][derfinder::findRegions].
#'
#' @param project A character vector with one SRA study id.
#' @param chr A character vector with the name of the chromosome.
#' @param cutoff The base-pair level cutoff to use.
#' @param outdir The destination directory for the downloaded file(s) that were
#' previously downloaded with [download_study]. If the files are missing,
#' but `outdir` is specified, they will get downloaded first. By default
#' `outdir` is set to `NULL` which will use the data from the web.
#' We only recommend downloading the full data if you will use it several times.
#' @param maxClusterGap This determines the maximum gap between candidate ERs.
#' @param chrlen The chromosome length in base pairs. If it's `NULL`, the
#' chromosome length is extracted from the Rail-RNA runs GitHub repository.
#' Alternatively check the `SciServer` section on the vignette to see
#' how to access all the recount data via a R Jupyter Notebook.
#' @param verbose If `TRUE` basic status updates will be printed along the
#' way.
#' @param ... Additional arguments passed to [download_study] when
#' `outdir` is specified but the required files are missing.
#'
#' @return A [GRanges-class][GenomicRanges::GRanges-class] object as created by
#' [findRegions][derfinder::findRegions].
#'
#' @author Leonardo Collado-Torres
#' @export
#'
#' @import derfinder GenomicRanges RCurl S4Vectors GenomeInfoDb
#' @importMethodsFrom rtracklayer import import.bw
#' @importFrom RCurl url.exists
#'
#' @seealso [download_study], [findRegions][derfinder::findRegions],
#' [railMatrix][derfinder::railMatrix]
#'
#' @examples
#' ## Define expressed regions for study SRP002001, chrY
#'
#' ## Workaround for https://github.com/lawremi/rtracklayer/issues/83
#' download_study("SRP002001", type = "mean")
#'
#' regions <- expressed_regions("SRP002001", "chrY",
#'     cutoff = 5L,
#'     maxClusterGap = 3000L,
#'     outdir = "SRP002001"
#' )
#'
#' \dontrun{
#' ## Define the regions for multiple chrs
#' regs <- sapply(chrs, expressed_regions, project = "SRP002001", cutoff = 5L)
#'
#' ## You can then combine them into a single GRanges object if you want to
#' library("GenomicRanges")
#' single <- unlist(GRangesList(regs))
#' }
#'
expressed_regions <- function(
        project, chr, cutoff, outdir = NULL,
        maxClusterGap = 300L, chrlen = NULL, verbose = TRUE, ...) {
    ## Check inputs
    stopifnot(is.character(project) & length(project) == 1)
    stopifnot(is.character(chr) & length(chr) == 1)

    ## Use table from the package
    url_table <- recount::recount_url

    ## Subset url data
    url_table <- url_table[url_table$project == project, ]
    if (nrow(url_table) == 0) {
        stop("Invalid 'project' argument. There's no such 'project' in the recount_url data.frame.")
    }

    ## Find chromosome length if absent
    if (is.null(chrlen)) {
        chrinfo <- read.table("https://raw.githubusercontent.com/nellore/runs/master/gtex/hg38.sizes",
            col.names = c("chr", "size"), colClasses = c(
                "character",
                "integer"
            )
        )
        chrlen <- chrinfo$size[chrinfo$chr == chr]
        stopifnot(length(chrlen) == 1)
    }

    ## Check if data is present, otherwise download it
    if (!is.null(outdir)) {
        ## Check mean file
        meanFile <- file.path(outdir, "bw", url_table$file_name[grep(
            "mean",
            url_table$file_name
        )])
        if (!file.exists(meanFile)) {
            download_study(
                project = project, type = "mean", outdir = outdir,
                download = TRUE, ...
            )
        }
    } else {
        meanFile <- download_study(
            project = project, type = "mean",
            download = FALSE
        )
    }

    ## Load coverage
    meanCov <- derfinder::loadCoverage(
        files = meanFile, chr = chr,
        chrlen = chrlen
    )

    ## Find regions
    regs <- derfinder::findRegions(
        position = S4Vectors::Rle(TRUE, length(meanCov$coverage[[1]])),
        fstats = meanCov$coverage[[1]], chr = chr,
        maxClusterGap = maxClusterGap, cutoff = cutoff, verbose = verbose
    )

    ## If there are no regions, return NULL
    if (is.null(regs)) regs <- GenomicRanges::GRanges()

    ## Format appropriately
    names(regs) <- seq_len(length(regs))

    ## Set the length
    GenomeInfoDb::seqlengths(regs) <- length(meanCov$coverage[[1]])

    ## Finish
    return(regs)
}

#' Add additional curated metadata to a recount rse object
#'
#' This function appends sample metadata information to a
#' [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class] from the
#' recount2 project. The sample metadata comes from curated efforts
#' independent from the original recount2 project. Currently the only
#' information comes from the recount_brain project described in more detail
#' at <http://lieberinstitute.github.io/recount-brain/>.
#'
#'
#' @param rse A [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class]
#' object as downloaded with [download_study]. If this argument is
#' not specified, the function will return the raw metadata table.
#' @param source A valid source name. The only supported options at this
#' moment are `recount_brain_v1` and `recount_brain_v2`.
#' @param is_tcga Set to `TRUE` only when `rse` is from TCGA.
#' Otherwise set to `FALSE` (default).
#' @param verbose If `TRUE` it will print a message of where the
#' predictions file is being downloaded to.
#'
#' @return A [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class]
#' object with the sample metadata columns appended to the `colData()`
#' slot.
#'
#' @details
#' For `source = "recount_brain_v1"` and
#' `source = "recount_brain_v2"`, the metadata columns are
#' described at <http://lieberinstitute.github.io/recount-brain/>.
#' Alternatively, you can explore `recount_brain_v2` interactively at
#' <https://jhubiostatistics.shinyapps.io/recount-brain/>.
#'
#' If you use the recount_brain data please cite the Razmara et al.
#' bioRxiv, 2019 <https://www.biorxiv.org/content/10.1101/618025v1>.
#' A bib file is available via citation('recount').
#'
#'
#' @references
#' Razmara et al, bioRxiv, 2019.
#' <https://www.biorxiv.org/content/10.1101/618025v1>
#'
#' @author Leonardo Collado-Torres
#' @export
#'
#' @import downloader
#' @import SummarizedExperiment
#'
#' @examples
#'
#' ## Add the sample metadata to an example rse_gene object
#' rse_gene <- add_metadata(rse_gene_SRP009615, "recount_brain_v2")
#'
#' ## Explore the metadata
#' colData(rse_gene)
#'
#' ## For a list of studies present in recount_brain check
#' ## http://lieberinstitute.github.io/recount-brain/.
#' ## recount_brain_v2 includes GTEx and TCGA brain samples in addition to the
#' ## recount_brain_v1 data, plus ontology information.
#'
#'
#' ## Obtain all the recount_brain_v2 metadata if you want to
#' ## explore the metadata manually
#' recount_brain_v2 <- add_metadata(source = "recount_brain_v2")
add_metadata <- function(
        rse, source = "recount_brain_v2", is_tcga = FALSE,
        verbose = TRUE) {
    stopifnot(length(source) == 1)

    ## For a NOTE in R CMD check
    valid_sources <- data.frame(
        name = c("recount_brain_v1", "recount_brain_v2"),
        url = c(
            "https://github.com/LieberInstitute/recount-brain/blob/master/merged_metadata/recount_brain_v1.Rdata?raw=true", "https://github.com/LieberInstitute/recount-brain/blob/master/cross_studies_metadata/recount_brain_v2.Rdata?raw=true"
        ),
        object = c("recount_brain", "recount_brain"),
        sample_id = c("run_s", "run_s"),
        stringsAsFactors = FALSE
    )

    stopifnot(tolower(source) %in% tolower(valid_sources$name))

    to_use <- valid_sources[tolower(valid_sources$name) == tolower(source), ]

    destfile <- file.path(tempdir(), paste0(to_use$name, ".Rdata"))


    if (verbose) message(paste(Sys.time(), "downloading the", to_use$object, "metadata to", destfile))
    download_retry(
        url = to_use$url,
        destfile = destfile,
        mode = "wb"
    )
    load_meta <- function() {
        load(destfile, verbose = verbose)
        get(to_use$object)
    }
    new_meta <- load_meta()

    if (missing(rse)) {
        return(new_meta)
    }

    if (is_tcga) {
        map <- match(colData(rse)$gdc_file_id, new_meta[, to_use$sample_id])
    } else {
        map <- match(colData(rse)$run, new_meta[, to_use$sample_id])
    }
    if (verbose) {
        message(paste(Sys.time(), "found", sum(!is.na(map)), "out of", length(map), "samples in the", to_use$object, "metadata"))
    }

    ## Make a dummy table with the new metadata to be added
    dummy <- as.data.frame(matrix(NA,
        nrow = ncol(rse),
        ncol = ncol(new_meta) - 1
    ))
    cols_to_drop <- which(colnames(new_meta) == to_use$sample_id)
    colnames(dummy) <- colnames(new_meta)[-cols_to_drop]

    ## In case new data is present
    if (any(!is.na(map))) {
        dummy[!is.na(map), ] <- new_meta[map[!is.na(map)], -cols_to_drop]
    }
    rownames(dummy) <- NULL

    ## Merge new metadata and return the rse
    colData(rse) <- cbind(colData(rse), dummy)
    return(rse)
}

## Disable the tests if the system variable 'R_DISABLE_TESTS' is set to TRUE

flag <- as.logical(Sys.getenv("R_DISABLE_TESTS"))
if (is.na(flag) | flag == FALSE) {
    library("testthat")
    test_check("recount")
}

#' Compute an RPKM matrix based on a RangedSummarizedExperiment object
#'
#' For some analyses you might be interested in transforming the counts into
#' RPKMs which you can do with this function.
#'
#' @param rse A [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class]
#' object as downloaded with [download_study].
#' @param length_var A length 1 character vector with the column name from
#' `rowData(rse)` that has the coding length. For gene level objects
#' from recount this is `bp_length`. If `NULL`, then it will use
#' `width(rowRanges(rse))` which should be used for exon RSEs.
#' @param mapped_var A length 1 character vector with the column name from
#' `colData(rse)` that has the number of reads mapped. For recount RSE
#' object this would be `mapped_read_count`. If `NULL` (default)
#' then it will use the column sums of the counts matrix. The results are
#' different because not all mapped reads are mapped to exonic segments of the
#' genome.
#'
#' @return A matrix with the RPKM values.
#'
#' @details For gene RSE objects, you will want to specify the `length_var`
#' because otherwise you will be adjusting for the total gene length instead
#' of the total exonic sequence length of the gene.
#'
#' @seealso [scale_counts]
#'
#' @author Andrew Jaffe, Leonardo Collado-Torres
#' @export
#' @import SummarizedExperiment
#'
#' @examples
#'
#' ## get RPKM matrix
#' rpkm <- getRPKM(rse_gene_SRP009615)
#'
#' ## You can also get an RPKM matrix after running scale_counts()
#' ## with similar RPKM values
#' rpkm2 <- getRPKM(scale_counts(rse_gene_SRP009615))
#' rpkm3 <- getRPKM(scale_counts(rse_gene_SRP009615, by = "mapped_reads"))
#'
#' summary(rpkm - rpkm2)
#' summary(rpkm - rpkm3)
#' summary(rpkm2 - rpkm3)
getRPKM <- function(rse, length_var = "bp_length", mapped_var = NULL) {
    mapped <- if (!is.null(mapped_var)) colData(rse)[, mapped_var] else colSums(assays(rse)$counts)

    bg <- matrix(mapped, ncol = ncol(rse), nrow = nrow(rse), byrow = TRUE)

    len <- if (!is.null(length_var)) rowData(rse)[, length_var] else width(rowRanges(rse))

    wid <- matrix(len, nrow = nrow(rse), ncol = ncol(rse), byrow = FALSE)
    assays(rse)$counts / (wid / 1000) / (bg / 1e6)
}

#' Find the GEO accession id for a given SRA run
#'
#' Given a SRA run id, this function will retrieve the GEO accession id
#' (starting with GSM) if it's available. Otherwise it will return `NA`.
#'
#' @param run A character vector of length 1 with the SRA run accession id.
#' @param verbose Whether to print a message for the run. Useful when looping
#' over a larger number of SRA run ids.
#' @param sleep The number of seconds (or fraction) to wait before downloading
#' data using [getGEO][GEOquery::getGEO]. This is important if you are looking over
#' `geo_info()` given the constraints published at
#' <https://www.ncbi.nlm.nih.gov/books/NBK25497/>.
#'
#' @return The GEO accession id for the corresponding sample.
#'
#' @author Leonardo Collado-Torres
#' @export
#'
#' @import rentrez
#'
#' @details Although the phenotype information already includes the GEO
#' accession ids, not all projects had GEO entries at the time these tables
#' were created. This function will then be useful to check if there is a GEO
#' accession id for a given sample (run). If there is, you can then retrieve
#' the information using [geo_info].
#'
#' @examples
#' ## Find the GEO accession id for for SRX110461
#' find_geo("SRX110461")
find_geo <- function(run, verbose = FALSE, sleep = 1 / 2) {
    ## Check inputs
    stopifnot(is.character(run) & length(run) == 1)
    if (run == "") {
        return(NA)
    }

    if (verbose) message(paste(Sys.time(), "finding GEO accession id for SRA run", run))
    Sys.sleep(sleep)

    ## Find uid first
    uid <- rentrez::entrez_search("sra", term = run)
    if (length(uid$ids) == 0) {
        return(NA)
    }

    ## Find linking ids
    linking <- rentrez::entrez_link("sra", id = uid$ids, db = "gds")
    if (length(linking$links$sra_gds) == 0) {
        return(NA)
    }

    ## Find GSM
    foundGSM <- FALSE
    for (i in linking$links$sra_gds) {
        gsm <- rentrez::entrez_summary(db = "gds", i)$accession
        if (grepl("GSM", gsm)) {
            foundGSM <- TRUE
            break
        }
    }

    ## Finish
    if (foundGSM) {
        return(gsm)
    } else {
        return(NA)
    }
}

#' Compute read counts
#'
#' As described in the recount workflow, the counts provided by the recount2
#' project are base-pair counts. You can scale them using [scale_counts]
#' or compute the read counts using the area under coverage information (AUC).
#' We use the AUC because Rail-RNA soft clips some reads.
#'
#' @param rse A [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class]
#' object as downloaded with [download_study].
#' @param use_paired_end A logical vector. When `TRUE` it uses the
#' paired-end flag (`colData(rse)$paired_end`) to determine whether
#' the sample is paired-end or not. If it's paired-end, then it divides the
#' counts by 2 to return paired-end read counts instead of fragment counts.
#' When `FALSE`, this information is ignored.
#' @param round Whether to round the counts to integers or not.
#'
#' @return Returns a
#' [RangedSummarizedExperiment-class][SummarizedExperiment::RangedSummarizedExperiment-class] object with
#' the read counts.
#'
#' @seealso [scale_counts]
#'
#' @details
#' Check the recount workflow for details about the counts provided by
#' the recount2 project.
#' Note that this function should not be used after [scale_counts] or it
#' will return non-sensical results.
#'
#' @references
#' Collado-Torres L, Nellore A and Jaffe AE. recount workflow: Accessing over
#' 70,000 human RNA-seq samples with Bioconductor version 1; referees: 1
#' approved, 2 approved with reservations. F1000Research 2017, 6:1558
#' doi: 10.12688/f1000research.12223.1.
#'
#' @author Leonardo Collado-Torres
#' @export
#' @import SummarizedExperiment
#'
#' @examples
#'
#' ## Difference between read counts and reads downloaded by Rail-RNA
#' colSums(assays(read_counts(rse_gene_SRP009615))$counts) / 1e6 -
#'     colData(rse_gene_SRP009615)$reads_downloaded / 1e6
#'
#' ## Paired-end read counts vs fragment counts (single-end counts)
#' download_study("DRP000499")
#' load("DRP000499/rse_gene.Rdata")
#' colSums(assays(read_counts(rse_gene, use_paired_end = FALSE))$counts) /
#'     colSums(assays(read_counts(rse_gene))$counts)
#'
#' ## Difference between paired-end read counts vs paired-end reads downloaded
#' colSums(assays(read_counts(rse_gene))$counts) / 1e6 -
#'     colData(rse_gene)$reads_downloaded / 1e6 / 2
read_counts <- function(rse, use_paired_end = TRUE, round = FALSE) {
    if (use_paired_end) {
        counts <- t(t(assays(rse)$counts) /
            (colData(rse)$auc / (colData(rse)$mapped_read_count /
                ifelse(colData(rse)$paired_end, 2, 1))))
    } else {
        counts <- t(t(assays(rse)$counts) / (colData(rse)$auc /
            colData(rse)$mapped_read_count))
    }
    if (round) counts <- round(counts, 0)
    assays(rse)$counts <- counts
    return(rse)
}

## ----docSetup, bootstrap.show.code = FALSE, dev = device, bootstrap.show.message=FALSE----
## knitrBoostrap and device chunk options
library('knitr')
opts_chunk$set(bootstrap.show.code = FALSE, dev = device, crop = NULL)
if(!outputIsHTML) opts_chunk$set(bootstrap.show.code = FALSE, dev = device, echo = FALSE)

## ----setup, bootstrap.show.message=FALSE--------------------------------------
#### Libraries needed

## Bioconductor
library('DESeq2')
if(isEdgeR) library('edgeR')

## CRAN
library('ggplot2')
if(!is.null(theme)) theme_set(theme)
library('knitr')
if(is.null(colors)) {
    library('RColorBrewer')
}
library('pheatmap')
library('DT')
library('sessioninfo')

#### Code setup

## For ggplot
res.df <- as.data.frame(res)

## Sort results by adjusted p-values
ord <- order(res.df$padj, decreasing = FALSE)
res.df <- res.df[ord, ]
features <- rownames(res.df)
res.df <- cbind(data.frame(Feature = features), res.df)
rownames(res.df) <- NULL

## ----'PCA'--------------------------------------------------------------------
## Transform count data
rld <- tryCatch(rlog(dds), error = function(e) { rlog(dds, fitType = 'mean') })

## Perform PCA analysis and make plot
plotPCA(rld, intgroup = intgroup)

## Get percent of variance explained
data_pca <- plotPCA(rld, intgroup = intgroup, returnData = TRUE)
percentVar <- round(100 * attr(data_pca, "percentVar"))

## ----'sampleDist'-------------------------------------------------------------
## Obtain the sample euclidean distances
sampleDists <- dist(t(assay(rld)))
sampleDistMatrix <- as.matrix(sampleDists)
## Add names based on intgroup
rownames(sampleDistMatrix) <- apply(as.data.frame(colData(rld)[, intgroup]), 1,
    paste, collapse = ' : ')
colnames(sampleDistMatrix) <- NULL

## Define colors to use for the heatmap if none were supplied
if(is.null(colors)) {
    colors <- colorRampPalette( rev(brewer.pal(9, "Blues")) )(255)
}

## Make the heatmap
pheatmap(sampleDistMatrix, clustering_distance_rows = sampleDists,
    clustering_distance_cols = sampleDists, color = colors)

## ----'MAplotalpha'------------------------------------------------------------
## MA plot with alpha used in DESeq2::results()
plotMA(res, alpha = metadata(res)$alpha, main = paste('MA plot with alpha =',
    metadata(res)$alpha))

## ----'MAplotalphaHalf'--------------------------------------------------------
## MA plot with alpha = 1/2 of the alpha used in DESeq2::results()
plotMA(res, alpha = metadata(res)$alpha / 2,
    main = paste('MA plot with alpha =', metadata(res)$alpha / 2))

## ----'MAplotalpha-nBest'------------------------------------------------------
## MA plot with alpha corresponding to the one that gives the nBest features
nBest.actual <- min(nBest, nrow(head(res.df, n = nBest)))
nBest.alpha <- head(res.df, n = nBest)$padj[nBest.actual]
plotMA(res, alpha = nBest.alpha * 1.00000000000001,
    main = paste('MA plot for top', nBest.actual, 'features'))

## ----pvalueHistogram----------------------------------------------------------
## P-value histogram plot
ggplot(res.df[!is.na(res.df$pvalue), ], aes(x = pvalue)) +
    geom_histogram(alpha=.5, position='identity', bins = 50) +
    labs(title='Histogram of unadjusted p-values') +
    xlab('Unadjusted p-values') +
    xlim(c(0, 1.0005))

## ----pvalueSumm---------------------------------------------------------------
## P-value distribution summary
summary(res.df$pvalue)

## ----pvalueTable, results = 'asis'--------------------------------------------
## Split features by different p-value cutoffs
pval_table <- lapply(c(1e-04, 0.001, 0.01, 0.025, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5,
    0.6, 0.7, 0.8, 0.9, 1), function(x) {
    data.frame('Cut' = x, 'Count' = sum(res.df$pvalue <= x, na.rm = TRUE))
})
pval_table <- do.call(rbind, pval_table)
if(outputIsHTML) {
    kable(pval_table, format = 'markdown', align = c('c', 'c'))
} else {
    kable(pval_table)
}

## ----padjHistogram------------------------------------------------------------
## Adjusted p-values histogram plot
ggplot(res.df[!is.na(res.df$padj), ], aes(x = padj)) +
    geom_histogram(alpha=.5, position='identity', bins = 50) +
    labs(title=paste('Histogram of', elementMetadata(res)$description[grep('adjusted', elementMetadata(res)$description)])) +
    xlab('Adjusted p-values') +
    xlim(c(0, 1.0005))

## ----padjSumm-----------------------------------------------------------------
## Adjusted p-values distribution summary
summary(res.df$padj)

## ----padjTable, results = 'asis'----------------------------------------------
## Split features by different adjusted p-value cutoffs
padj_table <- lapply(c(1e-04, 0.001, 0.01, 0.025, 0.05, 0.1, 0.2, 0.3, 0.4, 0.5,
    0.6, 0.7, 0.8, 0.9, 1), function(x) {
    data.frame('Cut' = x, 'Count' = sum(res.df$padj <= x, na.rm = TRUE))
})
padj_table <- do.call(rbind, padj_table)
if(outputIsHTML) {
    kable(padj_table, format = 'markdown', align = c('c', 'c'))
} else {
    kable(padj_table)
}

## ----'topFeatures', results = 'asis'------------------------------------------
## Add search url if appropriate
if(!is.null(searchURL) & outputIsHTML) {
    res.df$Feature <- paste0('<a href="', searchURL, res.df$Feature, '">',
        res.df$Feature, '</a>')
}

for(i in which(colnames(res.df) %in% c('pvalue', 'padj'))) res.df[, i] <- format(res.df[, i], scientific = TRUE)

if(outputIsHTML) {
    datatable(head(res.df, n = nBest), options = list(pagingType='full_numbers', pageLength=10, scrollX='100%'), escape = FALSE, rownames = FALSE) %>% formatRound(which(!colnames(res.df) %in% c('pvalue', 'padj', 'Feature')), digits)
} else {
    res.df_top <- head(res.df, n = 20)
    for(i in which(!colnames(res.df) %in% c('pvalue', 'padj', 'Feature'))) res.df_top[, i] <- round(res.df_top[, i], digits)
    kable(res.df_top)
}

## ----'plotCounts'-------------------------------------------------------------
plotCounts_gg <- function(i, dds, intgroup) {
    group <- if (length(intgroup) == 1) {
        colData(dds)[[intgroup]]
    } else if (length(intgroup) == 2) {
        lvls <- as.vector(t(outer(levels(colData(dds)[[intgroup[1]]]), 
            levels(colData(dds)[[intgroup[2]]]), function(x, 
                y) paste(x, y, sep = " : "))))
        droplevels(factor(apply(as.data.frame(colData(dds)[, 
            intgroup, drop = FALSE]), 1, paste, collapse = " : "), 
            levels = lvls))
    } else {
        factor(apply(as.data.frame(colData(dds)[, intgroup, drop = FALSE]), 
            1, paste, collapse = " : "))
    }
    data <- plotCounts(dds, gene=i, intgroup=intgroup, returnData = TRUE)
    ## Change in version 1.15.3
    ## It might not be necessary to have any of this if else, but I'm not
    ## sure that plotCounts(returnData) will always return the 'group' variable.
    if('group' %in% colnames(data)) {
        data$group <- group
    } else {
        data <- cbind(data, data.frame('group' = group))
    }

    ggplot(data, aes(x = group, y = count)) + geom_point() + ylab('Normalized count') + ggtitle(i) + coord_trans(y = "log10") + theme(axis.text.x = element_text(angle = 90, hjust = 1))
}
for(i in head(features, nBestFeatures)) {
    print(plotCounts_gg(i, dds = dds, intgroup = intgroup))
}

## ----'edgeR-BCV', eval = isEdgeR, echo = isEdgeR & outputIsHTML---------------
# ## Make BCV plot for edgeR results
# plotBCV(dge)

## ----'edgeR-MDS', eval = isEdgeR, echo = isEdgeR & outputIsHTML---------------
# ## Make MDS plot for edgeR results
# plotMDS(dge)

## ----child = customCode, eval = hasCustomCode---------------------------------
# NA

## ----thecall, echo=FALSE------------------------------------------------------
theCall

## ----reproducibility1, echo=FALSE---------------------------------------------
## Date the report was generated
Sys.time()

## ----reproducibility2, echo=FALSE---------------------------------------------
## Processing time in seconds
totalTime <- diff(c(startTime, Sys.time()))
round(totalTime, digits=3)

## ----reproducibility3, echo=FALSE-------------------------------------------------------------------------------------
## Session info
options(width = 120)
session_info()

## ----bibliography, results='asis', echo=FALSE, warning = FALSE--------------------------------------------------------
## Print bibliography
PrintBibliography(bib, .opts = list(hyperlink = "to.doc", style = "html"))

---
title: "recount quick start guide"
author: 
  - name: Leonardo Collado-Torres
    affiliation:
    - &libd Lieber Institute for Brain Development, Johns Hopkins Medical Campus
    - &ccb Center for Computational Biology, Johns Hopkins University
    email: lcolladotor@gmail.com
output: 
  BiocStyle::html_document:
    self_contained: yes
    toc: true
    toc_float: true
    toc_depth: 2
    code_folding: show
date: "`r doc_date()`"
package: "`r pkg_ver('recount')`"
vignette: >
  %\VignetteIndexEntry{recount quick start guide}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}  
---

```{r vignetteSetup, echo=FALSE, message=FALSE, warning = FALSE}
## Track time spent on making the vignette
startTime <- Sys.time()

## Bib setup
library("RefManageR")

## Write bibliography information
bib <- c(
    R = citation(),
    AnnotationDbi = RefManageR::BibEntry(
        bibtype = "manual",
        key = "AnnotationDbi",
        author = "Hervé Pagès and Marc Carlson and Seth Falcon and Nianhua Li",
        title = "AnnotationDbi: Annotation Database Interface",
        year = 2017, doi = "10.18129/B9.bioc.AnnotationDbi"
    ),
    BiocParallel = citation("BiocParallel")[1],
    BiocStyle = citation("BiocStyle")[1],
    derfinder = citation("derfinder")[1],
    DESeq2 = citation("DESeq2")[1],
    sessioninfo = citation("sessioninfo")[1],
    downloader = citation("downloader")[1],
    EnsDb.Hsapiens.v79 = citation("EnsDb.Hsapiens.v79")[1],
    GEOquery = citation("GEOquery")[1],
    GenomeInfoDb = RefManageR::BibEntry(
        bibtype = "manual",
        key = "GenomeInfoDb",
        author = "Sonali Arora and Martin Morgan and Marc Carlson and H. Pagès",
        title = "GenomeInfoDb: Utilities for manipulating chromosome and other 'seqname' identifiers",
        year = 2017, doi = "10.18129/B9.bioc.GenomeInfoDb"
    ),
    GenomicFeatures = citation("GenomicFeatures")[1],
    GenomicRanges = citation("GenomicRanges")[1],
    IRanges = citation("IRanges")[1],
    knitr = citation("knitr")[3],
    org.Hs.eg.db = citation("org.Hs.eg.db")[1],
    RCurl = citation("RCurl")[1],
    recount = citation("recount")[1],
    recountworkflow = citation("recount")[2],
    phenopredict = citation("recount")[3],
    regionReport = citation("regionReport")[1],
    rentrez = citation("rentrez")[1],
    RefManageR = citation("RefManageR")[1],
    rmarkdown = citation("rmarkdown")[1],
    rtracklayer = citation("rtracklayer")[1],
    S4Vectors = RefManageR::BibEntry(
        bibtype = "manual", key = "S4Vectors",
        author = "Hervé Pagès and Michael Lawrence and Patrick Aboyoun",
        title = "S4Vectors: S4 implementation of vector-like and list-like objects",
        year = 2017, doi = "10.18129/B9.bioc.S4Vectors"
    ),
    SummarizedExperiment = RefManageR::BibEntry(
        bibtype = "manual",
        key = "SummarizedExperiment",
        author = "Martin Morgan and Valerie Obenchain and Jim Hester and Hervé Pagès",
        title = "SummarizedExperiment: SummarizedExperiment container",
        year = 2017, doi = "10.18129/B9.bioc.SummarizedExperiment"
    ),
    testthat = citation("testthat")[1]
)
```

# Basics

## Install `r Biocpkg('recount')`

`R` is an open-source statistical environment which can be easily modified to enhance its functionality via packages. `r Biocpkg('recount')` is a `R` package available via the [Bioconductor](http://bioconductor/packages/recount) repository for packages. `R` can be installed on any operating system from [CRAN](https://cran.r-project.org/) after which you can install `r Biocpkg('recount')` by using the following commands in your `R` session:

```{r 'installDer', eval = FALSE}
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}

BiocManager::install("recount")

## Check that you have a valid Bioconductor installation
BiocManager::valid()
```

## Required knowledge

`r Biocpkg('recount')` is based on many other packages and in particular in those that have implemented the infrastructure needed for dealing with RNA-seq data. That is, packages like `r Biocpkg('GenomicFeatures')` and `r Biocpkg('rtracklayer')` that allow you to import the data. A `r Biocpkg('recount')` user is not expected to deal with those packages directly but will need to be familiar with `r Biocpkg('SummarizedExperiment')` to understand the results `r Biocpkg('recount')` generates. It might also prove to be highly beneficial to check the

* `r Biocpkg('DESeq2')` package for performing differential expression analysis with the _RangedSummarizedExperiment_ objects,
* `r Biocpkg('DEFormats')` package for switching the objects to those used by other differential expression packages such as `r Biocpkg('edgeR')`,
* `r Biocpkg('derfinder')` package for performing annotation-agnostic differential expression analysis.

If you are asking yourself the question "Where do I start using Bioconductor?" you might be interested in [this blog post](http://lcolladotor.github.io/2014/10/16/startbioc/#.VkOKbq6rRuU).

## Asking for help

As package developers, we try to explain clearly how to use our packages and in which order to use the functions. But `R` and `Bioconductor` have a steep learning curve so it is critical to learn where to ask for help. The blog post quoted above mentions some but we would like to highlight the [Bioconductor support site](https://support.bioconductor.org/) as the main resource for getting help: remember to use the `recount` tag and check [the older posts](https://support.bioconductor.org/t/recount/). Other alternatives are available such as creating GitHub issues and tweeting. However, please note that if you want to receive help you should adhere to the [posting guidelines](http://www.bioconductor.org/help/support/posting-guide/). It is particularly critical that you provide a small reproducible example and your session information so package developers can track down the source of the error.

## Workflow using `r Biocpkg('recount')`

We have written a workflow on how to use `r Biocpkg('recount')` that explains more details on how to use this package with other Bioconductor packages as well as the details on the actual counts provided by `r Biocpkg('recount')`. Check it at [f1000research.com/articles/6-1558/v1](https://f1000research.com/articles/6-1558/v1) or [bioconductor.org/help/workflows/recountWorkflow/](http://bioconductor.org/help/workflows/recountWorkflow/) `r Citep(bib[['recountworkflow']])`.

## Citing `r Biocpkg('recount')`

We hope that `r Biocpkg('recount')` will be useful for your research. Please use the following information to cite the package and the overall approach. Thank you!

```{r 'citation'}
## Citation info
citation("recount")
```

# Quick start to using to `r Biocpkg('recount')`

Main updates:

* __As of January 30, 2017 the annotation used for the exon and gene counts is Gencode v25.__
* __As of January 12, 2018 transcripts counts are available via recount2 thanks to the work of [Fu et al, bioRxiv, 2018](https://www.biorxiv.org/content/10.1101/247346v2). Disjoint exon counts (version 2) were also released as described in detail in the [recount website](https://jhubiostatistics.shinyapps.io/recount/) documentation tab.__
* __As of April 29, 2019 FANTOM-CAT/recount2 annotation quantifications area available via recount2 thanks to the work by [Imada, Sanchez, et al., bioRxiv, 2019](https://www.biorxiv.org/content/10.1101/659490v1).__


<a href="https://jhubiostatistics.shinyapps.io/recount/"><img src="https://raw.githubusercontent.com/leekgroup/recount-website/master/website/recount.png" align="center" width="600"></a>

Here is a very quick example of how to download a `RangedSummarizedExperiment` object with the gene counts for a 2 groups project (12 samples) with SRA study id [SRP009615](http://trace.ncbi.nlm.nih.gov/Traces/sra/?study=SRP009615) using the `r Biocpkg('recount')` package `r Citep(bib[['recount']])`. The `RangedSummarizedExperiment` object is defined in the `r Biocpkg('SummarizedExperiment')` `r Citep(bib[['SummarizedExperiment']])` package and can be used for differential expression analysis with different packages. Here we show how to use `r Biocpkg('DESeq2')` `r Citep(bib[['DESeq2']])` to perform the differential expresion analysis.

This quick analysis is explained in more detail later on in this document. Further information about the recount project can be found in the [main publication](https://www.nature.com/articles/nbt.3838). Check the [recount website](https://jhubiostatistics.shinyapps.io/recount/) for related publications.
 
```{r 'ultraQuick', eval = FALSE}
## Load library
library("recount")

## Find a project of interest
project_info <- abstract_search("GSE32465")

## Download the gene-level RangedSummarizedExperiment data
download_study(project_info$project)

## Load the data
load(file.path(project_info$project, "rse_gene.Rdata"))

## Browse the project at SRA
browse_study(project_info$project)

## View GEO ids
colData(rse_gene)$geo_accession

## Extract the sample characteristics
geochar <- lapply(split(colData(rse_gene), seq_len(nrow(colData(rse_gene)))), geo_characteristics)

## Note that the information for this study is a little inconsistent, so we
## have to fix it.
geochar <- do.call(rbind, lapply(geochar, function(x) {
    if ("cells" %in% colnames(x)) {
        colnames(x)[colnames(x) == "cells"] <- "cell.line"
        return(x)
    } else {
        return(x)
    }
}))

## We can now define some sample information to use
sample_info <- data.frame(
    run = colData(rse_gene)$run,
    group = ifelse(grepl("uninduced", colData(rse_gene)$title), "uninduced", "induced"),
    gene_target = sapply(colData(rse_gene)$title, function(x) {
        strsplit(strsplit(
            x,
            "targeting "
        )[[1]][2], " gene")[[1]][1]
    }),
    cell.line = geochar$cell.line
)

## Scale counts by taking into account the total coverage per sample
rse <- scale_counts(rse_gene)

## Add sample information for DE analysis
colData(rse)$group <- sample_info$group
colData(rse)$gene_target <- sample_info$gene_target

## Perform differential expression analysis with DESeq2
library("DESeq2")

## Specify design and switch to DESeq2 format
dds <- DESeqDataSet(rse, ~ gene_target + group)

## Perform DE analysis
dds <- DESeq(dds, test = "LRT", reduced = ~gene_target, fitType = "local")
res <- results(dds)

## Explore results
plotMA(res, main = "DESeq2 results for SRP009615")

## Make a report with the results
library("regionReport")
DESeq2Report(dds,
    res = res, project = "SRP009615",
    intgroup = c("group", "gene_target"), outdir = ".",
    output = "SRP009615-results"
)
```

The [recount project](https://jhubiostatistics.shinyapps.io/recount/) also hosts the necessary data to perform annotation-agnostic differential expression analyses with `r Biocpkg('derfinder')` `r Citep(bib[['derfinder']])`. An example analysis would like this:

```{r 'er_analysis', eval = FALSE}
## Define expressed regions for study SRP009615, only for chromosome Y
regions <- expressed_regions("SRP009615", "chrY",
    cutoff = 5L,
    maxClusterGap = 3000L
)

## Compute coverage matrix for study SRP009615, only for chromosome Y
system.time(rse_ER <- coverage_matrix("SRP009615", "chrY", regions))

## Round the coverage matrix to integers
covMat <- round(assays(rse_ER)$counts, 0)

## Get phenotype data for study SRP009615
pheno <- colData(rse_ER)

## Complete the phenotype table with the data we got from GEO
m <- match(pheno$run, sample_info$run)
pheno <- cbind(pheno, sample_info[m, 2:3])

## Build a DESeqDataSet
dds_ers <- DESeqDataSetFromMatrix(
    countData = covMat, colData = pheno,
    design = ~ gene_target + group
)

## Perform differential expression analysis with DESeq2 at the ER-level
dds_ers <- DESeq(dds_ers,
    test = "LRT", reduced = ~gene_target,
    fitType = "local"
)
res_ers <- results(dds_ers)

## Explore results
plotMA(res_ers, main = "DESeq2 results for SRP009615 (ER-level, chrY)")

## Create a more extensive exploratory report
DESeq2Report(dds_ers,
    res = res_ers,
    project = "SRP009615 (ER-level, chrY)",
    intgroup = c("group", "gene_target"), outdir = ".",
    output = "SRP009615-results-ER-level-chrY"
)
```



# Introduction



`r Biocpkg('recount')` is an R package that provides an interface to the [recount project website](https://jhubiostatistics.shinyapps.io/recount/). This package allows you to download the files from the recount project and has helper functions for getting you started with differential expression analyses. This vignette will walk you through an example.


# Sample DE analysis

This is a brief overview of what you can do with `r Biocpkg('recount')`. In this particular example we will download data from the [SRP009615](http://trace.ncbi.nlm.nih.gov/Traces/sra/?study=SRP009615) study which sequenced 12 samples as described in the previous link.

If you don't have `r Biocpkg('recount')` installed, please do so with:

```{r 'install', eval = FALSE}
install.packages("BiocManager")
BiocManager::install("recount")
```

Next we load the required packages. Loading `r Biocpkg('recount')` will load the required dependencies.

```{r 'start', message=FALSE}
## Load recount R package
library("recount")
```

Lets say that we don't know the actual SRA accession number for this study but we do know a particular term which will help us identify it. If that's the case, we can use the `abstract_search()` function to identify the study of interest as shown below.

```{r 'search_abstract'}
## Find a project of interest
project_info <- abstract_search("GSE32465")

## Explore info
project_info
```

## Download data

Now that we have a study that we are interested in, we can download the _RangedSummarizedExperiment_ object (see `r Biocpkg('SummarizedExperiment')`) with the data summarized at the gene level. The function `download_study()` helps us do this. If you are interested on how the annotation was defined, check `reproduce_ranges()` described in the _Annotation_ section further down.

```{r 'download'}
## Download the gene-level RangedSummarizedExperiment data
download_study(project_info$project)

## Load the data
load(file.path(project_info$project, "rse_gene.Rdata"))

## Delete it if you don't need it anymore
unlink(project_info$project, recursive = TRUE)
```

We can explore a bit this _RangedSummarizedExperiment_ as shown below.

```{r 'explore_rse'}
rse_gene

## This is the sample phenotype data provided by the recount project
colData(rse_gene)

## At the gene level, the row data includes the gene Gencode ids, the gene
## symbols and the sum of the disjoint exons widths, which can be used for
## taking into account the gene length.
rowData(rse_gene)

## At the exon level, you can get the gene Gencode ids from the names of:
# rowRanges(rse_exon)
```

## Finding phenotype information

Once we have identified the study of interest, we can use the `browse_study()` function to browse the study at the SRA website.

```{r 'browse'}
## Browse the project at SRA
browse_study(project_info$project)
```



The SRA website includes an _Experiments_ link which further describes each of the samples. From the information available for [SRP009615 at NCBI](http://www.ncbi.nlm.nih.gov/sra/?term=SRP009615) we have some further sample information that we can save for use in our differential expression analysis. We can get some of this information from [GEO](http://www.ncbi.nlm.nih.gov/geo/). The function `find_geo()` finds the [GEO](http://www.ncbi.nlm.nih.gov/geo/) accession id for a given SRA run accession id, which we can then use with `geo_info()` and `geo_characteristics()` to parse this information. The `rse_gene` object already has some of this information.

```{r 'sample_info', warning = FALSE}
## View GEO ids
colData(rse_gene)$geo_accession

## Extract the sample characteristics
geochar <- lapply(split(colData(rse_gene), seq_len(nrow(colData(rse_gene)))), geo_characteristics)

## Note that the information for this study is a little inconsistent, so we
## have to fix it.
geochar <- do.call(rbind, lapply(geochar, function(x) {
    if ("cells" %in% colnames(x)) {
        colnames(x)[colnames(x) == "cells"] <- "cell.line"
        return(x)
    } else {
        return(x)
    }
}))

## We can now define some sample information to use
sample_info <- data.frame(
    run = colData(rse_gene)$run,
    group = ifelse(grepl("uninduced", colData(rse_gene)$title), "uninduced", "induced"),
    gene_target = sapply(colData(rse_gene)$title, function(x) {
        strsplit(strsplit(
            x,
            "targeting "
        )[[1]][2], " gene")[[1]][1]
    }),
    cell.line = geochar$cell.line
)
```

## Predicted phenotype information

Shannon Ellis et at have predicted phenotypic information that can be used with any data from the recount project thanks to `add_predictions()`. Check that function for more details `r Citep(bib[['phenopredict']])`.

## DE analysis

The [recount project](https://jhubiostatistics.shinyapps.io/recount/) records the sum of the base level coverage for each gene (or exon). These raw counts have to be scaled and there are several ways in which you can choose to do so. The function `scale_counts()` helps you scale them in a way that is tailored to [Rail-RNA](http://rail.bio) output. If you prefer read counts without scaling, check the function `read_counts()`. Below we show some of the differences.

```{r 'scale_counts'}
## Scale counts by taking into account the total coverage per sample
rse <- scale_counts(rse_gene)

##### Details about counts #####

## Scale counts to 40 million mapped reads. Not all mapped reads are in exonic
## sequence, so the total is not necessarily 40 million.
colSums(assays(rse)$counts) / 1e6

## Compute read counts
rse_read_counts <- read_counts(rse_gene)
## Difference between read counts and number of reads downloaded by Rail-RNA
colSums(assays(rse_read_counts)$counts) / 1e6 -
    colData(rse_read_counts)$reads_downloaded / 1e6
## Check the help page for read_counts() for more details
```

We are almost ready to perform our differential expression analysis. Lets just add the information we recovered [GEO](http://www.ncbi.nlm.nih.gov/geo/) about these samples.


```{r 'add_sample_info'}
## Add sample information for DE analysis
colData(rse)$group <- sample_info$group
colData(rse)$gene_target <- sample_info$gene_target
```

Now that the _RangedSummarizedExperiment_ is complete, we can use `r Biocpkg('DESeq2')` or another package to perform the differential expression test. Note that you can use `r Biocpkg('DEFormats')` for switching between formats if you want to use another package, like `r Biocpkg('edgeR')`.

In this particular analysis, we'll test whether there is a group difference adjusting for the gene target.

```{r 'de_analysis'}
## Perform differential expression analysis with DESeq2
library("DESeq2")

## Specify design and switch to DESeq2 format
dds <- DESeqDataSet(rse, ~ gene_target + group)

## Perform DE analysis
dds <- DESeq(dds, test = "LRT", reduced = ~gene_target, fitType = "local")
res <- results(dds)
```

We can now use functions from `r Biocpkg('DESeq2')` to explore the results. For more details check the `r Biocpkg('DESeq2')` vignette. For example, we can make a MA plot as shown in Figure \@ref(fig:maplot).

```{r 'maplot', fig.cap = "MA plot for group differences adjusted by gene target for SRP009615 using gene-level data."}
## Explore results
plotMA(res, main = "DESeq2 results for SRP009615")
```

## Report results

We can also use the `r Biocpkg('regionReport')` package to generate interactive HTML reports exploring the `r Biocpkg('DESeq2')` results (or `r Biocpkg('edgeR')` results if you used that package).



```{r 'make_report', eval = FALSE}
## Make a report with the results
library("regionReport")
report <- DESeq2Report(dds,
    res = res, project = "SRP009615",
    intgroup = c("group", "gene_target"), outdir = ".",
    output = "SRP009615-results", nBest = 10, nBestFeatures = 2
)
```

```{r 'make_report_real', echo = FALSE, results = 'hide'}
library("regionReport")

## Load some packages used by regionReport
library("DT")
library("edgeR")
library("ggplot2")
library("RColorBrewer")

## Make it so that the report will be available as a vignette
original <- readLines(system.file("DESeq2Exploration", "DESeq2Exploration.Rmd",
    package = "regionReport"
))
vignetteInfo <- c(
    "vignette: >",
    "  %\\VignetteEngine{knitr::rmarkdown}",
    "  %\\VignetteIndexEntry{Basic DESeq2 results exploration}",
    "  %\\VignetteEncoding{UTF-8}"
)
new <- c(original[1:16], vignetteInfo, original[17:length(original)])
writeLines(new, "SRP009615-results-template.Rmd")

## Now create the report
report <- DESeq2Report(dds,
    res = res, project = "SRP009615",
    intgroup = c("group", "gene_target"), outdir = ".",
    output = "SRP009615-results", device = "png", template = "SRP009615-results-template.Rmd", nBest = 10, nBestFeatures = 2
)

## Clean up
file.remove("SRP009615-results-template.Rmd")
```

You can view the final report [here](SRP009615-results.html).


## Finding gene names

The gene Gencode ids are included in several objects in `recount`. With these ids you can find the gene symbols, gene names and other information. The `r Biocpkg('org.Hs.eg.db')` is useful for finding this information and the following code shows how to get the gene names and symbols.

```{r 'geneSymbols'}
## Load required library
library("org.Hs.eg.db")

## Extract Gencode gene ids
gencode <- gsub("\\..*", "", names(recount_genes))

## Find the gene information we are interested in
gene_info <- select(org.Hs.eg.db, gencode, c(
    "ENTREZID", "GENENAME", "SYMBOL",
    "ENSEMBL"
), "ENSEMBL")

## Explore part of the results
dim(gene_info)
head(gene_info)
```


# Sample `r Biocpkg('derfinder')` analysis

The [recount project](https://jhubiostatistics.shinyapps.io/recount/) also hosts for each project sample coverage [bigWig files](https://genome.ucsc.edu/goldenpath/help/bigWig.html) created by [Rail-RNA](http://rail.bio) and a mean coverage bigWig file. For the mean coverage bigWig file, all samples were normalized to libraries of 40 million reads, each a 100 base-pairs long. `r Biocpkg('recount')` can be used along with `r Biocpkg('derfinder')` `r Citep(bib[['derfinder']])` to identify expressed regions from the data. This type of analysis is annotation-agnostic which can be advantageous. The following subsections illustrate this type of analysis.

## Define expressed regions

For an annotation-agnostic differential expression analysis, we first need to define the regions of interest. With `r Biocpkg('recount')` we can do so using the `expressed_regions()` function as shown below for the same study we studied earlier.

```{r 'download_bigwigs'}
## Normally, one can use rtracklayer::import() to access remote parts of BigWig
## files without having to download the complete files. However, as of
## 2024-05-20 this doesn't seem to be working well. So this is a workaround to
## issue https://github.com/lawremi/rtracklayer/issues/83
download_study("SRP009615", type = "mean")
download_study("SRP009615", type = "samples")
```

```{r 'define_ers'}
## Define expressed regions for study SRP009615, only for chromosome Y
regions <- expressed_regions("SRP009615", "chrY",
    cutoff = 5L,
    maxClusterGap = 3000L, outdir = "SRP009615"
)

## Briefly explore the resulting regions
regions
```

Once the regions have been defined, you can export them into a BED file using `r Biocpkg('rtracklayer')` or other file formats.

## Compute coverage matrix

Having defined the expressed regions, we can now compute a coverage matrix for these regions. We can do so using the function `coverage_matrix()` from `r Biocpkg('recount')` as shown below. It returns a _RangedSummarizedExperiment_ object.

```{r 'compute_covMat'}
## Compute coverage matrix for study SRP009615, only for chromosome Y
system.time(rse_ER <- coverage_matrix("SRP009615", "chrY", regions, outdir = "SRP009615"))

## Explore the RSE a bit
dim(rse_ER)
rse_ER
```

The resulting count matrix has one row per region and one column per sample. The counts correspond to the number (or fraction) of reads overlapping the regions and are scaled by default to a library size of 40 million reads. For some differential expression methods, you might have to round this matrix into integers. We'll use `r Biocpkg('DESeq2')` to identify which expressed regions are differentially expressed.

```{r 'to_integer'}
## Round the coverage matrix to integers
covMat <- round(assays(rse_ER)$counts, 0)
```

You can use `scale = FALSE` if you want the raw base-pair coverage counts and then scale them with `scale_counts()`. If you want integer counts, use `round = TRUE`.

## Construct a `DESeqDataSet` object

We first need to get some phenotype information for these samples similar to the first analysis we did. We can get this data using `download_study()`.

```{r 'phenoData'}
## Get phenotype data for study SRP009615
pheno <- colData(rse_ER)

## Complete the phenotype table with the data we got from GEO
m <- match(pheno$run, sample_info$run)
pheno <- cbind(pheno, sample_info[m, 2:3])

## Explore the phenotype data a little bit
head(pheno)
```

Now that we have the necessary data, we can construct a `DESeqDataSet` object using the function `DESeqDataSetFromMatrix()` from `r Biocpkg('DESeq2')`.

```{r 'ers_dds'}
## Build a DESeqDataSet
dds_ers <- DESeqDataSetFromMatrix(
    countData = covMat, colData = pheno,
    design = ~ gene_target + group
)
```


## `r Biocpkg('DESeq2')` analysis

With the `DESeqDataSet` object in place, we can then use the function `DESeq()` from `r Biocpkg('DESeq2')` to perform the differential expression analysis (between groups) as shown below.

```{r 'de_analysis_ers'}
## Perform differential expression analysis with DESeq2 at the ER-level
dds_ers <- DESeq(dds_ers,
    test = "LRT", reduced = ~gene_target,
    fitType = "local"
)
res_ers <- results(dds_ers)
```

We can then visually explore the results as shown in Figure \@ref(fig:maploters), like we did before in Figure \@ref(fig:maplot).

```{r 'maploters', fig.cap = "MA plot for group differences adjusted by gene target for SRP009615 using ER-level data (just chrY)."}
## Explore results
plotMA(res_ers, main = "DESeq2 results for SRP009615 (ER-level, chrY)")
```

We can also use `r Biocpkg('regionReport')` to create a more extensive exploratory report.

```{r 'report2', eval = FALSE}
## Create the report
report2 <- DESeq2Report(dds_ers,
    res = res_ers,
    project = "SRP009615 (ER-level, chrY)",
    intgroup = c("group", "gene_target"), outdir = ".",
    output = "SRP009615-results-ER-level-chrY"
)
```

# Annotation used

This section describes the annotation used in `recount`.

We used the comprehensive gene annotation for (`CHR` regions) from [Gencode v25](https://www.gencodegenes.org/releases/25.html) (GRCh38.p7) to get the list of genes. Specifically [this GFF3](ftp://ftp.ebi.ac.uk/pub/databases/gencode/Gencode_human/release_25/gencode.v25.annotation.gff3.gz) file. We then used the `r Biocpkg('org.Hs.eg.db')` package to get the gene symbols. For each gene, we also counted the total length of exonic base pairs for that given gene and stored this information in the `bp_length` column. You can see below this information for the `rse_gene` object included in `r Biocpkg('recount')`. The `rownames()` are the Gencode `gene_id`.

```{r, 'gene_annotation'}
## Gene annotation information
rowRanges(rse_gene_SRP009615)

## Also accessible via
rowData(rse_gene_SRP009615)
```

For an `rse_exon` object, the `rownames()` correspond to the Gencode `gene_id`. You can add a `gene_id` column if you are interested in subsetting the whole `rse_exon` object.


```{r, 'exon_annotation'}
## Get the rse_exon object for study SRP009615
download_study("SRP009615", type = "rse-exon")
load(file.path("SRP009615", "rse_exon.Rdata"))

## Annotation information
rowRanges(rse_exon)

## Add a gene_id column
rowRanges(rse_exon)$gene_id <- rownames(rse_exon)

## Example subset
rse_exon_subset <- subset(rse_exon, subset = gene_id == "ENSG00000000003.14")
rowRanges(rse_exon_subset)
```

The exons in a `rse_exon` object are those that have a Gencode `gene_id` and that were disjointed as described in the `r Biocpkg('GenomicRanges')` package:

```
?`inter-range-methods`
```

You can reproduce the gene and exon information using the function `reproduce_ranges()`.

## Using another/newer annotation

If you are interested in using another annotation based on hg38 coordinates you can do so using the function `coverage_matrix()` or by running [bwtool](https://github.com/CRG-Barcelona/bwtool/wiki) thanks to the BigWig files in `recount`, in which case [recount.bwtool](https://github.com/LieberInstitute/recount.bwtool) will be helpful.

If you are re-processing a small set of samples, it simply might be easier to use `coverage_matrix()` as shown below using the annotation information provided by the `r Biocpkg('EnsDb.Hsapiens.v79')` package.

```{r "newer annotation"}
## Get the disjoint exons based on EnsDb.Hsapiens.v79 which matches hg38
exons <- reproduce_ranges("exon", db = "EnsDb.Hsapiens.v79")

## Change the chromosome names to match those used in the BigWig files
library("GenomeInfoDb")
seqlevelsStyle(exons) <- "UCSC"

## Get the count matrix at the exon level for chrY
exons_chrY <- keepSeqlevels(exons, "chrY", pruning.mode = "coarse")
exonRSE <- coverage_matrix("SRP009615", "chrY", unlist(exons_chrY),
    chunksize = 3000, outdir = "SRP009615"
)
exonMatrix <- assays(exonRSE)$counts
dim(exonMatrix)
head(exonMatrix)

## Summary the information at the gene level for chrY
exon_gene <- rep(names(exons_chrY), elementNROWS(exons_chrY))
geneMatrix <- do.call(rbind, lapply(split(
    as.data.frame(exonMatrix),
    exon_gene
), colSums))
dim(geneMatrix)
head(geneMatrix)
```

The above solution works well for a small set of samples. However, `bwtool` is faster for processing large sets. You can see how we used `bwtool` at [leekgroup/recount-website](https://github.com/leekgroup/recount-website). It's much faster than R and the small package [recount.bwtool](https://github.com/LieberInstitute/recount.bwtool) will be helpful for such scenarios. Note that you will need to run `scale_counts()` after using `coverage_matrix_bwtool()` and that will have to download the BigWig files first unless you are working at [JHPCE](https://jhpce.jhu.edu/) or [SciServer](http://www.sciserver.org/). `coverage_matrix_bwtool()` will download the files for you, but that till take time.

Finally, the BigWig files hosted by recount have the following chromosome names and sizes as specified in [hg38.sizes](https://github.com/nellore/runs/blob/master/gtex/hg38.sizes). You'll have to make sure that you match these names when using alternative annotations.


# Candidate gene fusions

If you are interested in finding possible gene fusion events in a particular study, you can download the _RangedSummarizedExperiment_ object at the exon-exon junctions level for that study. The objects we provide classify exon-exon junction events as shown below.

```{r 'gene_fusions'}
library("recount")

## Download and load RSE object for SRP009615 study
download_study("SRP009615", type = "rse-jx")
load(file.path("SRP009615", "rse_jx.Rdata"))

## Delete it if you don't need it anymore
unlink("SRP009615", recursive = TRUE)

## Exon-exon junctions by class
table(rowRanges(rse_jx)$class)

## Potential gene fusions by chromosome
fusions <- rowRanges(rse_jx)[rowRanges(rse_jx)$class == "fusion"]
fusions_by_chr <- sort(table(seqnames(fusions)), decreasing = TRUE)
fusions_by_chr[fusions_by_chr > 0]

## Genes with the most fusions
head(sort(table(unlist(fusions$symbol_proposed)), decreasing = TRUE))
```

If you are interested in checking the frequency of a particular exon-exon junction then check the `snaptron_query()` function described in the following section.


# Snaptron

Our collaborators built _SnaptronUI_ to query the [Intropolis](https://github.com/nellore/intropolis) database of the exon-exon junctions found in SRA. _SnaptronUI_ allows you to do several types of queries manually and perform analyses. `r Biocpkg('recount')` has the function `snaptron_query()` which uses [Snaptron](https://github.com/ChristopherWilks/snaptron) to check if particular exon-exon junctions are present in _Intropolis_. For example, here we use `snaptron_query()` to get in `R`-friendly format the information for 3 exon-exon junctions using _Intropolis_ version 1 which uses hg19 coordinates. Version 2 uses hg38 coordinates and the exon-exon junctions were derived from a larger data set: about 50,000 samples vs 21,000 in version 1. `snaptron_query()` can also be used to access the 30 million and 36 million exon-exon junctions derived from the GTEx and TCGA consortiums respectively (about 10 and 11 thousand samples respectively).

```{r snaptron}
library("GenomicRanges")
junctions <- GRanges(seqnames = "chr2", IRanges(
    start = c(28971711, 29555082, 29754983),
    end = c(29462418, 29923339, 29917715)
))

snaptron_query(junctions)
```

If you use `snaptron_query()` please cite [snaptron.cs.jhu.edu](http://snaptron.cs.jhu.edu). _Snaptron_ is separate from `r Biocpkg('recount')`. Thank you!

# FANTOM-CAT annotation

Thanks to [Imada, Sanchez, et al., bioRxiv, 2019](https://www.biorxiv.org/content/10.1101/659490v1) you can now use the `recount` package to access the FANTOM-CAT annotation that was merged with `recount2`. You can use the main `recount2` website or the `recount::download_study()` function by specifying `type = 'rse-fc'`. See their manuscript for further details on how this annotation was derived.

```{r 'rse-fc example'}
## Example use of download_study()
## for downloading the FANTOM-CAT `rse_fc`
## file for a given study
download_study("DRP000366", type = "rse-fc", download = FALSE)
```


# `recount-brain`

The `recount-brain` project by [Ramzara et al., bioRxiv, 2019](https://www.biorxiv.org/content/10.1101/618025v1) contains curated metadata for brain studies present in `recount2` that you can access via `recount::add_metadata(source = 'recount_brain_v2')`. The manuscript by Razmara et al describes a curation process that can be used for curating other tissues or projects. If you are interested in doing such a project and/or contributing curated metadata to the `recount` package via the `add_metadata()` function please let us know.

```{r 'recount-brain'}
## recount-brain citation details
citation("recount")[5]
```


# Download all the data

If you are interested in downloading all the data you can do so using the `recount_url` data.frame included in the package. That is:

```{r 'downloadAll'}
## Get data.frame with all the URLs
library("recount")
dim(recount_url)

## Explore URLs
head(recount_url$url)
```

You can then download all the data using the `download_study()` function or however you prefer to do so with `recount_url`.

```{r 'actuallyDownload', eval = FALSE}
## Download all files (will take a while! It's over 8 TB of data)
sapply(unique(recount_url$project), download_study, type = "all")
```

You can find the code for the website at [leekgroup/recount-website](https://github.com/leekgroup/recount-website/tree/master/website) if you want to deploy your own local mirror. Please let us know if you choose to do deploy a mirror.


# Accessing `r Biocpkg('recount')` via _SciServer_

Not everyone has over 8 terabytes of disk space available to download all the data from the `recount` project. However, thanks to [SciServer](http://www.sciserver.org/) you can access it locally via a Jupyter Notebook. If you do so and want to share your work, please let the [SciServer](http://www.sciserver.org/) maintainers know via Twitter at [IDIESJHU](https://twitter.com/IDIESJHU).


## Step by step


To use _SciServer_, you first have to go to http://www.sciserver.org/ then click on "Login to SciServer". If you are a first time user, click on "Register New Account" and enter the information they request as shown on Figure \@ref(fig:Figure1).

```{r Figure1, out.width="100%", fig.align="center", fig.cap = "SIgn up to SciServer", echo = FALSE}
knitr::include_graphics("https://raw.githubusercontent.com/leekgroup/recount-website/master/sciserver_demo/register.png")
```

Once you've registered, open the [SciServer compute tool](http://compute.sciserver.org/dashboard/) (see Figure \@ref(fig:Figure2)).

```{r Figure2, out.width="100%", fig.align="center", fig.cap = "Click on SciServer Compute,", echo = FALSE}
knitr::include_graphics("https://raw.githubusercontent.com/leekgroup/recount-website/master/sciserver_demo/sciserver-compute.png")
```

Then click on "Create container", choose a name of your preference and make sure that you choose to load R 3.3.x and the `recount` public volume (see Figure \@ref(fig:Figure3)). This will enable you to access all of the `recount` data as if it was on your computer. Click "Create" once you are ready.

```{r Figure3, out.width="100%", fig.align="center", fig.cap = "Select the appropriate image and load the recount public volume.", echo = FALSE}
knitr::include_graphics("https://raw.githubusercontent.com/leekgroup/recount-website/master/sciserver_demo/new-container.png")
```

Once you have a container, create a new R _Notebook_ as shown in Figure \@ref(fig:Figure4).

```{r Figure4, out.width="100%", fig.align="center", fig.cap = "Create a R notebook.", echo = FALSE}
knitr::include_graphics("https://raw.githubusercontent.com/leekgroup/recount-website/master/sciserver_demo/new-R.png")
```

In this R _Notebook_ you can insert new code _cells_ where you can type R code. For example, you can install `r Biocpkg('recount')` and `r Biocpkg('DESeq2')` to run the code example from this vignette. You first have to install the packages as shown in Figure \@ref(fig:Figure5).

```{r Figure5, out.width="100%", fig.align="center", fig.cap = "Install recount and DESeq2 on SciServer.", echo = FALSE}
knitr::include_graphics("https://raw.githubusercontent.com/leekgroup/recount-website/master/sciserver_demo/demo1.png")
```

Then, the main part when using _SciServer_ is to use to remember that all the `recount` data is available locally. So instead of using `download_study()`, you can simply use `load()` with the correct path as shown in Figure \@ref(fig:Figure6).

```{r Figure6, out.width="100%", fig.align="center", fig.cap = "recount files are available locally so remmeber to use the local data when working on SciServer.", echo = FALSE}
knitr::include_graphics("https://raw.githubusercontent.com/leekgroup/recount-website/master/sciserver_demo/demo2.png")
```

Several functions in the `r Biocpkg('recount')` package have the `outdir` argument, which you can specify to use the data locally. Try using `expressed_regions()` in _SciServer_ to get started with the following code:

```{r 'ER-sciserver', eval = FALSE}
## Expressed-regions SciServer example
regions <- expressed_regions("SRP009615", "chrY",
    cutoff = 5L,
    maxClusterGap = 3000L, outdir = file.path(scipath, "SRP009615")
)
regions
```


You can find the R code for a full _SciServer_ demo [here](https://github.com/leekgroup/recount-website/blob/master/sciserver_demo/sciserver_demo.R). You can copy and paste it into a cell and run it all to see how _SciServer compute_ works.

## SciServer help

If you encounter problems when using SciServer please check their [support page](http://www.sciserver.org/support/help/) and the [SciServer compute help section](http://compute.sciserver.org/dashboard/Home/Help).

## Citing _SciServer_

If you use [SciServer](http://www.sciserver.org/) for your published work, please cite it accordingly. _SciServer_ is administered by the [Institute for Data Intensive Engineering and Science](http://idies.jhu.edu/) at [Johns Hopkins University](http://www.jhu.edu/) and is funded by National Science Foundation award ACI-1261715.

# Reproducibility

The `r Biocpkg('recount')` package `r Citep(bib[['recount']])` was made possible thanks to:

* R `r Citep(bib[['R']])`
* `r Biocpkg('AnnotationDbi')` `r Citep(bib[['AnnotationDbi']])`
* `r Biocpkg('BiocParallel')` `r Citep(bib[['BiocParallel']])`
* `r Biocpkg('BiocStyle')` `r Citep(bib[['BiocStyle']])`
* `r Biocpkg('derfinder')` `r Citep(bib[['derfinder']])`
* `r CRANpkg('sessioninfo')` `r Citep(bib[['sessioninfo']])`
* `r CRANpkg('downloader')` `r Citep(bib[['downloader']])`
* `r Biocpkg('EnsDb.Hsapiens.v79')` `r Citep(bib[['EnsDb.Hsapiens.v79']])`
* `r Biocpkg('GEOquery')` `r Citep(bib[['GEOquery']])`
* `r Biocpkg('GenomeInfoDb')` `r Citep(bib[['GenomeInfoDb']])`
* `r Biocpkg('GenomicFeatures')` `r Citep(bib[['GenomicFeatures']])`
* `r Biocpkg('GenomicRanges')` `r Citep(bib[['GenomicRanges']])`
* `r CRANpkg('knitr')` `r Citep(bib[['knitr']])`
* `r Biocpkg('org.Hs.eg.db')` `r Citep(bib[['org.Hs.eg.db']])`
* `r CRANpkg('RCurl')` `r Citep(bib[['RCurl']])`
* `r CRANpkg('rentrez')` `r Citep(bib[['rentrez']])`
* `r CRANpkg("RefManageR")` `r Citep(bib[["RefManageR"]])`
* `r Biocpkg('rtracklayer')` `r Citep(bib[['rtracklayer']])`
* `r CRANpkg('rmarkdown')` `r Citep(bib[['rmarkdown']])`
* `r Biocpkg('S4Vectors')` `r Citep(bib[['S4Vectors']])`
* `r Biocpkg('SummarizedExperiment')` `r Citep(bib[['SummarizedExperiment']])`
* `r CRANpkg('testthat')` `r Citep(bib[['testthat']])`


Code for creating the vignette

```{r createVignette, eval=FALSE}
## Create the vignette
library("rmarkdown")
system.time(render("recount-quickstart.Rmd", "BiocStyle::html_document"))

## Extract the R code
library("knitr")
knit("recount-quickstart.Rmd", tangle = TRUE)
```

Date the vignette was generated.

```{r reproduce1, echo=FALSE}
## Date the vignette was generated
Sys.time()
```

Wallclock time spent generating the vignette.

```{r reproduce2, echo=FALSE}
## Processing time in seconds
totalTime <- diff(c(startTime, Sys.time()))
round(totalTime, digits = 3)
```

`R` session information.

```{r reproduce3, echo=FALSE}
## Session info
library("sessioninfo")
options(width = 120)
session_info()
```

```{r 'datasetup', echo = FALSE, eval = FALSE}
## Code for re-creating the data distributed in this package

## Genes/exons
library("recount")
recount_both <- reproduce_ranges("both", db = "Gencode.v25")
recount_exons <- recount_both$exon
save(recount_exons, file = "../data/recount_exons.RData", compress = "xz")
recount_genes <- recount_both$gene
save(recount_genes, file = "../data/recount_genes.RData", compress = "xz")

## URL table
load("../../recount-website/fileinfo/upload_table.Rdata")
recount_url <- upload_table
## Create URLs
is.bw <- grepl("[.]bw$", recount_url$file_name)
recount_url$url <- NA
recount_url$url[!is.bw] <- paste0(
    "http://duffel.rail.bio/recount/",
    recount_url$project[!is.bw], "/", recount_url$file_name[!is.bw]
)
recount_url$url[!is.bw & recount_url$version2] <-
    paste0(
        "http://duffel.rail.bio/recount/v2/",
        recount_url$project[!is.bw & recount_url$version2], "/",
        recount_url$file_name[!is.bw & recount_url$version2]
    )
recount_url$url[is.bw] <- paste0(
    "http://duffel.rail.bio/recount/",
    recount_url$project[is.bw], "/bw/", recount_url$file_name[is.bw]
)
save(recount_url, file = "../data/recount_url.RData", compress = "xz")

## Add FC-RC2 data
load("../data/recount_url.RData")
load("../../recount-website/fileinfo/fc_rc_url_table.Rdata")
recount_url <- rbind(recount_url, fc_rc_url_table)
rownames(recount_url) <- NULL
save(recount_url, file = "../data/recount_url.RData", compress = "xz")


## Abstract info
load("../../recount-website/website/meta_web.Rdata")
recount_abstract <- meta_web[, c("number_samples", "species", "abstract")]
recount_abstract$project <- gsub('.*">|</a>', "", meta_web$accession)
Encoding(recount_abstract$abstract) <- "latin1"
recount_abstract$abstract <- iconv(recount_abstract$abstract, "latin1", "UTF-8")
save(recount_abstract,
    file = "../data/recount_abstract.RData",
    compress = "bzip2"
)

## Example rse_gene file
system("scp e:/dcl01/leek/data/recount-website/rse/rse_sra/SRP009615/rse_gene.Rdata .")
load("rse_gene.Rdata")
rse_gene_SRP009615 <- rse_gene
save(rse_gene_SRP009615,
    file = "../data/rse_gene_SRP009615.RData",
    compress = "xz"
)
unlink("rse_gene.Rdata")
```

```{r 'cleanup', echo = FALSE, results = 'hide'}
## Clean up
unlink("SRP009615", recursive = TRUE)
```




# Bibliography

This vignette was generated using `r Biocpkg('BiocStyle')` `r Citep(bib[['BiocStyle']])`
with `r CRANpkg('knitr')` `r Citep(bib[['knitr']])` and `r CRANpkg('rmarkdown')` `r Citep(bib[['rmarkdown']])` running behind the scenes.

Citations made with `r CRANpkg('RefManageR')` `r Citep(bib[['RefManageR']])`.

```{r vignetteBiblio, results = 'asis', echo = FALSE, warning = FALSE, message = FALSE}
## Print bibliography
PrintBibliography(bib, .opts = list(hyperlink = "to.doc", style = "html"))
```

---
output:
  BiocStyle::html_document:
      toc: true
      toc_float: true
      code_folding: hide
  html_document:
    toc: true
    toc_float: true
    code_folding: hide
    theme: spacelab
  pdf_document:
      toc: true
  knitrBootstrap::bootstrap_document:
    theme.chooser: TRUE
    highlight.chooser: TRUE
vignette: >
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteIndexEntry{Basic DESeq2 results exploration}
  %\VignetteEncoding{UTF-8}
---


Basic DESeq2 results exploration
=================================

```{r}
library("recount")
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_counts.R
\name{scale_counts}
\alias{scale_counts}
\title{Scale the raw counts provided by the recount project}
\usage{
scale_counts(
  rse,
  by = "auc",
  targetSize = 4e+07,
  L = 100,
  factor_only = FALSE,
  round = TRUE
)
}
\arguments{
\item{rse}{A \link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class}
object as downloaded with \link{download_study}.}

\item{by}{Either \code{auc} or \code{mapped_reads}. If set to \code{auc} it
will scale the counts by the total coverage of the sample. That is, the area
under the curve (AUC) of the coverage. If set to \code{mapped_reads} it will
scale the counts by the number of mapped reads, whether the library was
paired-end or not, and the desired read length (\code{L}).}

\item{targetSize}{The target library size in number of single end reads.}

\item{L}{The target read length. Only used when \code{by = 'mapped_reads'}
since it cancels out in the calculation when using \code{by = 'auc'}.}

\item{factor_only}{Whether to only return the numeric scaling factor or
to return a \link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class}
object with the counts scaled. If set to \code{TRUE}, you have to multiply
the sample counts by this scaling factor.}

\item{round}{Whether to round the counts to integers or not.}
}
\value{
If \code{factor_only = TRUE} it returns a numeric vector with the
scaling factor for each sample. If \code{factor_only = FALSE} it returns a
\link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class} object with
the counts already scaled.
}
\description{
In preparation for a differential expression analysis, you will have to
choose how to scale the raw counts provided by the recount project. Note that
the raw counts are the sum of the base level coverage so you have to take
into account the read length or simply the total coverage for the given
sample (default option). You might want to do some further scaling to take
into account the gene or exon lengths. If you prefer to calculate read counts
without scaling check the function \link{read_counts}.
}
\details{
Rail-RNA \url{http://rail.bio} uses soft clipping when aligning
which is why we recommed using \code{by = 'auc'}.

If the reads are from a paired-end library, then the \code{avg_read_length}
is the average fragment length. This is taken into account when using
\code{by = 'mapped_reads'}.
}
\examples{
## Load an example rse_gene object
rse_gene <- rse_gene_SRP009615

## Scale counts
rse <- scale_counts(rse_gene)

## Find the project used as an example
project_info <- abstract_search("GSE32465")

## See some summary information for this project
project_info

## Use the following code to re-download this file
\dontrun{
## Download
download_study(project_info$project)

## Load file
load(file.path(project_info$project, "rse_gene.Rdata"))
identical(rse_gene, rse_gene_SRP009615)
}

}
\seealso{
\link{download_study}, \link{read_counts}
}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recount_url-data.R
\docType{data}
\name{recount_url}
\alias{recount_url}
\title{Files and URLs hosted by the recount project}
\format{
A data.frame with 6 columns.
\describe{
\item{path }{ the original path to the file before being uploaded,}
\item{file_name }{ the file name,}
\item{project }{ the SRA project id,}
\item{version1 }{ A logical vector indicating whether the file was
part of version 1 (reduced exons)}.
\item{version 2}{ A logical vector indicating whether the file was
updated in version 2 (disjoint exons)}. Further details in the recount
website documentation tab.
\item{url }{ the public URL for the given file.}
}
}
\description{
Files and URLs as provided by the recount project. This information is used
internally in \link{download_study}.
}
\references{
\url{https://jhubiostatistics.shinyapps.io/recount/}
}
\seealso{
\link{download_study}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/abstract_search.R
\name{abstract_search}
\alias{abstract_search}
\title{Search the abstracts from the SRA studies available via the recount project}
\usage{
abstract_search(query, id_only = FALSE, ...)
}
\arguments{
\item{query}{A character vector with the text to search for via
\link[base:grep]{grep} in the abstract info available at \link{recount_abstract}.}

\item{id_only}{Whether to only return the project id or to return summary
information for the project(s) that match the query.}

\item{...}{Additional arguments passed to \link[base:grep]{grep}.}
}
\value{
If \code{id_only = TRUE} it returns a character vector with the
project SRA ids (accession numbers). If \code{id_only = FALSE} it returns a
subset of \link{recount_abstract} for the abstracts that contained the query.
}
\description{
Given a text query, find the SRA project ids (study accession numbers) that
contain the text in their abstract as provided by the SRAdb Bioconductor
package.
}
\details{
Both the query and the abstracts are searched in lower case.

For a more powerful search use the recount project website at
\url{https://jhubiostatistics.shinyapps.io/recount/}.
}
\examples{
## Find the Geuvadis consortium project
project_info <- abstract_search("Geuvadis consortium")

## See some summary information for this project
project_info
}
\seealso{
\link{browse_study}, \link{recount_abstract}
}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rse_gene_SRP009615-data.R
\docType{data}
\name{rse_gene_SRP009615}
\alias{rse_gene_SRP009615}
\title{RangedSummarizedExperiment at the gene level for study SRP009615}
\format{
A \link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class} as
created by the recount project for study with SRA id (accession number)
SRP009615.
}
\description{
\link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class} at the gene
level for study SRP009615. Used as an example in \link{scale_counts}.
Matches the version2 file
(details at \url{https://jhubiostatistics.shinyapps.io/recount/}) under the
documentation tab.
}
\references{
\url{https://jhubiostatistics.shinyapps.io/recount/}
}
\seealso{
\link{scale_counts}, \link{download_study}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/read_counts.R
\name{read_counts}
\alias{read_counts}
\title{Compute read counts}
\usage{
read_counts(rse, use_paired_end = TRUE, round = FALSE)
}
\arguments{
\item{rse}{A \link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class}
object as downloaded with \link{download_study}.}

\item{use_paired_end}{A logical vector. When \code{TRUE} it uses the
paired-end flag (\code{colData(rse)$paired_end}) to determine whether
the sample is paired-end or not. If it's paired-end, then it divides the
counts by 2 to return paired-end read counts instead of fragment counts.
When \code{FALSE}, this information is ignored.}

\item{round}{Whether to round the counts to integers or not.}
}
\value{
Returns a
\link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class} object with
the read counts.
}
\description{
As described in the recount workflow, the counts provided by the recount2
project are base-pair counts. You can scale them using \link{scale_counts}
or compute the read counts using the area under coverage information (AUC).
We use the AUC because Rail-RNA soft clips some reads.
}
\details{
Check the recount workflow for details about the counts provided by
the recount2 project.
Note that this function should not be used after \link{scale_counts} or it
will return non-sensical results.
}
\examples{

## Difference between read counts and reads downloaded by Rail-RNA
colSums(assays(read_counts(rse_gene_SRP009615))$counts) / 1e6 -
    colData(rse_gene_SRP009615)$reads_downloaded / 1e6

## Paired-end read counts vs fragment counts (single-end counts)
download_study("DRP000499")
load("DRP000499/rse_gene.Rdata")
colSums(assays(read_counts(rse_gene, use_paired_end = FALSE))$counts) /
    colSums(assays(read_counts(rse_gene))$counts)

## Difference between paired-end read counts vs paired-end reads downloaded
colSums(assays(read_counts(rse_gene))$counts) / 1e6 -
    colData(rse_gene)$reads_downloaded / 1e6 / 2
}
\references{
Collado-Torres L, Nellore A and Jaffe AE. recount workflow: Accessing over
70,000 human RNA-seq samples with Bioconductor version 1; referees: 1
approved, 2 approved with reservations. F1000Research 2017, 6:1558
doi: 10.12688/f1000research.12223.1.
}
\seealso{
\link{scale_counts}
}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/all_metadata.R
\name{all_metadata}
\alias{all_metadata}
\title{This function downloads the metadata for all projects.}
\usage{
all_metadata(subset = "sra", verbose = TRUE)
}
\arguments{
\item{subset}{Either \code{sra},  \code{gtex} or \code{tcga}. Specifies
which metadata file to download.}

\item{verbose}{If \code{TRUE} it will print a message of where the file is
being downloaded to.}
}
\value{
A \link[S4Vectors:DataFrame-class]{DataFrame-class} object with the phenotype
metadata.
}
\description{
Download the metadata from all the projects. This can be useful for finding
samples of interests across all projects.
}
\details{
Note that for \code{subset = 'gtex'}, there are more variables than
the ones we have for 'sra'. This information corresponds to file
GTEx_Data_V6_Annotations_SampleAttributesDS.txt available at
\url{http://www.gtexportal.org/home/datasets}. There you can find the
information describing these variables.

For TCGA we acquired metadata information from 3 different sources:
\itemize{
\item GDC: via a json query
\item CGC: via json queries and a custom script to merge the tables
\item TCGAbiolinks: we used to to parse GDC's XML files
For more information, check \url{https://github.com/leekgroup/recount-website/tree/master/metadata/tcga_prep}.
}
}
\examples{

metadata <- all_metadata()
}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getRPKM.R
\name{getRPKM}
\alias{getRPKM}
\title{Compute an RPKM matrix based on a RangedSummarizedExperiment object}
\usage{
getRPKM(rse, length_var = "bp_length", mapped_var = NULL)
}
\arguments{
\item{rse}{A \link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class}
object as downloaded with \link{download_study}.}

\item{length_var}{A length 1 character vector with the column name from
\code{rowData(rse)} that has the coding length. For gene level objects
from recount this is \code{bp_length}. If \code{NULL}, then it will use
\code{width(rowRanges(rse))} which should be used for exon RSEs.}

\item{mapped_var}{A length 1 character vector with the column name from
\code{colData(rse)} that has the number of reads mapped. For recount RSE
object this would be \code{mapped_read_count}. If \code{NULL} (default)
then it will use the column sums of the counts matrix. The results are
different because not all mapped reads are mapped to exonic segments of the
genome.}
}
\value{
A matrix with the RPKM values.
}
\description{
For some analyses you might be interested in transforming the counts into
RPKMs which you can do with this function.
}
\details{
For gene RSE objects, you will want to specify the \code{length_var}
because otherwise you will be adjusting for the total gene length instead
of the total exonic sequence length of the gene.
}
\examples{

## get RPKM matrix
rpkm <- getRPKM(rse_gene_SRP009615)

## You can also get an RPKM matrix after running scale_counts()
## with similar RPKM values
rpkm2 <- getRPKM(scale_counts(rse_gene_SRP009615))
rpkm3 <- getRPKM(scale_counts(rse_gene_SRP009615, by = "mapped_reads"))

summary(rpkm - rpkm2)
summary(rpkm - rpkm3)
summary(rpkm2 - rpkm3)
}
\seealso{
\link{scale_counts}
}
\author{
Andrew Jaffe, Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_study.R
\name{download_study}
\alias{download_study}
\title{Download data for a given SRA study id from the recount project}
\usage{
download_study(
  project,
  type = "rse-gene",
  outdir = project,
  download = TRUE,
  version = 2,
  ...
)
}
\arguments{
\item{project}{A character vector with one SRA study id.}

\item{type}{Specifies which files to download. The options are:
\describe{
\item{rse-gene}{ the gene-level
\link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class} object in
a file named rse_gene.Rdata.}
\item{rse-exon}{ the exon-level
\link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class} object in
a file named rse_exon.Rdata.}
\item{rse-jx}{ the exon-exon junction level
\link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class} object in
a file named rse_jx.Rdata.}
\item{rse-tx}{ the transcript level
\link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class} object in
a file named rse_tx.RData.}
\item{counts-gene}{ the gene-level counts in a tsv file named
counts_gene.tsv.gz.}
\item{counts-exon}{ the exon-level counts in a tsv file named
counts_exon.tsv.gz.}
\item{counts-jx}{ the exon-exon junction level counts in a tsv file named
counts_jx.tsv.gz.}
\item{phenotype}{ the phenotype data for the study in a tsv file named
\code{project}.tsv.}
\item{files-info}{ the files information for the given study (including
md5sum hashes) in a tsv file named files_info.tsv.}
\item{samples}{ one bigWig file per sample in the study.}
\item{mean}{ one mean bigWig file for the samples in the study,
with each sample normalized to a 40 million 100 bp library using the
total coverage sum (area under the coverage curve, AUC) for the given
sample.}
\item{all}{ Downloads all the above types. Note that it might take some
time if the project has many samples. When using \code{type = 'all'} a
small delay will be added before each download request to avoid
request issues.}
\item{rse-fc}{ Downloads the FANTOM-CAT/recount2 rse file described in
Imada, Sanchez, et al., bioRxiv, 2019.}
}}

\item{outdir}{The destination directory for the downloaded file(s).
Alternatively check the \code{SciServer} section on the vignette to see
how to access all the recount data via a R Jupyter Notebook.}

\item{download}{Whether to download the files or just get the download urls.}

\item{version}{A single integer specifying which version of the files to
download. Valid options are 1 and 2, as described in
\url{https://jhubiostatistics.shinyapps.io/recount/} under the
documentation tab. Briefly, version 1 are counts based on reduced exons while
version 2 are based on disjoint exons. This argument mostly just matters for
the exon counts. Defaults to version 2 (disjoint exons).
Use \code{version = 1} for backward compatability with exon counts
prior to version 1.5.3 of the package.}

\item{...}{Additional arguments passed to \link[downloader:download]{download}.}
}
\value{
Returns invisibly the URL(s) for the files that were downloaded.
}
\description{
Download the gene or exon level
\link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class} objects
provided by the recount project. Alternatively download the counts, metadata
or file information for a given SRA study id. You can also download the
sample bigWig files or the mean coverage bigWig file.
}
\details{
Check \url{http://stackoverflow.com/a/34383991} if you need to find
the effective URLs. For example,
\url{http://duffel.rail.bio/recount/DRP000366/bw/mean_DRP000366.bw} points to
a link from SciServer.

Transcript quantifications are described in Fu et al, bioRxiv, 2018.
\url{https://www.biorxiv.org/content/10.1101/247346v2}

FANTOM-CAT/recount2 quantifications are described in Imada,
Sanchez, et al., bioRxiv, 2019.
\url{https://www.biorxiv.org/content/10.1101/659490v1}
}
\examples{
## Find the URL to download the RangedSummarizedExperiment for the
## Geuvadis consortium study.
url <- download_study("ERP001942", download = FALSE)

## See the actual URL
url
\dontrun{
## Download the example data included in the package for study SRP009615

url2 <- download_study("SRP009615")
url2

## Load the data
load(file.path("SRP009615", "rse_gene.Rdata"))

## Compare the data
library("testthat")
expect_equivalent(rse_gene, rse_gene_SRP009615)
}

}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_predictions.R
\name{add_predictions}
\alias{add_predictions}
\title{Add predicted phenotypes to a recount rse object}
\usage{
add_predictions(rse, is_tcga = FALSE, version = "latest", verbose = TRUE)
}
\arguments{
\item{rse}{A \link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class}
object as downloaded with \link{download_study}. If this argument is
not specified, the function will return the full predictions table.}

\item{is_tcga}{Set to \code{TRUE} only when \code{rse} is from TCGA.
Otherwise set to \code{FALSE} (default).}

\item{version}{The version number for the predicted phenotypes data. It has
to match one of the available numbers at
\url{https://github.com/leekgroup/recount-website/blob/master/predictions/}.
Feel free to check if there is a newer version than the default. The version
used is printed as part of the file name.}

\item{verbose}{If \code{TRUE} it will print a message of where the
predictions file is being downloaded to.}
}
\value{
A \link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class}
object with the prediction columns appended to the \code{colData()} slot.
The predicted phenotypes are:
\describe{
\item{sex }{ male or female,}
\item{samplesource }{ cell_line or tissue,}
\item{tissue }{ tissue predicted based off of 30 tissues in GTEx,}
\item{sequencingstrategy }{ single or paired end sequencing.}
}
For each of the predicted phenotypes there are several columns as described
next:
\describe{
\item{reported_phenotype }{ \code{NA} when not available,}
\item{predicted_phenotype }{ \code{NA} when we did not predict, "Unassigned"
when prediction was ambiguous,}
\item{accuracy_phenotype }{ accuracy is assigned per dataset based on
comparison to samples for which we had reported phenotype information so
there are three distinct values per predictor (GTEx, TCGA, SRA) across all
studies.}
}
}
\description{
Shannon Ellis et al (2017) predicted phenotypes based on expression data for
the samples in the recount2 project. Using this function you can add the
predictions to a
\link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class} object
to the \code{colData()} slot.
}
\details{
If you use these predicted phenotypes please cite the Ellis et al
bioRxiv pre-print available at
https://www.biorxiv.org/content/early/2017/06/03/145656. See citation
details with citation('recount').
}
\examples{

## Add the predictions to an example rse_gene object
rse_gene <- add_predictions(rse_gene_SRP009615)

## Explore the predictions
colData(rse_gene)

## Download all the latest predictions
PredictedPhenotypes <- add_predictions()
}
\references{
Ellis et al, bioRxiv, 2017.
https://www.biorxiv.org/content/early/2017/06/03/145656
}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geo_info.R
\name{geo_info}
\alias{geo_info}
\title{Extract information from GEO for a given sample}
\usage{
geo_info(
  geoid,
  verbose = FALSE,
  sleep = 1/2,
  getGPL = FALSE,
  destdir = tempdir(),
  ...
)
}
\arguments{
\item{geoid}{A character vector of length 1 with the GEO accession id for
a given sample.}

\item{verbose}{If \code{TRUE} the \code{geoid} will be shown.}

\item{sleep}{The number of seconds (or fraction) to wait before downloading
data using \link[GEOquery:getGEO]{getGEO}. This is important if you are looking over
\code{geo_info()} given the constraints published at
\url{https://www.ncbi.nlm.nih.gov/books/NBK25497/}.}

\item{getGPL}{This argument is passed to \link[GEOquery:getGEO]{getGEO} and is set
to \code{FALSE} by default to speed up the process.}

\item{destdir}{This argument is passed to \link[GEOquery:getGEO]{getGEO}.}

\item{...}{Additional arguments passed to \link[GEOquery:getGEO]{getGEO}.}
}
\value{
Returns a \link[S4Vectors:DataFrame-class]{DataFrame-class} with the information
from GEO available for the given sample.
}
\description{
This function uses GEOquery to extract information for a given sample. The
GEO accession ids for the sample can be found in the study phenotype table.
}
\examples{
geo_info("GSM836270")
}
\author{
Leonardo Collado-Torres, Andrew Jaffe
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/expressed_regions.R
\name{expressed_regions}
\alias{expressed_regions}
\title{Identify expressed regions from the mean coverage for a given SRA project}
\usage{
expressed_regions(
  project,
  chr,
  cutoff,
  outdir = NULL,
  maxClusterGap = 300L,
  chrlen = NULL,
  verbose = TRUE,
  ...
)
}
\arguments{
\item{project}{A character vector with one SRA study id.}

\item{chr}{A character vector with the name of the chromosome.}

\item{cutoff}{The base-pair level cutoff to use.}

\item{outdir}{The destination directory for the downloaded file(s) that were
previously downloaded with \link{download_study}. If the files are missing,
but \code{outdir} is specified, they will get downloaded first. By default
\code{outdir} is set to \code{NULL} which will use the data from the web.
We only recommend downloading the full data if you will use it several times.}

\item{maxClusterGap}{This determines the maximum gap between candidate ERs.}

\item{chrlen}{The chromosome length in base pairs. If it's \code{NULL}, the
chromosome length is extracted from the Rail-RNA runs GitHub repository.
Alternatively check the \code{SciServer} section on the vignette to see
how to access all the recount data via a R Jupyter Notebook.}

\item{verbose}{If \code{TRUE} basic status updates will be printed along the
way.}

\item{...}{Additional arguments passed to \link{download_study} when
\code{outdir} is specified but the required files are missing.}
}
\value{
A \link[GenomicRanges:GRanges-class]{GRanges-class} object as created by
\link[derfinder:findRegions]{findRegions}.
}
\description{
This function uses the pre-computed mean coverage for a given SRA project to
identify the expressed regions (ERs) for a given chromosome. It returns a
\link[GenomicRanges:GRanges-class]{GRanges-class} object with the expressed regions as
defined by \link[derfinder:findRegions]{findRegions}.
}
\examples{
## Define expressed regions for study SRP002001, chrY

## Workaround for https://github.com/lawremi/rtracklayer/issues/83
download_study("SRP002001", type = "mean")

regions <- expressed_regions("SRP002001", "chrY",
    cutoff = 5L,
    maxClusterGap = 3000L,
    outdir = "SRP002001"
)

\dontrun{
## Define the regions for multiple chrs
regs <- sapply(chrs, expressed_regions, project = "SRP002001", cutoff = 5L)

## You can then combine them into a single GRanges object if you want to
library("GenomicRanges")
single <- unlist(GRangesList(regs))
}

}
\seealso{
\link{download_study}, \link[derfinder:findRegions]{findRegions},
\link[derfinder:railMatrix]{railMatrix}
}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_metadata.R
\name{add_metadata}
\alias{add_metadata}
\title{Add additional curated metadata to a recount rse object}
\usage{
add_metadata(rse, source = "recount_brain_v2", is_tcga = FALSE, verbose = TRUE)
}
\arguments{
\item{rse}{A \link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class}
object as downloaded with \link{download_study}. If this argument is
not specified, the function will return the raw metadata table.}

\item{source}{A valid source name. The only supported options at this
moment are \code{recount_brain_v1} and \code{recount_brain_v2}.}

\item{is_tcga}{Set to \code{TRUE} only when \code{rse} is from TCGA.
Otherwise set to \code{FALSE} (default).}

\item{verbose}{If \code{TRUE} it will print a message of where the
predictions file is being downloaded to.}
}
\value{
A \link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class}
object with the sample metadata columns appended to the \code{colData()}
slot.
}
\description{
This function appends sample metadata information to a
\link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class} from the
recount2 project. The sample metadata comes from curated efforts
independent from the original recount2 project. Currently the only
information comes from the recount_brain project described in more detail
at \url{http://lieberinstitute.github.io/recount-brain/}.
}
\details{
For \code{source = "recount_brain_v1"} and
\code{source = "recount_brain_v2"}, the metadata columns are
described at \url{http://lieberinstitute.github.io/recount-brain/}.
Alternatively, you can explore \code{recount_brain_v2} interactively at
\url{https://jhubiostatistics.shinyapps.io/recount-brain/}.

If you use the recount_brain data please cite the Razmara et al.
bioRxiv, 2019 \url{https://www.biorxiv.org/content/10.1101/618025v1}.
A bib file is available via citation('recount').
}
\examples{

## Add the sample metadata to an example rse_gene object
rse_gene <- add_metadata(rse_gene_SRP009615, "recount_brain_v2")

## Explore the metadata
colData(rse_gene)

## For a list of studies present in recount_brain check
## http://lieberinstitute.github.io/recount-brain/.
## recount_brain_v2 includes GTEx and TCGA brain samples in addition to the
## recount_brain_v1 data, plus ontology information.


## Obtain all the recount_brain_v2 metadata if you want to
## explore the metadata manually
recount_brain_v2 <- add_metadata(source = "recount_brain_v2")
}
\references{
Razmara et al, bioRxiv, 2019.
\url{https://www.biorxiv.org/content/10.1101/618025v1}
}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/download_retry.R
\name{download_retry}
\alias{download_retry}
\title{Retry multiple times to download a file}
\usage{
download_retry(url, destfile = basename(url), mode = "wb", N.TRIES = 3L, ...)
}
\arguments{
\item{url}{The URL to download. Passed to \link[downloader:download]{download}.}

\item{destfile}{The destination file. Defaults to the base name of the URL.
Passed to \link[downloader:download]{download}.}

\item{mode}{Mode for writing the file. The default \code{wb} is used for
binary files. This value is passed to \link[downloader:download]{download} which
passes it to \link[utils:download.file]{download.file}.}

\item{N.TRIES}{The number of download attempts before giving up; default: 3.
Should be an integer of length one with a value greater than 0.}

\item{...}{Additional arguments passed to \link[downloader:download]{download}.}
}
\value{
An invisible integer code as specified in
\link[utils:download.file]{download.file}.
}
\description{
This function is based on the Bioconductor guidelines for querying data from
the web at \url{http://bioconductor.org/developers/how-to/web-query/}. It
will run \link[downloader:download]{download} a set of \code{N.TRIES} times before
giving up. We implemented this function to reduce the number of Bioconductor
build errors due to the occassional errors from our data hosting server.
}
\examples{

## Download the first files_info.tsv file (version 1)
download_retry(
    recount_url$url[which(recount_url$file_name == "files_info.tsv")[1]]
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/snaptron_query.R
\name{snaptron_query}
\alias{snaptron_query}
\title{Query Snaptron to get data from exon-exon junctions present in Intropolis}
\usage{
snaptron_query(junctions, version = "srav1", verbose = TRUE, async = TRUE)
}
\arguments{
\item{junctions}{A \link[GenomicRanges:GRanges-class]{GRanges-class} object with the
exon-exon junctions of interest. The chromosome names should be in UCSC
format, such as 'chr1'. The strand information is ignored in the query.}

\item{version}{Either \code{srav1}, \code{srav2}, \code{gtex} or
\code{tcga}. SRA Version 1 of Intropolis has the
exon-exon junctions from about 20 thousand RNA-seq samples in hg19
coordinates. SRA Version 2 has the data from about 50 thousand RNA-seq
samples aligned to hg38. GTEx has about 30 million junctions from about 10
thousand samples from the GTEx consortium on hg38 coordinates. Finally,
TCGA has about 36 million junctions from about 11 thousand samples
from the TCGA consortium on hg38 coordinates.}

\item{verbose}{If \code{TRUE} status updates will be printed.}

\item{async}{Defaults to \code{TRUE} but in some situations it might be
preferrable to set it to \code{FALSE}. This argument gets passed to
\link[RCurl:getURL]{getURL}. Check \url{https://github.com/ChristopherWilks/snaptron/issues/11}
for more details.}
}
\value{
A \link[GenomicRanges:GRanges-class]{GRanges-class} object with the results from
the Snaptron query. For information on the different columns please see
\url{http://snaptron.cs.jhu.edu}.
}
\description{
This function uses the Snaptron API to query specific exon-exon junctions
that are available via Intropolis as described in the vignette.
}
\examples{

library("GenomicRanges")
## Define some exon-exon junctions (hg19 coordinates)
junctions <- GRanges(seqnames = "chr2", IRanges(
    start = c(28971710:28971712, 29555081:29555083, 29754982:29754984),
    end = c(29462417:29462419, 29923338:29923340, 29917714:29917716)
))

## Check against Snaptron SRA version 1 (hg19 coordinates)
snaptron_query(junctions)
\dontrun{
## Check another set of junctions against SRA version 2 (more data, hg38
## coordinates)
junctions_v2 <- GRanges(seqnames = "chr2", IRanges(
    start = 29532116:29532118, end = 29694848:29694850
))
snaptron_query(junctions_v2, version = "srav2")

## Check these junctions in GTEx and TCGA data
snaptron_query(junctions_v2, version = "gtex")
snaptron_query(junctions_v2, version = "tcga")
}

}
\references{
Please cite \url{http://snaptron.cs.jhu.edu}
if you use this function as Snaptron is a separate project from recount.
Thank you!
}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_geo.R
\name{find_geo}
\alias{find_geo}
\title{Find the GEO accession id for a given SRA run}
\usage{
find_geo(run, verbose = FALSE, sleep = 1/2)
}
\arguments{
\item{run}{A character vector of length 1 with the SRA run accession id.}

\item{verbose}{Whether to print a message for the run. Useful when looping
over a larger number of SRA run ids.}

\item{sleep}{The number of seconds (or fraction) to wait before downloading
data using \link[GEOquery:getGEO]{getGEO}. This is important if you are looking over
\code{geo_info()} given the constraints published at
\url{https://www.ncbi.nlm.nih.gov/books/NBK25497/}.}
}
\value{
The GEO accession id for the corresponding sample.
}
\description{
Given a SRA run id, this function will retrieve the GEO accession id
(starting with GSM) if it's available. Otherwise it will return \code{NA}.
}
\details{
Although the phenotype information already includes the GEO
accession ids, not all projects had GEO entries at the time these tables
were created. This function will then be useful to check if there is a GEO
accession id for a given sample (run). If there is, you can then retrieve
the information using \link{geo_info}.
}
\examples{
## Find the GEO accession id for for SRX110461
find_geo("SRX110461")
}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recount-package.R
\docType{package}
\name{recount-package}
\alias{recount}
\alias{recount-package}
\title{recount: Explore and download data from the recount project}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

Explore and download data from the recount project available at https://jhubiostatistics.shinyapps.io/recount/. Using the recount package you can download RangedSummarizedExperiment objects at the gene, exon or exon-exon junctions level, the raw counts, the phenotype metadata used, the urls to the sample coverage bigWig files or the mean coverage bigWig file for a particular study. The RangedSummarizedExperiment objects can be used by different packages for performing differential expression analysis. Using http://bioconductor.org/packages/derfinder you can perform annotation-agnostic differential expression analyses with the data from the recount project as described at http://www.nature.com/nbt/journal/v35/n4/full/nbt.3838.html.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/leekgroup/recount}
  \item Report bugs at \url{https://support.bioconductor.org/t/recount/}
}

}
\author{
\strong{Maintainer}: Leonardo Collado-Torres \email{lcolladotor@gmail.com} (\href{https://orcid.org/0000-0003-2140-308X}{ORCID})

Authors:
\itemize{
  \item Jeffrey T. Leek \email{jtleek@gmail.com} (\href{https://orcid.org/0000-0002-2873-2671}{ORCID}) [thesis advisor]
}

Other contributors:
\itemize{
  \item Abhinav Nellore \email{anellore@gmail.com} [contributor]
  \item Andrew E. Jaffe \email{andrew.jaffe@libd.org} (\href{https://orcid.org/0000-0001-6886-1454}{ORCID}) [contributor]
  \item Margaret A. Taub \email{mtaub@jhsph.edu} [contributor]
  \item Kai Kammers \email{kai.kammers@gmail.com} [contributor]
  \item Shannon E. Ellis \email{sellis18@jhmi.edu} (\href{https://orcid.org/0000-0002-9231-0481}{ORCID}) [contributor]
  \item Kasper Daniel Hansen \email{kasperdanielhansen@gmail.com} (\href{https://orcid.org/0000-0003-0086-0687}{ORCID}) [contributor]
  \item Ben Langmead \email{langmea@cs.jhu.edu} (\href{https://orcid.org/0000-0003-2437-1976}{ORCID}) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/coverage_matrix.R
\name{coverage_matrix}
\alias{coverage_matrix}
\title{Given a set of regions for a chromosome, compute the coverage matrix for a
given SRA study.}
\usage{
coverage_matrix(
  project,
  chr,
  regions,
  chunksize = 1000,
  bpparam = NULL,
  outdir = NULL,
  chrlen = NULL,
  verbose = TRUE,
  verboseLoad = verbose,
  scale = TRUE,
  round = FALSE,
  ...
)
}
\arguments{
\item{project}{A character vector with one SRA study id.}

\item{chr}{A character vector with the name of the chromosome.}

\item{regions}{A \link[GenomicRanges:GRanges-class]{GRanges-class} object with regions
for \code{chr} for which to calculate the coverage matrix.}

\item{chunksize}{A single integer vector defining the chunksize to use for
computing the coverage matrix. Regions will be split into different chunks
which can be useful when using a parallel instance as defined by
\code{bpparam}.}

\item{bpparam}{A \link[BiocParallel:BiocParallelParam-class]{BiocParallelParam-class} instance which
will be used to calculate the coverage matrix in parallel. By default,
\link[BiocParallel:SerialParam-class]{SerialParam-class} will be used.}

\item{outdir}{The destination directory for the downloaded file(s) that were
previously downloaded with \link{download_study}. If the files are missing,
but \code{outdir} is specified, they will get downloaded first. By default
\code{outdir} is set to \code{NULL} which will use the data from the web.
We only recommend downloading the full data if you will use it several times.}

\item{chrlen}{The chromosome length in base pairs. If it's \code{NULL}, the
chromosome length is extracted from the Rail-RNA runs GitHub repository.
Alternatively check the \code{SciServer} section on the vignette to see
how to access all the recount data via a R Jupyter Notebook.}

\item{verbose}{If \code{TRUE} basic status updates will be printed along the
way.}

\item{verboseLoad}{If \code{TRUE} basic status updates for loading the data
will be printed.}

\item{scale}{If \code{TRUE}, the coverage counts will be scaled to read
counts based on a library size of 40 million reads. Set \code{scale} to
\code{FALSE} if you want the raw coverage counts. The scaling method is by
AUC, as in the default option of \link{scale_counts}.}

\item{round}{If \code{TRUE}, the counts are rounded to integers. Set to
\code{TRUE} if you want to match the defaults of \link{scale_counts}.}

\item{...}{Additional arguments passed to \link{download_study} when
\code{outdir} is specified but the required files are missing.}
}
\value{
A \link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class}
object with the counts stored in the assays slot.
}
\description{
Given a set of genomic regions as created by \link{expressed_regions}, this
function computes the coverage matrix for a library size of 40 million 100 bp
reads for a given SRA study.
}
\details{
When using \code{outdir = NULL} the information will be accessed
from the web on the fly. If you encounter internet access problems, it might
be best to first download the BigWig files using \link{download_study}. This
might be the best option if you are accessing all chromosomes for a given
project and/or are thinking of using different sets of \code{regions} (for
example, from different cutoffs applied to \link{expressed_regions}).
Alternatively check the \code{SciServer} section on the vignette to see
how to access all the recount data via a R Jupyter Notebook.

If you have \code{bwtool} installed, you can use
\url{https://github.com/LieberInstitute/recount.bwtool} for faster results.
Note that you will need to run \link{scale_counts} after running
\code{coverage_matrix_bwtool()}.
}
\examples{

## Workaround for https://github.com/lawremi/rtracklayer/issues/83
download_study("SRP002001", type = "mean")
download_study("SRP002001", type = "samples")

## Define expressed regions for study DRP002835, chrY
regions <- expressed_regions("SRP002001", "chrY",
    cutoff = 5L,
    maxClusterGap = 3000L,
    outdir = "SRP002001"
)

## Now calculate the coverage matrix for this study
rse <- coverage_matrix("SRP002001", "chrY", regions, outdir = "SRP002001")

## One row per region
identical(length(regions), nrow(rse))
}
\seealso{
\link{download_study}, \link[derfinder:findRegions]{findRegions},
\link[derfinder:railMatrix]{railMatrix}
}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/browse_study.R
\name{browse_study}
\alias{browse_study}
\title{Open a SRA study id in the SRA website}
\usage{
browse_study(project, browse = interactive())
}
\arguments{
\item{project}{A character vector with at least one SRA study id.}

\item{browse}{Whether to open the resulting URL in the browser.}
}
\value{
Returns invisibly the URL for exploring the study in the SRA website.
}
\description{
Given a SRA study id get the url to browse the study using the SRA website.
}
\examples{
## Find the Geuvadis consortium project
id <- abstract_search("Geuvadis consortium", id_only = TRUE)
id

## Explore the Geuvadis consortium project
url <- browse_study(id)

## See the actual URL
url
}
\seealso{
\link{abstract_search}
}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reproduce_ranges.R
\name{reproduce_ranges}
\alias{reproduce_ranges}
\title{Reproduce the gene or exons used in the RangedSummarizedExperiment objects}
\usage{
reproduce_ranges(level = "gene", db = "Gencode.v25")
}
\arguments{
\item{level}{Either \code{genes} or \code{exon}. It specifies whether to
return Gene or exon level information as a
\link[GenomicRanges:GRanges-class]{GRanges-class} or
\link[GenomicRanges:GRangesList-class]{GRangesList-class} object respectively. The gene level
information contains the width of the disjoint exons for that given gene
which can be used to normalize the counts provided by recount.
Can also be \code{both} in which case a 2 element list with the exon and the
gene output is returned.}

\item{db}{Either \code{Gencode.v25} (default) or
\code{EnsDb.Hsapiens.v79}. The default option reproduces the annotation
used when creating recount. EnsDb.Hsapiens.v79 can be used
for an alternative annotation as showcased in the recount vignette.}
}
\value{
Either a \link[GenomicRanges:GRanges-class]{GRanges-class} object like
\link{recount_genes} or a \link[GenomicRanges:GRangesList-class]{GRangesList-class} object
like \link{recount_exons}.
}
\description{
This function reproduces the gene or exon level information used for creating
the \link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class}
objects provided by recount. The annotation is based on
Gencode v25 with the gene-level
information extracted with \code{genes()} (see
\link[GenomicFeatures:transcripts]{transcripts} with default arguments.
}
\details{
For Gencode.v25, we use the comprehensive gene annotation (regions:
\code{CHR}) from \url{https://www.gencodegenes.org/releases/25.html}
(GRCh38.p7).

Note that gene symbols have changed over time. This answer in the
Bioconductor support forum details how to obtain the latest gene symbol
mappings: \url{https://support.bioconductor.org/p/126148/#126173}.
}
\examples{
\dontrun{
## Reproduce gene level information
genes <- reproduce_ranges()

## Compare against recount_genes
length(genes)
length(recount_genes)
}

}
\seealso{
\link{recount_genes}, \link{recount_exons},
\url{https://github.com/nellore},
\url{https://jhubiostatistics.shinyapps.io/recount/}
}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recount_abstract-data.R
\docType{data}
\name{recount_abstract}
\alias{recount_abstract}
\title{Summary information at the project level for the recount project}
\format{
A data.frame with 4 columns.
\describe{
\item{number_samples }{ the number of samples in the study,}
\item{species }{ the species of the study,}
\item{abstract }{ the abstract text as provided by the SRAdb Bioconductor
package,}
\item{project }{ the SRA project id.}
}
}
\description{
A data.frame with summary information at the project level for the studies
analyzed in the recount project.
}
\references{
\url{https://jhubiostatistics.shinyapps.io/recount/}
}
\seealso{
\link{download_study}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recount_exons-data.R
\docType{data}
\name{recount_exons}
\alias{recount_exons}
\title{Exon annotation used in recount}
\format{
A \link[GenomicRanges:GRangesList-class]{GRangesList-class} with one element per
gene. The names match the gene Gencode v25 ids.
}
\description{
Exon annotation extracted from Gencode v25 (GRCh38.p7) used in recount.
Data extraced on October 12th, 2017.
}
\references{
\url{https://jhubiostatistics.shinyapps.io/recount/}
}
\seealso{
\link{reproduce_ranges}, \link{recount_genes}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/recount_genes-data.R
\docType{data}
\name{recount_genes}
\alias{recount_genes}
\title{Gene annotation used in recount}
\format{
A \link[GenomicRanges:GRanges-class]{GRanges-class} with one range per gene. The
names match their Gencode v25 ids. The \link[GenomicRanges:GRanges-class]{GRanges-class}
has three metadata columns.
\describe{
\item{gene_id }{  the Gencode v25 ids, identical to the names.}
\item{bp_length }{ the sum of the width of the disjoint exons for that
given gene.}
\item{symbol }{ a CharacterList with the corresponding gene symbols.}
}
}
\description{
Gene annotation extracted from Gencode v25 (GRCh38.p7) used in recount.
Data extraced on October 12th, 2017. The symbols were updated compared
to the version from January 17th, 2017.
It includes the sum of the width of the disjoint exons which can be
used for normalizing the counts provided in the
\link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class} objects.
}
\references{
\url{https://jhubiostatistics.shinyapps.io/recount/}
}
\seealso{
\link{reproduce_ranges}, \link{recount_exons}
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geo_characteristics.R
\name{geo_characteristics}
\alias{geo_characteristics}
\title{Build a data.frame from GEO's charactersitics for a given sample}
\usage{
geo_characteristics(pheno)
}
\arguments{
\item{pheno}{A \link[S4Vectors:DataFrame-class]{DataFrame-class} as created by
\link{geo_info}.}
}
\value{
A 1 row data.frame with the characteristic fields as column names
and the values as the entries on the first row. If the authors of the study
used the same names for all samples, you can then combine them using
\link[base:cbind]{rbind}.
}
\description{
This function builds a data.frame from the GEO characteristics extracted
for a given sample. The names of the of columns correspond to the field
names. For a given SRA project, this information can be combined for all
samples as shown in the examples section.
}
\examples{

## Load required library
library("SummarizedExperiment")

## Get the GEO accession ids
# geoids <- sapply(colData(rse_gene_SRP009615)$run[1:2], find_geo)
## The previous example code works nearly all the time but it
## can occassionally fail depending on how rentrez is doing.
## This code makes sure that the example code runs.
geoids <- tryCatch(
    sapply(colData(rse_gene_SRP009615)$run[1:2], find_geo),
    error = function(e) {
        c(
            "SRR387777" = "GSM836270",
            "SRR387778" = "GSM836271"
        )
    }
)

## Get the data from GEO
geodata <- do.call(rbind, sapply(geoids, geo_info))

## Add characteristics in a way that we can access easily later on
geodata <- cbind(geodata, geo_characteristics(geodata))

## Explore the original characteristics and the result from
## geo_characteristics()
geodata[, c("characteristics", "cells", "shrna.expression", "treatment")]
}
\author{
Leonardo Collado-Torres
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getTPM.R
\name{getTPM}
\alias{getTPM}
\title{Compute a TPM matrix based on a RangedSummarizedExperiment object}
\usage{
getTPM(rse, length_var = "bp_length", mapped_var = NULL)
}
\arguments{
\item{rse}{A \link[SummarizedExperiment:RangedSummarizedExperiment-class]{RangedSummarizedExperiment-class}
object as downloaded with \link{download_study}.}

\item{length_var}{A length 1 character vector with the column name from
\code{rowData(rse)} that has the coding length. For gene level objects
from recount this is \code{bp_length}. If \code{NULL}, then it will use
\code{width(rowRanges(rse))} which should be used for exon RSEs.}

\item{mapped_var}{A length 1 character vector with the column name from
\code{colData(rse)} that has the number of reads mapped. For recount RSE
object this would be \code{mapped_read_count}. If \code{NULL} (default)
then it will use the column sums of the counts matrix. The results are
different because not all mapped reads are mapped to exonic segments of the
genome.}
}
\value{
A matrix with the TPM values.
}
\description{
For some analyses you might be interested in transforming the counts into
TPMs which you can do with this function. This function uses the gene-level
RPKMs to derive TPM values (see Details).
}
\details{
For gene RSE objects, you will want to specify the \code{length_var}
because otherwise you will be adjusting for the total gene length instead
of the total exonic sequence length of the gene.

As noted in https://support.bioconductor.org/p/124265/,
Sonali Arora et al computed TPMs in
https://www.biorxiv.org/content/10.1101/445601v2 using the formula:
TPM = FPKM / (sum of FPKM over all genes/transcripts) * 10^6

Arora et al mention in their code that the formula comes from
https://doi.org/10.1093/bioinformatics/btp692; specifically
\verb{1.1.1 Comparison to RPKM estimation} where they mention an important
assumption: Under the assumption of uniformly distributed reads, we note
that RPKM measures are estimates of ...

There's also a blog post by Harold Pimentel explaining the relationship
between FPKM and TPM:
https://haroldpimentel.wordpress.com/2014/05/08/what-the-fpkm-a-review-rna-seq-expression-units/.
}
\examples{

## Compute the TPM matrix from the raw gene-level base-pair counts.
tpm <- getTPM(rse_gene_SRP009615)
}
\references{
https://www.biorxiv.org/content/10.1101/445601v2
https://arxiv.org/abs/1104.3889
}
\seealso{
\link{getRPKM}
}
\author{
Sonali Arora, Leonardo Collado-Torres
}
